SystemOrganization addCategory: #'CollectionsTests-Abstract'!
SystemOrganization addCategory: #'CollectionsTests-Arrayed'!
SystemOrganization addCategory: #'CollectionsTests-Sequenceable'!
SystemOrganization addCategory: #'CollectionsTests-Stack'!
SystemOrganization addCategory: #'CollectionsTests-Streams'!
SystemOrganization addCategory: #'CollectionsTests-Support'!
SystemOrganization addCategory: #'CollectionsTests-Text'!
SystemOrganization addCategory: #'CollectionsTests-Unordered'!
SystemOrganization addCategory: #'CollectionsTests-Weak'!

ClassTestCase subclass: #ArrayTest
	instanceVariableNames: 'example1 literalArray selfEvaluatingArray otherArray nonSEArray1 nonSEarray2 example2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Arrayed'!

!ArrayTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Array. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!ArrayTest methodsFor: 'initialize-release' stamp: 'ar 3/5/2010 20:25'!
setUp

	literalArray := #(1 true 3 #four).
	selfEvaluatingArray := { 1. true. (3/4). Color black. (2 to: 4) . 5 }.
	nonSEArray1 := { 1 . Set with: 1 }.
	nonSEarray2 := { Smalltalk globals associationAt: #Array }.
	example1 := #(1 2 3 4 5).
	example2 := {1. 2. 3/4. 4. 5}. ! !

!ArrayTest methodsFor: 'tests - accessing' stamp: 'dc 3/3/2007 17:40'!
testAtWrap
	|tabTest|
	tabTest := #(5 6 8).
	self assert:(tabTest atWrap: 2) = 6.
	self assert:(tabTest atWrap:7) = 5.
	self assert:( tabTest atWrap:5) = 6.
	self assert:(tabTest atWrap:0)= 8.
	self assert:(tabTest atWrap:1)= 5.
	self assert:(tabTest atWrap:-2) = 5.

! !

!ArrayTest methodsFor: 'testing' stamp: 'zz 12/5/2005 17:12'!
testIsArray
	
	self assert: example1 isArray! !

!ArrayTest methodsFor: 'testing' stamp: 'nice 11/3/2009 21:28'!
testIsLiteral
	"We work with a copy of literalArray, to avoid corrupting the code."
	
	| aLiteralArray |
	aLiteralArray := literalArray copy.
	self assert: aLiteralArray isLiteral.
	aLiteralArray at: 1 put: self class.
	self deny: aLiteralArray isLiteral.
	self deny: (literalArray as: WeakArray) isLiteral description: 'instances of Array subclasses are not literal'.! !

!ArrayTest methodsFor: 'testing' stamp: 'zz 12/5/2005 17:50'!
testLiteralEqual
	self
		deny: (example1 literalEqual: example1 asIntegerArray)! !

!ArrayTest methodsFor: 'testing' stamp: 'nice 11/1/2009 19:14'!
testLiteralStoreOn
	"Test that a literal Array is stored and evaluated back unchanged"
	
	| anArray |
	anArray := {true. false. nil. #a. 'a'. $a. -1. 0. 1. Float pi. Float halfPi. Float halfPi negated.}.
	anArray := anArray copyWith: anArray.
	self assert: anArray isLiteral.
	self assert: (Compiler evaluate: anArray storeString) = anArray.! !

!ArrayTest methodsFor: 'testing' stamp: 'dc 5/24/2007 10:56'!
testNewWithSize
	|array|
	array := Array new: 5.
	self assert: array size = 5.
	1 to: 5 do: [:index | self assert: (array at: index) isNil]! !

!ArrayTest methodsFor: 'testing' stamp: 'zz 12/5/2005 17:50'!
testPremultiply
	self assert: example1 +* #(2 ) = #(2 4 6 8 10 ) ! !

!ArrayTest methodsFor: 'testing' stamp: 'ul 11/11/2009 16:27'!
testPrinting
	self assert: literalArray printString = '#(1 true 3 #four)'.
	self assert: (literalArray = (Compiler evaluate: literalArray printString)).
	self assert: selfEvaluatingArray printString =  '{1 . true . (3/4) . Color black . (2 to: 4) . 5}'.
	self assert: (selfEvaluatingArray = (Compiler evaluate: selfEvaluatingArray printString)).
	self assert: nonSEArray1 printString =  '{1 . a Set(1)}'.
	self assert: nonSEarray2 printString =  '{#Array}'
! !

ClassTestCase subclass: #AssociationTest
	instanceVariableNames: 'a b'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Support'!

!AssociationTest methodsFor: 'setup' stamp: 'zz 12/5/2005 18:33'!
setUp

	a := 1 -> 'one'.
	b := 1 -> 'een'.! !

!AssociationTest methodsFor: 'tests' stamp: 'md 3/8/2004 16:37'!
testEquality
	
	self 
		assert: (a key = b key);
		deny: (a value = b value);
		deny: (a = b)

! !

!AssociationTest methodsFor: 'tests' stamp: 'nice 2/13/2010 05:25'!
testHash

	self 
		assert: (a hash = a copy hash)! !

ClassTestCase subclass: #ByteArrayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Arrayed'!

!ByteArrayTest methodsFor: 'hex tests' stamp: 'ar 2/23/2010 07:16'!
testHex
	self assert: #[122 43 213 7] hex = '7a2bd507'.
	self assert: (UUID fromString: '97c1f2dd-f920-9948-b329-319a30c16386') hex
					= '97c1f2ddf9209948b329319a30c16386'.! !

!ByteArrayTest methodsFor: 'hex tests' stamp: 'nice 6/5/2010 10:06'!
testIsLiteral
	self assert: #[122 43 213 7] isLiteral description: 'ByteArray instances are literal'.
	self deny: thisContext method isLiteral description: 'ByteArray sub instances are not literal'! !

!ByteArrayTest methodsFor: 'hex tests' stamp: 'ar 2/23/2010 07:15'!
testReadHexFrom
	self assert: (ByteArray readHexFrom: '7a2bd507') = #[122 43 213 7].
	self assert: (UUID readHexFrom: '97c1f2ddf9209948b329319a30c16386')
					= (UUID fromString: '97c1f2dd-f920-9948-b329-319a30c16386').! !

ClassTestCase subclass: #CharacterSetComplementTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Support'!

!CharacterSetComplementTest commentStamp: '<historical>' prior: 0!
CharacterSetComplementTest hold unit tests for CharacterSetComplement!

!CharacterSetComplementTest methodsFor: 'testing' stamp: 'nice 9/2/2008 10:29'!
testPrintString
	"This is about CharacterSetComplementTest"
	
	self shouldnt: [CharacterSet separators complement printString] raise: Error! !

ClassTestCase subclass: #CharacterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!CharacterTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Character. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!CharacterTest methodsFor: 'tests - various' stamp: 'ar 10/25/2010 18:59'!
testCharacterSeparators
	"Regression test"

	| result |
	self shouldnt:[result := '/', Character separators] raise: Error.
	self assert: result size = (Character separators size + 1).

	self shouldnt:[result := '/', CharacterSet separators] raise: Error.
	self assert: result size = (CharacterSet separators asArray size + 1).
! !

!CharacterTest methodsFor: 'tests - creation' stamp: 'ar 9/1/2009 00:41'!
testCodePoint
	self assert: (Character codePoint: $A asciiValue) = $A.
	self shouldnt:[Character codePoint: 500] raise: Error.
	self assert: (Character codePoint: 500) asciiValue = 500.! !

!CharacterTest methodsFor: 'testing - Class Methods' stamp: 'sd 6/5/2005 09:25'!
testNew

	self should: [Character new] raise: Error.! !

!CharacterTest methodsFor: 'testing-printing' stamp: 'lr 11/21/2005 17:41'!
testPrintString
	self assert: $a printString = '$a'.
	self assert: $5 printString = '$5'.
	self assert: $@ printString = '$@'.

	self assert: Character cr printString = 'Character cr'.
	self assert: Character lf printString = 'Character lf'.
	self assert: Character space printString = 'Character space'.

	self assert: (Character value: 0) printString = 'Character value: 0'.
	self assert: (Character value: 17) printString = 'Character value: 17'.! !

!CharacterTest methodsFor: 'testing-printing' stamp: 'nice 2/23/2011 22:01'!
testPrintStringAll
	Character allCharacters do: [ :each |
		self assert: (self class newCompiler 
			evaluate: each printString) = each ].! !

!CharacterTest methodsFor: 'testing-printing' stamp: 'lr 11/21/2005 17:22'!
testStoreString
	self assert: $a storeString = '$a'.
	self assert: $5 storeString = '$5'.
	self assert: $@ storeString = '$@'.

	self assert: Character cr storeString = 'Character cr'.
	self assert: Character lf storeString = 'Character lf'.
	self assert: Character space storeString = 'Character space'.

	self assert: (Character value: 0) storeString = '(Character value: 0)'.
	self assert: (Character value: 17) storeString = '(Character value: 17)'.! !

!CharacterTest methodsFor: 'testing-printing' stamp: 'nice 2/23/2011 22:01'!
testStoreStringAll
	Character allCharacters do: [ :each |
		self assert: (self class newCompiler 
			evaluate: each storeString) = each ].! !

ClassTestCase subclass: #FloatArrayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Arrayed'!

!FloatArrayTest commentStamp: 'nice 5/30/2006 01:24' prior: 0!
These tests are used to assert that FloatArrayPlugin has same results as Float asIEEE32BitWord!

!FloatArrayTest methodsFor: 'testing' stamp: 'nice 11/23/2007 23:53'!
testArithmeticCoercion
	"This test is related to http://bugs.squeak.org/view.php?id=6782"
	
	self should: [3.0 / (FloatArray with: 2.0) = (FloatArray with: 1.5)].
	self should: [3.0 * (FloatArray with: 2.0) = (FloatArray with: 6.0)].
	self should: [3.0 + (FloatArray with: 2.0) = (FloatArray with: 5.0)].
	self should: [3.0 - (FloatArray with: 2.0) = (FloatArray with: 1.0)].! !

!FloatArrayTest methodsFor: 'testing' stamp: 'nice 5/30/2006 03:17'!
testFloatArrayPluginPrimitiveAt
	"if FloatArrayPlugin primitive are not here, this test is dumb.
	Otherwise, it will compare primitive and #fromIEEE32Bit:"
	
	#(
		"regular numbers no truncation or rounding"
		2r0.0 2r1.0 2r1.1 2r1.00000000000000000000001
		2r1.0e-10 2r1.1e-10 2r1.00000000000000000000001e-10
		2r1.0e10 2r1.1e10 2r1.00000000000000000000001e10
		
		"smallest float32 before gradual underflow"
		2r1.0e-126
		
		"biggest float32"
		2r1.11111111111111111111111e127
		
		"overflow"
		2r1.11111111111111111111111e128
		
		"gradual underflow"
		2r0.11111111111111111111111e-126
		2r0.00000000000000000000001e-126
		
		"with rounding mode : tests on 25 bits"
		
		2r1.0000000000000000000000001
		2r1.0000000000000000000000010
		2r1.0000000000000000000000011
		2r1.0000000000000000000000100
		2r1.0000000000000000000000101
		2r1.0000000000000000000000110
		2r1.0000000000000000000000111
		2r1.1111111111111111111111001
		2r1.1111111111111111111111010
		2r1.1111111111111111111111011
		2r1.1111111111111111111111101
		2r1.1111111111111111111111110
		2r1.1111111111111111111111111
		
		"overflow"
		2r1.1111111111111111111111110e127
		
		"gradual underflow"
		2r0.1111111111111111111111111e-126
		2r0.1111111111111111111111110e-126
		2r0.1111111111111111111111101e-126
		2r0.1111111111111111111111011e-126
		2r0.1111111111111111111111010e-126
		2r0.1111111111111111111111001e-126
		2r0.0000000000000000000000111e-126
		2r0.0000000000000000000000110e-126
		2r0.0000000000000000000000101e-126
		2r0.0000000000000000000000011e-126
		2r0.0000000000000000000000010e-126
		2r0.0000000000000000000000001e-126
		2r0.0000000000000000000000010000000000000000000000000001e-126
		) do: [:e |
			self assert: ((FloatArray with: e) at: 1) = (Float fromIEEE32Bit: ((FloatArray with: e) basicAt: 1)).
			self assert: ((FloatArray with: e negated) at: 1) = (Float fromIEEE32Bit: ((FloatArray with: e negated) basicAt: 1))].
		
	"special cases"
	(Array with: Float infinity with: Float infinity negated with: Float negativeZero)
		do: [:e | self assert: ((FloatArray with: e) at: 1) = (Float fromIEEE32Bit: ((FloatArray with: e) basicAt: 1))].
		
	"Cannot compare NaN"
	(Array with: Float nan)
		do: [:e | self assert: (Float fromIEEE32Bit: ((FloatArray with: e) basicAt: 1)) isNaN].! !

!FloatArrayTest methodsFor: 'testing' stamp: 'nice 5/30/2006 03:17'!
testFloatArrayPluginPrimitiveAtPut
	"if FloatArrayPlugin primitive are not here, this test is dumb.
	Otherwise, it will compare primitive and #asIEEE32BitWord"
	
	#(
		"regular numbers no truncation or rounding"
		2r0.0 2r1.0 2r1.1 2r1.00000000000000000000001
		2r1.0e-10 2r1.1e-10 2r1.00000000000000000000001e-10
		2r1.0e10 2r1.1e10 2r1.00000000000000000000001e10
		
		"smallest float32 before gradual underflow"
		2r1.0e-126
		
		"biggest float32"
		2r1.11111111111111111111111e127
		
		"overflow"
		2r1.11111111111111111111111e128
		
		"gradual underflow"
		2r0.11111111111111111111111e-126
		2r0.00000000000000000000001e-126
		
		"with rounding mode : tests on 25 bits"
		
		2r1.0000000000000000000000001
		2r1.0000000000000000000000010
		2r1.0000000000000000000000011
		2r1.0000000000000000000000100
		2r1.0000000000000000000000101
		2r1.0000000000000000000000110
		2r1.0000000000000000000000111
		2r1.1111111111111111111111001
		2r1.1111111111111111111111010
		2r1.1111111111111111111111011
		2r1.1111111111111111111111101
		2r1.1111111111111111111111110
		2r1.1111111111111111111111111
		
		"overflow"
		2r1.1111111111111111111111110e127
		
		"gradual underflow"
		2r0.1111111111111111111111111e-126
		2r0.1111111111111111111111110e-126
		2r0.1111111111111111111111101e-126
		2r0.1111111111111111111111011e-126
		2r0.1111111111111111111111010e-126
		2r0.1111111111111111111111001e-126
		2r0.0000000000000000000000111e-126
		2r0.0000000000000000000000110e-126
		2r0.0000000000000000000000101e-126
		2r0.0000000000000000000000011e-126
		2r0.0000000000000000000000010e-126
		2r0.0000000000000000000000001e-126
		2r0.0000000000000000000000010000000000000000000000000001e-126
		) do: [:e |
			self assert: ((FloatArray with: e) basicAt: 1) = e asIEEE32BitWord.
			self assert: ((FloatArray with: e negated) basicAt: 1) = e negated asIEEE32BitWord].
		
	"special cases"
	(Array with: Float infinity with: Float infinity negated with: Float negativeZero with: Float nan)
		do: [:e | self assert: ((FloatArray with: e) basicAt: 1) = e asIEEE32BitWord].
		! !

ClassTestCase subclass: #HeapTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

!HeapTest methodsFor: 'examples' stamp: 'md 2/12/2006 15:33'!
heapExample	"HeapTest new heapExample"
	"Create a sorted collection of numbers, remove the elements
	sequentially and add new objects randomly.
	Note: This is the kind of benchmark a heap is designed for."
	| n rnd array time sorted |
	n := 5000. "# of elements to sort"
	rnd := Random new.
	array := (1 to: n) collect:[:i| rnd next].
	"First, the heap version"
	time := Time millisecondsToRun:[
		sorted := Heap withAll: array.
		1 to: n do:[:i| 
			sorted removeFirst.
			sorted add: rnd next].
	].
	Transcript cr; show:'Time for Heap: ', time printString,' msecs'.
	"The quicksort version"
	time := Time millisecondsToRun:[
		sorted := SortedCollection withAll: array.
		1 to: n do:[:i| 
			sorted removeFirst.
			sorted add: rnd next].
	].
	Transcript cr; show:'Time for SortedCollection: ', time printString,' msecs'.
! !

!HeapTest methodsFor: 'examples' stamp: 'md 2/12/2006 15:34'!
heapSortExample	"HeapTest new heapSortExample"
	"Sort a random collection of Floats and compare the results with
	SortedCollection (using the quick-sort algorithm) and 
	ArrayedCollection>>mergeSortFrom:to:by: (using the merge-sort algorithm)."
	| n rnd array  time sorted |
	n := 10000. "# of elements to sort"
	rnd := Random new.
	array := (1 to: n) collect:[:i| rnd next].
	"First, the heap version"
	time := Time millisecondsToRun:[
		sorted := Heap withAll: array.
		1 to: n do:[:i| sorted removeFirst].
	].
	Transcript cr; show:'Time for heap-sort: ', time printString,' msecs'.
	"The quicksort version"
	time := Time millisecondsToRun:[
		sorted := SortedCollection withAll: array.
	].
	Transcript cr; show:'Time for quick-sort: ', time printString,' msecs'.
	"The merge-sort version"
	time := Time millisecondsToRun:[
		array mergeSortFrom: 1 to: array size by: [:v1 :v2| v1 <= v2].
	].
	Transcript cr; show:'Time for merge-sort: ', time printString,' msecs'.
! !

!HeapTest methodsFor: 'testing' stamp: 'nice 12/27/2009 03:11'!
test1
	| data |

	"The first element of each array is the sort value, and the second will be updated by the heap with the index of the element within the heap."
	data :=  (1 to: 8) collect: [:i | {i*2. 0}].

	"Repeat with different data ordering."
	5 timesRepeat: [ | h |
		h := Heap new sortBlock: [:e1 :e2 | e1 first < e2 first].
		h indexUpdateBlock: [:array :index | array at: 2 put: index].

		data shuffled do: [:d | h add: d].
		data do: [:d | self should: (h at: d second) == d].
	]! !

!HeapTest methodsFor: 'basic tests' stamp: 'nm 2/28/2006 09:37'!
testAdd
	"self run: #testAdd"

	| heap |
	heap := Heap new.
	self assert: heap size = 0.
	heap add: 3.
	self assert: heap size = 1.
	self assert: heap isEmpty not.
	self assert: heap first = 3.
	self assert: (heap at: 1) = 3.
	heap add: 2.
	self assert: heap size = 2.
	self assert: heap first = 2.
	self assert: (heap at: 2) = 3.
	! !

!HeapTest methodsFor: 'basic tests' stamp: 'sd 4/8/2006 21:37'!
testAt
	"self run: #testAt"

	| heap |
	heap := Heap new.
	heap add: 3.
	self assert: (heap at: 1) = 3.
	self should: [heap at: 2] raise: Error.
	heap add: 4.
	self assert: (heap at: 1) = 3.
	self assert: (heap at: 2) = 4.

	! !

!HeapTest methodsFor: 'basic tests' stamp: 'nm 4/3/2006 11:22'!
testDo
	"self run: #testDo"

	| heap coll |
	heap := Heap withAll: #(1 3 5).
	coll := OrderedCollection new.
	
	heap do: [:each | coll add: each].
	
	self assert: coll = #(1 3 5) asOrderedCollection.
! !

!HeapTest methodsFor: 'testing' stamp: 'md 2/12/2006 15:35'!
testExamples
	self shouldnt: [self heapExample] raise: Error.
	self shouldnt: [self heapSortExample] raise: Error.! !

!HeapTest methodsFor: 'basic tests' stamp: 'nm 2/28/2006 09:36'!
testFirst
	"self run: #testFirst"
	| heap |
	heap := Heap new.
	heap add: 5.
	heap add: 12.
	heap add: 1.
	self assert: heap first = 1.
	heap removeFirst.
	self assert: heap first = 5.! !

!HeapTest methodsFor: 'basic tests' stamp: 'nm 4/3/2006 11:27'!
testHeap
	"self run: #testHeap"

	| heap |
	heap := Heap new.
	self assert: heap isHeap.
	
	self assert: heap isEmpty.
	heap add: 1.
	self assert: heap isEmpty not
	
! !

!HeapTest methodsFor: 'testing' stamp: 'nice 8/21/2010 16:33'!
testIfEqualIsTransitive
	"This is http://bugs.squeak.org/view.php?id=6943"

    | anArray heap1 heap2 |
    anArray := #(1 2 3).
    heap1 := Heap withAll: (1 to: 3) sortBlock: [:a :b | a < b].
    heap2 := Heap withAll: (1 to: 3) sortBlock: [:a :b | b > a].
    self
		assert: (heap1 = anArray) & (heap2 = anArray) ==> (heap1 = heap2)
		description: 'Heap equality should be transitive'! !

!HeapTest methodsFor: 'basic tests' stamp: 'nm 2/15/2006 09:55'!
testRemove
	"self run: #testRemove"
	
	| heap |
	heap := Heap new.
	self should: [heap removeFirst] raise: Error.
	heap add: 5.
	self shouldnt: [heap removeFirst] raise: Error.
	self assert: heap size = 0.
	heap add: 5.
	self should: [heap removeAt: 2] raise: Error.! !

!HeapTest methodsFor: 'basic tests' stamp: 'nice 8/21/2010 16:11'!
testSortBlock
	"self run: #testSortBlock"

	| heap |
	heap := Heap withAll: #(1 3 5).
	self assert: heap asArray = #(1 3 5).
	
	heap sortBlock: [ :e1 :e2 | e1 >= e2 ].
	self assert: heap asArray = #(5 3 1)
! !

ClassTestCase subclass: #IntegerArrayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Arrayed'!

!IntegerArrayTest commentStamp: 'dtl 10/3/2009 16:37' prior: 0!
Verify storage of integer values in an IntegerArray. An IntegerArray stores 32-bit signed integer values. Lack of range check in primitive fallback code leads to arithmetic overflow errors, so attempts to store out of range values should raise errors.!

!IntegerArrayTest methodsFor: 'tests' stamp: 'dtl 10/3/2009 16:36'!
testStoreLargeNegativeInteger
	"A LargeNegativeInteger that does not exceed the range of a 32-bit twos
	complement integer may be stored."

	| ia val |
	ia := IntegerArray new: 1.
	val := (2 raisedTo: 31) negated. "smallest negative 32-bit integer, a LargeNegativeInteger"
	ia at: 1 put: val.
	self assert: ((ia at: 1) = val)
! !

!IntegerArrayTest methodsFor: 'tests' stamp: 'dtl 10/3/2009 16:34'!
testStoreLargePositiveInteger
	"A LargePositiveInteger that does not exceed the range of a 32-bit twos
	complement integer may be stored."

	| ia val |
	ia := IntegerArray new: 1.
	val := (2 raisedTo: 31) - 1. "largest positive 32-bit integer, a LargePositiveInteger"
	ia at: 1 put: val.
	self assert: ((ia at: 1) = val)
! !

!IntegerArrayTest methodsFor: 'tests' stamp: 'nice 12/27/2009 20:52'!
testStoreNegativeOutOfRange
	"A value too small for a 32-bit signed integer should raise an error."

	| ia val |
	ia := IntegerArray new: 1.
	val := (2 raisedTo: 31) negated - 1. " -16r80000001 of range for 32-bit twos complement integer"
	self should: [ | storedValue |
				ia at: 1 put: val. "should raise error here"
				storedValue := ia at: 1. "lack of range check leads to value of 16r7FFFFFFF"
				self assert: val = storedValue "fail here"]
		raise: Error
! !

!IntegerArrayTest methodsFor: 'tests' stamp: 'nice 12/27/2009 20:52'!
testStorePositiveOutOfRange
	"A value too large for a 32-bit signed integer should raise an error."

	| ia val |
	ia := IntegerArray new: 1.
	val := 2 raisedTo: 31. "16r80000000 is out of range for 32-bit twos complement"
	self should: [ | storedValue |
				ia at: 1 put: val. "should raise error here"
				storedValue := ia at: 1. "lack of range check leads to value of 16r-80000000"
				self assert: val = storedValue "fail here"]
		raise: Error
! !

!IntegerArrayTest methodsFor: 'tests' stamp: 'dtl 10/3/2009 16:31'!
testStoreSmallInteger
	"Any SmallInteger may be stored in an IntegerArray."

	| ia val |
	ia := IntegerArray new: 1.
	val := SmallInteger maxVal.
	ia at: 1 put: val.
	self assert: ((ia at: 1) = val).

	val := SmallInteger minVal.
	ia at: 1 put: val.
	self assert: ((ia at: 1) = val)
! !

ClassTestCase subclass: #IntervalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:23'!
testAdd
	self assert: (1 to: 10)
			+ 5
			= (6 to: 15)! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 2/9/2010 20:44'!
testAnyOne
	self assert: ((2 to: 5) includes: (2 to: 5) anyOne).
	self should: [(2 to: 1) anyOne] raise: Error description: 'This interval is empty'! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 1/18/2010 18:07'!
testAsInterval
	"This is the same as newFrom:"

	self shouldnt: [
		self assert: (#(1 2 3) as: Interval) = (1 to: 3).
		self assert: (#(33 5 -23) as: Interval) = (33 to: -23 by: -28).
		self assert: (#[2 4 6] as: Interval) = (2 to: 6 by: 2).
	] raise: Error.

	self should: [#(33 5 -22) as: Interval]
		raise: Error
		description: 'This is not an arithmetic progression'
! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:15'!
testAt
	self assert: ((1 to: 10)
			at: 1)
			= 1.
	self assert: ((1 to: 10)
			at: 3)
			= 3.
	self assert: ((1 to: 10 by: 2)
			at: 1)
			= 1.
	self assert: ((1 to: 10 by: 2)
			at: 3)
			= 5! !

!IntervalTest methodsFor: 'tests' stamp: 'apb 4/22/2007 12:34'!
testDo
	| s i |
	s := OrderedCollection new.
	i := (10 to: 20).
	i do: [ :each | s addLast: each].
	self assert: (s hasEqualElements: i)! !

!IntervalTest methodsFor: 'tests' stamp: 'md 10/12/2003 20:13'!
testEquals

	self shouldnt: [
		self assert: (3 to: 5) = #(3 4 5).
		self deny: (3 to: 5) = #(3 5).
		self deny: (3 to: 5) = #().

		self assert: #(3 4 5) = (3 to: 5).
		self deny: #(3 5) = (3 to: 5).
		self deny: #() = (3 to: 5).
	] raise: MessageNotUnderstood.! !

!IntervalTest methodsFor: 'tests' stamp: 'md 10/12/2003 20:13'!
testEquals2

	self assert: (3 to: 5) = #(3 4 5).
	self deny: (3 to: 5) = #(3 5).
	self deny: (3 to: 5) = #().

	self assert: #(3 4 5) = (3 to: 5).
	self deny: #(3 5) = (3 to: 5).
	self deny: #() = (3 to: 5).! !

!IntervalTest methodsFor: 'tests' stamp: 'md 10/12/2003 20:13'!
testEquals3

	self assert: (3 to: 5 by: 2) first = (3 to: 6 by: 2) first.
	self assert: (3 to: 5 by: 2) last = (3 to: 6 by: 2) last.
	self assert: (3 to: 5 by: 2) = (3 to: 6 by: 2).! !

!IntervalTest methodsFor: 'tests' stamp: 'md 10/12/2003 20:13'!
testEquals4

	self assert: (3 to: 5 by: 2) = #(3 5).
	self deny: (3 to: 5 by: 2) = #(3 4 5).
	self deny: (3 to: 5 by: 2) = #().

	self assert: #(3 5) = (3 to: 5 by: 2).
	self deny: #(3 4 5) = (3 to: 5 by: 2).
	self deny: #() = (3 to: 5 by: 2).! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:08'!
testExtent
	self assert: (1 to: 10) extent = 9.
	self assert: (1 to: 10 by: 2) extent = 9.
	self assert: (1 to:-1) extent = -2! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 12/22/2009 16:06'!
testInclusion
	"Non regression test for another bug of fuzzy inclusion"
	
	self deny: ((1.0 to: 3.0 by: 1.0 successor) includes: 3.0) description: 'The last element of this Interval is closed to 2'! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 4/29/2007 21:34'!
testInclusionBug1603
	"This test is by german morales.
	It is about mantis bug 1603"
	
	self shouldnt: ((1 to: 5 by: 1) includes: 2.5). "obvious"
	self shouldnt: ((100000000000000 to: 500000000000000 by: 100000000000000)
 		  includes: 250000000000000). "same as above with 14 zeros appended"! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 2/3/2008 21:43'!
testInclusionBug6455
	"This test is about mantis bug http://bugs.squeak.org/view.php?id=6455
	It should work as long as Fuzzy inclusion test feature for Interval of Float is maintained.
	This is a case when tested element is near ones of actual value, but by default.
	Code used to work only in the case of close numbers by excess..."
	
	self assert: ((0 to: Float pi by: Float pi / 100) includes: Float pi * (3/100))! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 4/29/2007 21:35'!
testIndexOfBug1602
	"This test is by german morales.
	It is about mantis bug 1602"
	
	self should: ((1 to: 5 by: 1) indexOf: 2.5) = 0. "obvious"
	self should: ((100000000000000 to: 500000000000000 by: 100000000000000)
 		  indexOf: 250000000000000) = 0. "same as above with 14 zeros appended"! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 2/3/2008 21:35'!
testIndexOfBug6455
	"This test is about mantis bug http://bugs.squeak.org/view.php?id=6455
	It should work as long as Fuzzy inclusion test feature for Interval of Float is maintained.
	This is a case when tested element is near ones of actual value, but by default.
	Code used to work only in the case of close numbers by excess..."
	
	self assert: ((0 to: Float pi by: Float pi / 100) indexOf: Float pi * (3/100)) = 4! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 12/22/2009 16:19'!
testInfiniteLoopBug6456
	"This is a non regression test against mantis bug #6456.
	Some Float interval size was not consistent with do: loop.
	Some Float Interval used to do: infinite loops"
	
	| x interval counter size |
	x := (1.0 timesTwoPower: Float precision). "Note: x+1 = x due to inexact arithmetic"
	interval := x to: x+4.
	size := interval size.
	counter := 0.
	interval do: [:each | self assert: (counter := counter + 1) <= size].! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 5/12/2007 00:53'!
testIntervalStoreOn
	"this is a test for http://bugs.squeak.org/view.php?id=4378"
	
	| interval1 interval2 |
	interval1 := 0 to: 1 by: 0.5s1 squared.
	interval2 := Compiler evaluate: interval1 storeString.
	self assert: interval1 size = interval2 size! !

!IntervalTest methodsFor: 'tests' stamp: ''!
testInvalid
	"empty, impossible ranges"
	self assert: (1 to: 0) = #().
	self assert: (1 to: -1) = #().
	self assert: (-1 to: -2) = #().
	self assert: (1 to: 5 by: -1) = #().
	
	"always contains only start value."
	self assert: (1 to: 1) = #(1).
	self assert: (1 to: 5 by: 10) = #(1).
	self assert: (1 to: 0 by: -2) = #(1).
! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/7/2005 13:29'!
testIsInterval
	self assert: (1 to: 10) isInterval.
	self assert: (1 to: 10 by: 2) isInterval! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/7/2005 13:29'!
testLast

self assert: (1 to:10) last = 10.
self assert: (1 to:10 by:2) last = 9 ! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:28'!
testMinus
	self assert: (1 to: 10)
			- 5
			= (-4 to: 5)! !

!IntervalTest methodsFor: 'tests' stamp: 'md 1/14/2004 11:43'!
testNewFrom

	self shouldnt: [
		 self assert: ( (Interval newFrom: (1 to: 1)) = (1 to: 1)).
		 self assert: ( (Interval newFrom: #(1)) = (1 to: 1)).
		 self assert: ( (Interval newFrom: #()) =  ( 1 to: 0)) .
	] raise: Error.! !

!IntervalTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:17'!
testNumericTypes

	(3 asNumber) to: 5 = #(3 4 5).
	
	3.0 to: 5.0 = #(3.0 4.0 5.0).
	3.0 to: 5.0 by: 0.5 = #(3.0 3.5 4.0 4.5 5.0).
	
	3/1 to: 5/1 = #(3 4 5).
	1/2 to: 5/2 by: 1/2 = #(1/2 1 3/2 2 5/2).! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:03'!
testOtherNewFrom

	self assert: (Interval newFrom: #(1 2 3 )) = (1 to: 3).
	self assert: (Interval newFrom: #(33  5 -23 )) = (33 to: -23 by: -28).
	self should: [(Interval newFrom: #(33  5 -22 ))] raise: Error.
	self assert: (#(33  5 -23) as: Interval) = (33 to: -23 by: -28).
	self should: [( #(33  5 -22 ) as: Interval)] raise: Error.
	
	self assert: ( (-4 to: -12 by: -1) as: Interval) = (-4 to: -12 by: -1).
	self assert: ( Interval newFrom: (1 to: 1)) = (1 to: 1).
	self assert: ( Interval newFrom: (1 to: 0)) = (1 to: 0).
	self assert: (#(1) as: Interval) = (1 to: 1).
	self assert: (#() as: Interval) = (1 to: 0).! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 11/15/2010 21:02'!
testPermutationsDo

	| i oc |
	i := (1.234 to: 4.234).
	oc := OrderedCollection new.
	i permutationsDo: [:e | oc add: e].
	self assert: (oc size = i size factorial).
	^ oc! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:20'!
testRangeIncludes
	self
		assert: ((1 to: 10)
				rangeIncludes: 3).
	self
		assert: ((1 to: 10 by: 2)
				rangeIncludes: 3).
	self
		deny: ((1 to: 10)
				rangeIncludes: 0).
	self
		deny: ((1 to: 10)
				rangeIncludes: 11).
	self
		deny: ((1 to: 10 by: 2)
				rangeIncludes: 0).
	self
		deny: ((1 to: 10 by: 2)
				rangeIncludes: 11)! !

!IntervalTest methodsFor: 'tests' stamp: 'apb 4/22/2007 12:35'!
testReverseDo
	| s i |
	s := OrderedCollection new.
	i := 10 to: 20.
	i
		reverseDo: [:each | s addFirst: each].
	self
		assert: (s hasEqualElements: i)! !

!IntervalTest methodsFor: 'tests' stamp: 'apb 4/22/2007 12:36'!
testReverseUnevenDo
	| s i |
	s := OrderedCollection new.
	i := 10 to: 20 by: 3.
	i
		reverseDo: [:each | s addFirst: each].
	self
		assert: (s hasEqualElements: i)! !

!IntervalTest methodsFor: 'tests' stamp: 'apb 4/22/2007 12:39'!
testUnevenDo
	| s i |
	s := OrderedCollection new.
	i := 10 to: 20 by: 3.
	i
		do: [:each | s addLast: each].
	self
		assert: (s hasEqualElements: i)! !

ClassTestCase subclass: #KeyedSetTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

!KeyedSetTest commentStamp: 'nice 5/22/2008 14:12' prior: 0!
KeyedSetTest hold sunit tests for class KeyedSet!

!KeyedSetTest methodsFor: 'tests' stamp: 'ul 2/19/2010 15:57'!
testLike
	
	| k |
	k := KeyedSet keyBlock: [ :each | each * 2 ].
	self assert: (k like: 1) isNil.
	k add: 1.
	self assert: (k like: 1) = 1.
	k add: 2.
	self assert: (k like: 2) = 2.
	self assert: (k like: 3) isNil.! !

!KeyedSetTest methodsFor: 'tests' stamp: 'ul 2/19/2010 15:57'!
testLikeIfAbsent
	
	| k missing |
	k := KeyedSet keyBlock: [ :each | each * 2 ].
	missing := false.
	self assert: (k like: 1 ifAbsent: [ missing := true. nil ]) isNil.
	self assert: missing.
	k add: 1.
	missing := false.
	self assert: (k like: 1 ifAbsent: [ missing := true. nil ]) = 1.
	self deny: missing.
	k add: 2.
	self assert: (k like: 2 ifAbsent: nil) = 2.
	self assert: (k like: 3 ifAbsent: nil) isNil! !

!KeyedSetTest methodsFor: 'tests' stamp: 'nice 5/22/2008 14:18'!
testSelect
	"this is a non regression test for http://bugs.squeak.org/view.php?id=6535"
	
	| ks ks2 |
	
	"Creare a KeyedSet"
	ks := KeyedSet keyBlock: [:e | e asInteger \\ 4].
	ks addAll: #(1.2 1.5 3.8 7.7 9.1 12.4 13.25 14.0 19.2 11.4).
	
	"There is non more than 4 possible keys (0 1 2 3)"
	self assert: ks size <= 4.
	
	"Select some elements"
	ks2 := ks select: [:e | e fractionPart > 0.5].

	"If keyBlock was preserved, then still no more than 4 keys..."
	ks2 addAll: #(1.2 1.5 3.8 7.7 9.1 12.4 13.25 14.0 19.2 11.4).
	self assert: ks2 size <= 4.! !

ClassTestCase subclass: #LinkedListTest
	instanceVariableNames: 'nextLink n list link1 link2 link3 link4'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

!LinkedListTest commentStamp: 'mk 8/3/2005 11:55' prior: 0!
A set of test cases which thoroughly test functionality of the LinkedList class.!

!LinkedListTest methodsFor: 'accessing' stamp: 'md 10/14/2004 10:47'!
n
	^n! !

!LinkedListTest methodsFor: 'accessing' stamp: 'md 10/14/2004 10:47'!
n: number
	n := number.
	! !

!LinkedListTest methodsFor: 'accessing' stamp: 'md 10/14/2004 10:46'!
nextLink
	^nextLink! !

!LinkedListTest methodsFor: 'accessing' stamp: 'md 10/14/2004 10:46'!
nextLink: aLink
	nextLink := aLink! !

!LinkedListTest methodsFor: 'running' stamp: 'zz 12/7/2005 19:07'!
setUp
	super setUp.
	list := LinkedList new.
	link1 := Link new.
	link2 := Link new.
	link3 := Link new.
	link4 := Link new! !

!LinkedListTest methodsFor: 'running' stamp: 'zz 12/7/2005 19:08'!
tearDown
	list := nil.
	link1 := nil.
	link2 := nil.
	link3 := nil.
	link4 := nil.
	^ super tearDown! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:04'!
test01add
	self assert: list isEmpty.
	list add: link1.
	self assert: list size = 1.
	self assert: list first = link1.
	
	list add: link2.
	self assert: list size = 2.
	self assert: list first = link1.
	self assert: list second = link2.
	
	list add: link3.
	self assert: list size = 3.
	self assert: list first = link1.
	self assert: list second = link2.
	self assert: list third = link3.
	
	list add: link4.
	self assert: list size = 4.
	self assert: list first = link1.
	self assert: list second = link2.
	self assert: list third = link3.
	self assert: list fourth = link4! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:04'!
test02addLast
	self assert: list isEmpty.
	
	list addLast: link1.
	self assert: list size = 1.
	self assert: list first = link1.
	
	list addLast: link2.
	self assert: list size = 2.
	self assert: list first = link1.
	self assert: list second = link2.
	
	list addLast: link3.
	self assert: list size = 3.
	self assert: list first = link1.
	self assert: list second = link2.
	self assert: list third = link3.
	
	list addLast: link4.
	self assert: list size = 4.
	self assert: list first = link1.
	self assert: list second = link2.
	self assert: list third = link3.
	self assert: list fourth = link4! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:04'!
test03addFirst
	self assert: list isEmpty.
	
	list addFirst: link1.
	self assert: list size = 1.
	self assert: list first = link1.
	
	list addFirst: link2.
	self assert: list size = 2.
	self assert: list first = link2.
	self assert: list second = link1.
	
	list addFirst: link3.
	self assert: list size = 3.
	self assert: list first = link3.
	self assert: list second = link2.
	self assert: list third = link1.
	
	list addFirst: link4.
	self assert: list size = 4.
	self assert: list first = link4.
	self assert: list second = link3.
	self assert: list third = link2.
	self assert: list fourth = link1! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:05'!
test04addBefore
	self assert: list isEmpty.
	
	list add: link1.
	self assert: list size = 1.
	self assert: list first == link1.
	
	list add: link2 before: link1.
	self assert: list size = 2.
	self assert: list first == link2.
	self assert: list second == link1.
	
	list add: link3 before: link1.
	self assert: list size = 3.
	self assert: list first == link2.
	self assert: list second == link3.
	self assert: list third == link1.
	
	list add: link4 before: link1.
	self assert: list size = 4.
	self assert: list first == link2.
	self assert: list second == link3.
	self assert: list third == link4.
	self assert: list fourth == link1! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:05'!
test05addBefore
	self assert: list isEmpty.
	
	list add: link1.
	self assert: list size = 1.
	self assert: list first == link1.
	
	list add: link2 before: link1.
	self assert: list size = 2.
	self assert: list first == link2.
	self assert: list second == link1.
	
	list add: link3 before: link2.
	self assert: list size = 3.
	self assert: list first == link3.
	self assert: list second == link2.
	self assert: list third == link1.
	
	list add: link4 before: link3.
	self assert: list size = 4.
	self assert: list first == link4.
	self assert: list second == link3.
	self assert: list third == link2.
	self assert: list fourth == link1! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:05'!
test06addAfter
	self assert: list isEmpty.
	
	list add: link1.
	self assert: list size = 1.
	self assert: list first == link1.
	
	list add: link2 after: link1.
	self assert: list size = 2.
	self assert: list first == link1.
	self assert: list second == link2.
	
	list add: link3 after: link2.
	self assert: list size = 3.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	
	list add: link4 after: link3.
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	self assert: list fourth == link4! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:10'!
test07addAfter
	self assert: list isEmpty.
	
	list add: link1.
	self assert: list size = 1.
	self assert: list first == link1.
	
	list add: link2 after: link1.
	self assert: list size = 2.
	self assert: list first == link1.
	self assert: list second == link2.
	
	list add: link3 after: link1.
	self assert: list size = 3.
	self assert: list first == link1.
	self assert: list second == link3.
	self assert: list third == link2.
	
	list add: link4 after: link1.
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link4.
	self assert: list third == link3.
	self assert: list fourth == link2! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:09'!
test08addAfter
	| l first |
	l := LinkedList new.
	first := self class new n: 1.
	
	l add: first.
	l add: (self class new n: 3).

	self assert: (l collect:[:e | e n]) asArray  = #(1 3).
	
	l add: (self class new n: 2) after: first.

	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3).! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:09'!
test09addAfter
	| l last |
	l := LinkedList new.
	last := self class new n: 2.
	l add: (self class new n: 1).
	l add: last.
	
	self assert: (l collect:[:e | e n]) asArray  = #(1 2).
	 
	l add: (self class new n: 3) after: last.

	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3).! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:10'!
test10removeFirst
	list add: link1.
	list add: link2.
	list add: link3.
	list add: link4.
	
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	self assert: list fourth == link4.
	
	list removeFirst.
	self assert: list size = 3.
	self assert: list first == link2.
	self assert: list second == link3.
	self assert: list third == link4.
	
	list removeFirst.
	self assert: list size = 2.
	self assert: list first == link3.
	self assert: list second == link4.
	
	list removeFirst.
	self assert: list size = 1.
	self assert: list first == link4.
	
	list removeFirst.
	self assert: list isEmpty! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:10'!
test11removeLast
	list add: link1.
	list add: link2.
	list add: link3.
	list add: link4.
	
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	self assert: list fourth == link4.
	
	list removeLast.
	self assert: list size = 3.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	
	list removeLast.
	self assert: list size = 2.
	self assert: list first == link1.
	self assert: list second == link2.
	
	list removeLast.
	self assert: list size = 1.
	self assert: list first == link1.
	
	list removeFirst.
	self assert: list isEmpty! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:11'!
test12remove
	list add: link1.
	list add: link2.
	list add: link3.
	list add: link4.
	
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	self assert: list fourth == link4.
	
	list remove: link3.
	self assert: list size = 3.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link4.
	
	list remove: link2.
	self assert: list size = 2.
	self assert: list first == link1.
	self assert: list second == link4.
	
	list remove: link1.
	self assert: list size = 1.
	self assert: list first == link4.
	
	list remove: link4.
	self assert: list isEmpty! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:12'!
test13remove
	list add: link1.
	list add: link2.
	list add: link3.
	list add: link4.
	
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	self assert: list fourth == link4.
	
	list remove: link1.
	self assert: list size = 3.
	self assert: list first == link2.
	self assert: list second == link3.
	self assert: list third == link4.
	
	list remove: link4.
	self assert: list size = 2.
	self assert: list first == link2.
	self assert: list second == link3.
	
	list remove: link2.
	self assert: list size = 1.
	self assert: list first == link3.
	
	list remove: link3.
	self assert: list isEmpty! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:13'!
test14removeIfAbsent
	list add: link1.
	
	self assert: list size = 1.
	self assert: list first == link1.
	
	list remove: link1.
	self assert: list isEmpty.
	
	[list remove: link1]
		on: Error
		do: [^ self].
		
	"The execution should not get here. If yes, something went wrong."
	self assert: false! !

!LinkedListTest methodsFor: 'testing' stamp: 'zz 12/7/2005 19:08'!
test22addAll
	| link5 link6 link7 link8 listToBeAdded |
	link5 := Link new.
	link6 := Link new.
	link7 := Link new.
	link8 := Link new.
	
	list
		add: link1;
		add: link2;
		add: link3;
		add: link4.
		
	listToBeAdded := LinkedList new.
	listToBeAdded
		add: link5;
		add: link6;
		add: link7;
		add: link8.
		
	list addAll: listToBeAdded.
	
	self should: [(list at: 1) == link1].
	self should: [(list at: 2) == link2].
	self should: [(list at: 3) == link3].
	self should: [(list at: 4) == link4].
	self should: [(list at: 5) == link5].
	self should: [(list at: 6) == link6].
	self should: [(list at: 7) == link7].
	self should: [(list at: 8) == link8].! !

!LinkedListTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:20'!
testAddAfter

	| l first |
	l := LinkedList new.
	first := self class new n: 1.
	
	l add: first.
	l add: (self class new n: 3).
	self assert: (l collect:[:e | e n]) asArray  = #(1 3).
	l add: (self class new n: 2) after: first.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3).! !

!LinkedListTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:20'!
testAddAfterLast

	| l last |
	l := LinkedList new.
	last := self class new n: 2.
	l add: (self class new n: 1).
	l add: last.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2). 
	l add: (self class new n: 3) after: last.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3).! !

!LinkedListTest methodsFor: 'tests' stamp: 'nice 7/28/2008 22:27'!
testAddAfterLast2
	"LinkedListTest new testAddAfterLast2"

	| l first second third fourth |
	l := LinkedList new.
	first := self class new n: 1.
	second := self class new n: 2.
	third := self class new n: 3.
	fourth :=self class new n: 4.
	l addLast: first.
	l addLast: second.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2). 
	l add: third after: second.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3).
	l addLast: fourth.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3 4).! !

!LinkedListTest methodsFor: 'testing' stamp: 'nice 9/14/2009 20:57'!
testRemoveAll
	| list2 |
	list add: link1.
	list add: link2.
	list add: link3.
	list add: link4.
	list2 := list copy.
	list removeAll.
	
	self assert: list size = 0.
	self assert: list2 size = 4 description: 'the copy has not been modified'! !

ClassTestCase subclass: #MatrixTest
	instanceVariableNames: 'matrix1 matrix2 matrix3'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

!MatrixTest methodsFor: 'testing' stamp: 'dc 3/3/2007 17:58'!
setUp
	matrix1 := Matrix new: 2.
	matrix1 at:1 at:1 put: 1.
	matrix1 at:1 at:2 put: 3.
	matrix1 at:2 at:1 put: 2.
	matrix1 at:2 at:2 put: 4.
	
	matrix2 := Matrix new: 2.
	matrix2 at:1 at:1 put: 3.
	matrix2 at:1 at:2 put: 7.
	matrix2 at:2 at:1 put: 4.
	matrix2 at:2 at:2 put: 8.! !

!MatrixTest methodsFor: 'tests - accessing' stamp: 'dc 3/3/2007 17:47'!
testAtAt
	self should:[matrix1 at: 2 at: 3] raise: Error.
	self should:[matrix1 at: 3 at: 2] raise: Error.
	self should:[matrix1 at: 3 at: 3] raise: Error.
	self should:[matrix1 at: 0 at: 1] raise: Error.
	self should:[matrix1 at: 1 at: 0] raise: Error.
	self should:[matrix1 at: 0 at: 0] raise: Error.
	
	self assert: (matrix1 at: 1 at: 1) = 1! !

!MatrixTest methodsFor: 'tests - copying' stamp: 'dc 3/3/2007 17:48'!
testCopy

	| copyMatrix |
	
	copyMatrix := matrix1 copy.
	self assert: matrix1 = copyMatrix ! !

!MatrixTest methodsFor: 'tests - testing' stamp: 'dc 3/3/2007 17:49'!
testIncludes
	self assert:
			((1 to: 4)
				allSatisfy: [:i | matrix1 includes: i])
! !

!MatrixTest methodsFor: 'tests - arithmetic' stamp: 'dc 3/3/2007 17:50'!
testMultiply
	
	| result |
	self	should: [matrix1	preMultiplyByMatrix: (Matrix new: 3)]raise: Error.

	result := matrix2 preMultiplyByMatrix: matrix1.
	self assert: (result at: 1 at: 1) = 15.
	self assert: (result at: 1 at: 2) = 31.
	self assert: (result at: 2 at: 1) = 22.
	self assert: (result at: 2 at: 2) = 46! !

!MatrixTest methodsFor: 'tests - accessing' stamp: 'dc 3/3/2007 17:52'!
testReplaceAll

	matrix1 replaceAll: 1 with: 10.
	self assert: (matrix1 at:1 at:1) = 10.
	self assert: (matrix1 at:2 at:1) = 2.
	self assert: (matrix1 at:1 at:2) = 3.
	self assert: (matrix1 at:2 at:2) = 4.! !

!MatrixTest methodsFor: 'tests - accessing' stamp: 'dc 3/3/2007 17:53'!
testSwap
	matrix1 swap: 1 at: 2 with: 1 at: 1.
	self assert: (matrix1 at: 1 at: 1) = 3.
	self assert: (matrix1 at: 1 at: 2) = 1.! !

!MatrixTest methodsFor: 'tests - accessing' stamp: 'dc 3/3/2007 17:58'!
testTransposed
	| transposedMatrix |
	
	transposedMatrix := matrix1 transposed.
	self assert: [(transposedMatrix at:1 at:1) = 1].
	self assert: [(transposedMatrix at:1 at:2) = 2].
	self assert: [(transposedMatrix at:2 at:1) = 3].
	self assert: [(transposedMatrix at:2 at:2) = 4].! !

ClassTestCase subclass: #OrderedCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

!OrderedCollectionTest commentStamp: 'BG 1/10/2004 22:07' prior: 0!
These test cases demonstrate addition of items into an OrderedCollection as well as item removal.

Some of the assertions are quite complicated and use a lot of collection protocol. Such methods do not test one single method, but protocol in general.!

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'zz 12/7/2005 19:00'!
testAdd
	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l add: 88.
	self assert: (l =  #(1 2 3 4 88) asOrderedCollection).
	l add: 99.
	self assert: (l =  #(1 2 3 4 88 99) asOrderedCollection). 

! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'zz 12/7/2005 18:52'!
testAddAfter

	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l add: 88 after: 1.
	self assert: (l =  #(1 88 2 3 4) asOrderedCollection).
	l add: 99 after: 2.
	self assert: (l =  #(1 88 2 99 3 4) asOrderedCollection). 

! !

!OrderedCollectionTest methodsFor: 'testing-public methods' stamp: 'ul 3/21/2010 17:46'!
testAddAfterIndex
	"self run: #testAddAfterIndex"
	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l add: 77 afterIndex: 0.
	self assert: (l =  #(77 1 2 3 4) asOrderedCollection).
	l add: 88 afterIndex: 2.
	self assert: (l =  #(77 1 88 2 3 4) asOrderedCollection). 
	l add: 99 afterIndex: l size.
	self assert: (l =  #(77 1 88 2 3 4 99) asOrderedCollection). 
	self should:[l add: 666 afterIndex: -1] raise: Error.
	self should:[l add: 666 afterIndex: l size+1] raise: Error.
	
	"Now make room by removing first two and last two elements,
	and see if the illegal bounds test still fails"
	(l first: 2) , (l last: 2) reversed do: [:e | l remove: e].
	self should: [l add: 666 afterIndex: -1] raise: Error.
	self should: [l add: 666 afterIndex: l size+1] raise: Error.! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'sd 3/21/2006 22:36'!
testAddAll
	"Allows one to add each element of an orderedCollection at the end of another
	orderedCollection "
	"self run:#testAddAll" 
	
	| c1 c2 |
	c1 := #(1 2 3 4 ) asOrderedCollection.
	c2 := #(5 6 7 8 9 ) asOrderedCollection.
	c1 addAll: c2.
	self assert: c1 = #(1 2 3 4 5 6 7 8 9) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'sd 3/21/2006 22:36'!
testAddAllFirst
	"Allows one to add each element of an orderedCollection at the beginning of another
	orderedCollection "
	"self run:#testAddAllFirst" 
	
	| c1 c2 |
	c1 := #(1 2 3 4 ) asOrderedCollection.
	c2 := #(5 6 7 8 9 ) asOrderedCollection.
	c2 addAllFirst: c1.
	self assert: c2 = #(1 2 3 4 5 6 7 8 9) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'sd 3/21/2006 22:36'!
testAddAllFirstUnlessAlreadyPresent
	"Allows one to add each element of an orderedCollection at the beginning of
	another orderedCollection preserving the order but no duplicate element"
	"self run:#testAddAllFirstUnlessAlreadyPresent" 
	
	| c1 c2 c3 |
	c1 := #(1 2 3 4 ) asOrderedCollection.
	c2 := #(5 6 7 8 9 ) asOrderedCollection.
	c3 := #(0 1 ) asOrderedCollection.
	c2 addAllFirstUnlessAlreadyPresent: c1.
	self assert: c2 = #(1 2 3 4 5 6 7 8 9 ) asOrderedCollection.
	c1 addAllFirstUnlessAlreadyPresent: c3.
	self deny: c1 = #(0 1 1 2 3 4 ) asOrderedCollection.
	self assert: c1 = #(0 1 2 3 4 ) asOrderedCollection.
	! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'sd 3/21/2006 22:36'!
testAddAllLast
	"Allows one to add each element of an orderedCollection at the beginning of another
	orderedCollection "
	"self run:#testAddAllLast" 
	
	| c1 c2 |
	c1 := #(1 2 3 4 ) asOrderedCollection.
	c2 := #(5 6 7 8 9 ) asOrderedCollection.
	c1 addAllLast: c2.
	self assert: c1 = #(1 2 3 4 5 6 7 8 9) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'zz 12/7/2005 18:57'!
testAddBefore

	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l add: 88 before: 1.
	self assert: (l =  #(88 1 2 3 4) asOrderedCollection).
	l add: 99 before: 2.
	self assert: (l =  #(88 1 99 2 3 4) asOrderedCollection). 

! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'zz 12/7/2005 13:42'!
testAddBeforeAndRemove

	| l initialCollection |
	l := #(1 2 3 4) asOrderedCollection.
	initialCollection := l shallowCopy.
	l add: 88 before: 1.
	self assert: (l =  #(88 1 2 3 4) asOrderedCollection).
	l add: 99 before: 2.
	self assert: (l =  #(88 1 99 2 3 4) asOrderedCollection). 
	l remove: 99.
	l remove: 88.
	self assert: l = initialCollection.

! !

!OrderedCollectionTest methodsFor: 'testing-public methods' stamp: 'ul 3/21/2010 17:46'!
testAddBeforeIndex
	"self run: #testAddBeforeIndex"
	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l add: 77 beforeIndex: 1.
	self assert: (l =  #(77 1 2 3 4) asOrderedCollection).
	l add: 88 beforeIndex: 3.
	self assert: (l =  #(77 1 88 2 3 4) asOrderedCollection). 
	l add: 99 beforeIndex: l size+1.
	self assert: (l =  #(77 1 88 2 3 4 99) asOrderedCollection). 
	self should:[l add: 666 beforeIndex: 0] raise: Error.
	self should:[l add: 666 beforeIndex: l size+2] raise: Error.
	
	"Now make room by removing first two and last two elements,
	and see if the illegal bounds test still fails"
	(l first: 2) , (l last: 2) reversed do: [:e | l remove: e].
	self should:[l add: 666 beforeIndex: 0] raise: Error.
	self should:[l add: 666 beforeIndex: l size+2] raise: Error.

! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'zz 12/7/2005 13:43'!
testAddDuplicateItem1
	| collection |
	collection := #('Jim' 'Mary' 'John' 'Andrew' ) asOrderedCollection.
	collection add: 'John' before: 'John'.
	self
		assert: ((collection asBag occurrencesOf: 'John')
					= 2
				and: [(collection at: (collection indexOf: 'John')
							+ 1)
						= (collection
								at: (collection indexOf: 'John'))])! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'zz 12/7/2005 18:58'!
testAddFirst
	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l addFirst: 88.
	self assert: (l =  #(88 1 2 3 4) asOrderedCollection).
	l addFirst: 99.
	self assert: (l =  #(99 88 1 2 3 4) asOrderedCollection). 

! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'sd 6/5/2005 09:21'!
testAddItem1

   | collection size |
   collection := #('Jim' 'Mary' 'John' 'Andrew' ) asOrderedCollection.
   size := collection size.
   collection add: 'James' before: 'Jim'.
   collection add: 'Margaret' before: 'Andrew'.
   self assert: size + 2 = collection size.
! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'zz 12/7/2005 19:07'!
testAddItem2
	| collection |
	collection := #('Jim' 'Mary' 'John' 'Andrew' ) asOrderedCollection.
	collection add: 'James' before: 'Jim'.
	collection add: 'Margaret' before: 'Andrew'.
	self assert: (collection indexOf: 'James')
			+ 1
			= (collection indexOf: 'Jim').
	self assert: (collection indexOf: 'Margaret')
			+ 1
			= (collection indexOf: 'Andrew')! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'zz 12/7/2005 18:59'!
testAddLast
	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l addLast: 88.
	self assert: (l =  #(1 2 3 4 88) asOrderedCollection).
	l addLast: 99.
	self assert: (l =  #(1 2 3 4 88 99) asOrderedCollection). 

! !

!OrderedCollectionTest methodsFor: 'testsAccessing' stamp: 'zz 12/7/2005 18:50'!
testAt
	| collection |
	collection := #('Jim' 'Mary' 'John' 'Andrew' ) asOrderedCollection.
	self assert: (collection at:1) = 'Jim'.
	self assert: (collection at:2) = 'Mary'! !

!OrderedCollectionTest methodsFor: 'testsAdding' stamp: 'sd 3/21/2006 22:37'!
testAtIfAbsentPut
	"Allows one to add an element at an index if no element exist at this index"
	"self run:#testAtIfAbsentPut" 
	
	| c |
	c := #(1 2 3 4 ) asOrderedCollection.
	self
		shouldnt: [c at: 2 ifAbsentPut: [5]]
		raise: Error.
	self assert: c = #(1 2 3 4 ) asOrderedCollection.
	c at: 5 ifAbsentPut: [5].
	self assert: c = #(1 2 3 4 5 ) asOrderedCollection.
	c at: 7 ifAbsentPut: [7].
	self assert: c = #(1 2 3 4 5 nil 7 ) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'testsAccessing' stamp: 'sd 3/21/2006 22:38'!
testAtPut
	"Allows one to replace an element but not at an off range index"
	"self run:#testAtPut"
	| c |
	c := #(1 2 3 4 ) asOrderedCollection.
	c at: 2 put: 5.
	self assert: c = #(1 5 3 4 ) asOrderedCollection.
	self
		should: [c at: 5 put: 8]
		raise: Error.
	self deny: c = #(1 5 3 4 8 ) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'testsAccessing' stamp: 'sd 3/21/2006 22:39'!
testCapacity
	"Allows one to check the current capacity of an Ordered collection"
	"self run:#testCapacity"
	
	| c1 c2 c3 |
	c1 := #(1 2 ) asOrderedCollection.
	self assert: (c1 capacity =  2).
	c2 := OrderedCollection new: 10.
	c2 add: 3.
	self assert: (c2 capacity = 10).	
	c3 := OrderedCollection new.
	self deny: (c3 capacity =  0).
	! !

!OrderedCollectionTest methodsFor: 'testsEnumerating' stamp: 'sd 3/21/2006 22:41'!
testCollect
	"Allows one to collect some element of a collection into another collection"
	"self run: #testCollect"
	 
	| c1 c2 res |
	c1 := #(-1 2 -3 4) asOrderedCollection.
	c2 := #(1 2 3 4) asOrderedCollection.
	res := c1 collect: [:each | each abs].
	self assert: (c2 = res).! !

!OrderedCollectionTest methodsFor: 'testsEnumerating' stamp: 'cm 3/8/2006 09:09'!
testCollectFromTo
	"Allows one to collect some element of a collection into another collection between a first index and an end index for the collect"
	"self run: #testCollectFromTo"
	
	| c1 res |
	c1 := #(-1 2 -3 4 -5 6 -7 8) asOrderedCollection.
	res := c1 collect: [:each | each abs] from: 1 to: 3.
	self assert: (res = #(1 2 3) asOrderedCollection).
	self should: [c1 collect: [:each | each abs] from: 10 to: 13] raise: Error.
	self should: [c1 collect: [:each | each abs] from: 5 to: 2] raise: Error.! !

!OrderedCollectionTest methodsFor: 'testsCopying' stamp: 'sd 3/21/2006 22:40'!
testCopyEmpty
	"Allows one to create a copy of the receiver that contains no elements"
	"self run:#testCopyEmpty"
	
	| c1 c2 |
	c1 := #(1 2 3 4 ) asOrderedCollection.
	c2 := c1 copyEmpty.
	self assert: (c2 size = 0).! !

!OrderedCollectionTest methodsFor: 'testsCopying' stamp: 'sd 3/21/2006 22:41'!
testCopyFromTo
	"Allows one to create a copy of the receiver that contains elements from position start to end"
	"self run: #testCopyFromTo"
	
	| c1 c2 c3 | 
	c1 := #(1 2 3 4) asOrderedCollection.
	c2 := (c1 copyFrom: 1 to: 2).
	self assert: c2 = #(1 2) asOrderedCollection.
	self should: [c1 copyFrom: 10 to: 20] raise: Error.
	
	c3 := c1 copyFrom: 4 to: 2.
	self assert: c3 isEmpty.
	
	self should: [c1 copyFrom: 4 to: 5 ] raise: Error.
	
	
	
! !

!OrderedCollectionTest methodsFor: 'testsCopying' stamp: 'sd 3/21/2006 22:41'!
testCopyReplaceFromToWith
	"Allows one to create a copy from the receiver which elements between start and end of the 	receiver being replace by 	element of the collection after with:"
	"self run:#testCopyReplaceFromToWith"

	| c1 c2 c3 c4 |
	c1 := #(1 2 3 4) asOrderedCollection.
	c2 := #(5 6 7 8 9) asOrderedCollection.
	c3 := (c2 copyReplaceFrom: 1 to: 2 with: c1).
	self assert: c3 = #(1 2 3 4 7 8 9) asOrderedCollection.
	self should: [c2 copyReplaceFrom: 3 to: 1 with: c1] raise: Error.
	
	c4 := (c2 copyReplaceFrom: 10 to: 25 with: c1).
	self assert: c4 = #(5 6 7 8 9 1 2 3 4) asOrderedCollection.
	
	! !

!OrderedCollectionTest methodsFor: 'testsCopying' stamp: 'sd 3/21/2006 22:41'!
testCopyWith
	"Allows one to create a copy of the receiver that contains the new element at the end"
	"self run: #testCopyWith"
	
	| c1 | 
	c1 := #(1 2 3 4) asOrderedCollection.
	c1 := c1 copyWith: 6.
	self assert: c1 = #(1 2 3 4 6) asOrderedCollection.
	
	

	
	
	
! !

!OrderedCollectionTest methodsFor: 'testsCopying' stamp: 'nice 2/26/2009 11:54'!
testForceToPadding
	"This is a non regression bug for http://bugs.squeak.org/view.php?id=7296"
	
	| c1 c2 paddingElement |
	"source collection"
	c1 := #(5 3 7 2 ) asOrderedCollection.
	paddingElement := nil.
	
	"force length to longer"
	self shouldnt: [c2 := c1 forceTo: 10 paddingWith: paddingElement] raise: Error.
	self assert: (c1 ~~ c2) description: 'forceTo:paddingWith: should modify a copy'.
	self assert: (c2 size = 10).
	self assert: (c2 copyFrom: 1 to: c1 size) = c1.
	self assert: ((c1 size + 1 to: c2 size) allSatisfy: [:i | (c2 at: i) = paddingElement]).
	
	"force length to shorter"
	self shouldnt: [c2 := c1 forceTo: 3 paddingWith: paddingElement] raise: Error.
	self assert: (c1 ~~ c2) description: 'forceTo:paddingWith: should modify a copy'.
	self assert: (c2 size = 3).
	self assert: (c2 copyFrom: 1 to: 3) = (c1 copyFrom: 1 to: 3).
	
	"SAME TEST BUT PADDING AT START FOLLOWING..."
	
	"force length to longer"
	self shouldnt: [c2 := c1 forceTo: 10 paddingStartWith: paddingElement] raise: Error.
	self assert: (c1 ~~ c2) description: 'forceTo:paddingStartWith: should modify a copy'.
	self assert: (c2 size = 10).
	self assert: (c2 copyFrom: c2 size - c1 size + 1 to: c2 size) = c1.
	self assert: ((1 to: c2 size - c1 size) allSatisfy: [:i | (c2 at: i) = paddingElement]).
	
	"force length to shorter"
	self shouldnt: [c2 := c1 forceTo: 3 paddingStartWith: paddingElement] raise: Error.
	self assert: (c1 ~~ c2) description: 'forceTo:paddingStartWith: should modify a copy'.
	self assert: (c2 size = 3).
	self assert: (c2 copyFrom: 1 to: 3) = (c1 copyFrom: 1 to: 3).! !

!OrderedCollectionTest methodsFor: 'testsEnumerating' stamp: 'zz 12/7/2005 19:06'!
testIndexOf
	| collection indices |
	collection := #('Jim' 'Mary' 'John' 'Andrew' ) asOrderedCollection.
	indices := collection
				collect: [:item | collection indexOf: item].
	self assert: (1 to: 4) asOrderedCollection = indices! !

!OrderedCollectionTest methodsFor: 'testsEnumerating' stamp: 'sd 6/5/2005 09:21'!
testIndexOfWithDuplicates

   | collection indices bagOfIndices |
   collection := #('Jim' 'Mary' 'John' 'Andrew' 'Mary' 'John' 'Jim' 'Micheal') asOrderedCollection.
   indices := collection collect: [:item | collection indexOf: item].
   self assert: indices asSet size = collection asSet size.
   bagOfIndices := indices asBag.
   self assert: (indices asSet 
                    allSatisfy: [:index | (bagOfIndices occurrencesOf: index)
	                                       = (collection occurrencesOf: (collection at: index))]).

  "  indexOf:  returns the index of the first occurrence of an item.
     For an item with n occurrences, the index of its first occurrence
     is found  n  times. "! !

!OrderedCollectionTest methodsFor: 'testsEnumerating' stamp: 'ul 3/1/2011 08:21'!
testOverlappingPairsCollect

	| o |
	o := #(5 4 3 2 1) asOrderedCollection.
	self
		assert: #(9 7 5 3) asOrderedCollection
		equals: (o overlappingPairsCollect: [ :a :b | a + b ])! !

!OrderedCollectionTest methodsFor: 'testsRemoving' stamp: 'nice 9/14/2009 20:57'!
testRemoveAll
	"Allows one to remove all elements of a collection" 
	
	| c1 c2 s2 |
	c1 := #(2 3 4 6) asOrderedCollection.
	c1 addAll: (1 to: 200).
	c2 := c1 copy.
	s2 := c2 size.
	
	c1 removeAll.
	
	self assert: c1 size = 0.
	self assert: c2 size = s2 description: 'the copy has not been modified'
	! !

!OrderedCollectionTest methodsFor: 'testsRemoving' stamp: 'zz 12/7/2005 19:05'!
testRemoveAllSuchThat
	| collection |
	collection := (1 to: 10) asOrderedCollection.
	collection
		removeAllSuchThat: [:e | e even].
	self assert: collection = (1 to: 10 by: 2) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'testsRemoving' stamp: 'sd 3/21/2006 22:39'!
testRemoveAt
	"Allows one to remove an element from a collection at an index"
	"self run:#testRemoveAt" 
	
	| c1 |
	c1 := #(2 3 4 6) asOrderedCollection.
	c1 removeAt: 2.
	self assert: (c1 = #(2 4 6) asOrderedCollection).
	self should: [c1 removeAt: 10] raise: Error.
	self should: [c1 removeAt: -1] raise: Error.
	! !

!OrderedCollectionTest methodsFor: 'testsRemoving' stamp: 'sd 3/21/2006 22:39'!
testRemoveFirst
	"Allows one to remove n element of a collection at the first"
	"self run:#testRemoveFirst" 
	
	| c1 |
	c1 := #(2 3 4 6) asOrderedCollection.
	c1 removeFirst: 1.
	self assert: (c1 = #(3 4 6) asOrderedCollection).
	c1 removeFirst: 2.
	self assert: (c1 = #(6) asOrderedCollection).
	self should: [c1 removeFirst: 10] raise: Error.
	
	! !

!OrderedCollectionTest methodsFor: 'testsRemoving' stamp: 'sd 3/21/2006 22:39'!
testRemoveIfAbsent
	"Allows one to remove an element from a collection and to copy it in another collection."
	"If the element isn't in the first collection, the second collection copy the element after ifAbsent"
	"self run:#testRemoveIfAbsent"
	
	| c1 c2 |
	c1 := #(1 2 3 4) asOrderedCollection.
	c2 := OrderedCollection new.
	
	c2 add: (c1 remove: 2 ifAbsent: [6]).
	self assert: (c1 = #(1 3 4) asOrderedCollection).
	self assert: (c2 = #(2) asOrderedCollection).
	
	c2 add: (c1 remove: 18 ifAbsent: [6]).
	self assert: (c1 = #(1 3 4) asOrderedCollection).
	self assert: (c2 = #(2 6) asOrderedCollection).! !

!OrderedCollectionTest methodsFor: 'testsRemoving' stamp: 'sd 3/21/2006 22:39'!
testRemoveLast
	"Allows one to remove n element of a collection at the end"
	"self run:#testRemoveLast" 
	
	| c1 |
	c1 := #(2 3 4 6) asOrderedCollection.
	c1 removeLast: 1.
	self assert: (c1 = #(2 3 4) asOrderedCollection).
	c1 removeLast: 2.
	self assert: (c1 = #(2) asOrderedCollection).
	self should: [c1 removeLast: 10] raise: Error.! !

!OrderedCollectionTest methodsFor: 'testsCopying' stamp: 'zz 12/7/2005 13:47'!
testReversed
	| collection1 collection2 |
	collection1 := #('Jim' 'Mary' 'John' 'Andrew' ) asOrderedCollection.
	collection2 := collection1 reversed.
	self assert: collection2 first = 'Andrew'.
	self assert: collection2 last = 'Jim'! !

!OrderedCollectionTest methodsFor: 'testsAccessing' stamp: 'sd 3/21/2006 22:38'!
testSize
	"Allows one to check the size of an Ordered collection"
	"self run:#testSize"
	
	| c1 c2 |
	c1 := #(1 2 ) asOrderedCollection.
	self assert: (c1 size =  2).
	
	c2 := OrderedCollection new.
	self assert: (c2 size = 0)
	! !

!OrderedCollectionTest methodsFor: 'testStreaming' stamp: 'nice 2/26/2009 11:35'!
testStreamContents
	| oc |
	self shouldnt: [oc := OrderedCollection streamContents: [:stream | 1 to: 3 do: [:i | stream nextPut: i]]]
		raise: Error
		description: 'it should be possible to stream over an OrderedCollection'.
	self assert: oc class = OrderedCollection.
	self assert: oc = (1 to: 3) asOrderedCollection.! !

!OrderedCollectionTest methodsFor: 'testsEnumerating' stamp: 'cm 3/8/2006 10:02'!
testWithCollect
	"Allows one to collect some element of two collections into another collection with element corresponding to the condition in the blocks"
	"self run: #testWithCollect"
	
	| c1 c2 res |
	c1 := #(-1 2 -3 4 -5 6 -7 8) asOrderedCollection.
	c2 := #(-9 10 -11 12 -13 14 -15 16) asOrderedCollection.
	res := c1 with: c2 collect: [:each1 :each2 | each1 < each2
		ifTrue: [each1]
		ifFalse: [each2]].
	self assert: (res = #(-9 2 -11 4 -13 6 -15 8) asOrderedCollection).
	! !

ClassTestCase subclass: #ReadStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Streams'!

!ReadStreamTest commentStamp: 'tlk 12/5/2004 14:36' prior: 0!
I am an SUnit test for ReadStream.
I have no test fixtures.!

!ReadStreamTest methodsFor: 'accessing' stamp: 'dc 3/7/2007 13:48'!
classUnderTest
	^ ReadStream! !

!ReadStreamTest methodsFor: 'accessing - defaults' stamp: 'dc 2/27/2007 16:58'!
emptyStream
	^ ReadStream on: String new.! !

!ReadStreamTest methodsFor: 'helpers' stamp: 'dc 3/7/2007 13:55'!
streamOn: aCollection
	^ self classUnderTest on: aCollection! !

!ReadStreamTest methodsFor: 'helpers' stamp: 'dc 3/7/2007 13:58'!
streamOn: collection upToAll: subcollection

	^ (self streamOn: collection) upToAll: subcollection! !

!ReadStreamTest methodsFor: 'helpers' stamp: 'dc 2/28/2007 14:14'!
streamOn: collection upToAll: subcollection1 upToAll: subcollection2

	^(self streamOn: collection)
		upToAll: subcollection1;
		upToAll: subcollection2! !

!ReadStreamTest methodsFor: 'accessing - defaults' stamp: 'dc 2/27/2007 16:58'!
streamOnArray
	^ ReadStream on: (Array with: 1 with: #(a b c) with: false).! !

!ReadStreamTest methodsFor: 'accessing - defaults' stamp: 'dc 2/27/2007 16:58'!
streamOnString
	^ ReadStream on: 'abcde'.! !

!ReadStreamTest methodsFor: 'tests - testing' stamp: 'dc 2/28/2007 16:10'!
testAtEnd
	| stream |
	self assert: self emptyStream atEnd.

	stream := self streamOnArray.
	self deny: stream atEnd.
	stream next: 3.
	self assert: stream atEnd.! !

!ReadStreamTest methodsFor: 'tests - back' stamp: 'dc 8/1/2007 16:19'!
testBack
	"Test the new implemtation of the method back."
	|stream|
	stream := self streamOn: 'abc'.
	stream next: 2.
	self assert: stream back = $b.! !

!ReadStreamTest methodsFor: 'tests - back' stamp: 'dc 8/1/2007 16:22'!
testBackOnPosition1
	"Test the new implemtation of the method back."
	|stream|
	stream := self streamOn: 'abc'.
	stream next.
	self assert: stream back = $a.! !

!ReadStreamTest methodsFor: 'tests - positioning' stamp: 'dc 8/2/2007 12:07'!
testBackUpTo1
	|stream|
	stream := ReadStream on: 'abcdabg'.
	stream setToEnd.
	self assert: (stream backUpTo: 'ab').
	self assert: stream peek = $g! !

!ReadStreamTest methodsFor: 'tests - positioning' stamp: 'dc 8/2/2007 12:11'!
testBackUpToEmptyPattern1
	"This test represents the current behavior which is not clearly defined and could be revised."
	|stream|
	stream := ReadStream on: 'abcdabg'.
	stream setToEnd.
	self should: [stream backUpTo: ''] raise: Error.! !

!ReadStreamTest methodsFor: 'tests - positioning' stamp: 'dc 8/2/2007 12:08'!
testBackUpToPatternNotFound1
	|stream|
	stream := ReadStream on: 'abcdabg'.
	stream setToEnd.
	self deny: (stream backUpTo: 'zz').
	self assert: stream position = 0! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'dc 2/28/2007 15:32'!
testContents
	| stream |
	self assert: self emptyStream contents = ''.
	
	stream := self streamOnArray.
	self assert: stream contents = #(1 #(a b c) false).
	stream position: 3.
	self assert: stream contents = #(1 #(a b c) false).
	
	stream := self streamOnString.
	self assert: stream contents = 'abcde'.
	stream setToEnd.
	self assert: stream contents = 'abcde'.! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'dc 3/6/2007 18:26'!
testContents2
	"From ANSI Smalltalk Standard draft 1.9: 
	
		it is unspecified whether or not the returned collection [using #contents] is the same object as the backing store collection. However, if the returned collection is not the same object as the stream backing store collection then the class of the returned collection is the same class as would be returned if the message #select: was sent to the backing store collection."
		
	"In Squeak, there is #species to know what class should be used on copy, selection..."
	| interval stream streamContents |
	interval := 1 to: 32.
	stream := self streamOn: interval.
	streamContents := stream contents.
	
	(streamContents == interval)
		ifFalse: [self assert: streamContents class = Interval new species]! !

!ReadStreamTest methodsFor: 'tests - enumerating' stamp: 'dc 2/28/2007 16:20'!
testDo
	self emptyStream do: [:value | self fail]! !

!ReadStreamTest methodsFor: 'tests - enumerating' stamp: 'dc 2/28/2007 16:20'!
testDo2
	| stream string |
	stream := self streamOnArray.
	string := String new.
	
	stream do: [:value | string := string, ' ', value asString].
	
	self assert: string = (' ', 1 asString, ' ', #(a b c) asString, ' ', false asString)! !

!ReadStreamTest methodsFor: 'tests - testing' stamp: 'dc 2/27/2007 16:58'!
testIsEmpty
	| stream |
	self assert: self emptyStream isEmpty.
	
	stream := self streamOnArray.
	self deny: stream isEmpty.
	stream skip: 3.
	self deny: stream isEmpty.
	
	stream := self streamOnString.
	self deny: stream isEmpty.
	stream next;next;next.
	self deny: stream isEmpty.
	stream setToEnd.
	self deny: stream isEmpty.! !

!ReadStreamTest methodsFor: 'tests - instance creation' stamp: 'dc 3/7/2007 13:50'!
testNew
	self should: [self classUnderTest new] raise: Error.! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'dc 2/28/2007 16:23'!
testNext
	|stream|

	stream := self streamOnArray.
	self assert: stream next = 1.
	self assert: stream next = #(a b c).
	self assert: stream next = false.
	
	stream := self streamOnString.
	self assert: stream next = $a.
	self assert: stream next = $b.
	self assert: stream next = $c.
	self assert: stream next = $d.
	self assert: stream next = $e.
	! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'nice 12/7/2009 09:51'!
testNextLine
	| lines text stream |
	lines := #('line 1' ' and line 2' '' 'fourth').
	text := lines first , String cr , lines second , String crlf , lines third , String lf , lines fourth.
	stream := text readStream.
	lines do: [:e |
		self assert: stream nextLine = e].
	self assert: stream nextLine = nil! !

!ReadStreamTest methodsFor: 'tests - testing' stamp: 'dc 2/28/2007 16:30'!
testNextMatchFor
	| stream |
	stream := self streamOnArray.
	self assert: (stream nextMatchFor: 1).
	self assert: (stream nextMatchFor: #(a b c)).
	self assert: (stream nextMatchFor: false).
	
	stream := self streamOnArray.
	self deny: (stream nextMatchFor: false).
	self assert: (stream nextMatchFor: #(a b c)).
	self assert: (stream nextMatchFor: false).
! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'dc 2/28/2007 16:24'!
testNexts
	self assert: (self emptyStream next: 0) isEmpty.
	self assert: (self streamOnArray next: 0) isEmpty.
	self assert: (self streamOnArray next: 1) = #(1).
	self assert: (self streamOnArray next: 2) = #(1 #(a b c)).
	self assert: (self streamOnArray next: 3) = #(1 #(a b c) false).! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'dc 2/28/2007 15:40'!
testNexts2

	| stream |
	stream := self streamOnArray.
	self assert: (stream next: 2) = #(1 #(a b c)).
	self assert: (stream next: 1) = #(false).! !

!ReadStreamTest methodsFor: 'tests - back' stamp: 'dc 8/1/2007 16:20'!
testOldBack
	"Test the old behavior of the method back. The method #oldBack is a misconception about what a stream is. A stream contains a pointer *between* elements with past and future elements. The method #oldBack considers that the pointer is *on* an element. (Damien Cassou - 1 August 2007)"
	|stream|
	stream := self streamOn: 'abc'.
	stream next: 2.
	self assert: stream oldBack = $a.! !

!ReadStreamTest methodsFor: 'tests - back' stamp: 'dc 8/1/2007 16:22'!
testOldBackOnPosition1
	"Test the old behavior of the method back. The method #oldBack is a misconception about what a stream is. A stream contains a pointer *between* elements with past and future elements. The method #oldBack considers that the pointer is *on* an element. (Damien Cassou - 1 August 2007)"
	|stream|
	stream := self streamOn: 'abc'.
	stream next.
	self assert: stream oldBack = nil.! !

!ReadStreamTest methodsFor: 'tests - instance creation' stamp: 'dc 2/28/2007 15:39'!
testOn
	self shouldnt: [self streamOn: '  '] raise: Error.
	self assert: (self streamOn: '  ') position isZero.! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'dc 2/28/2007 16:35'!
testPeek
	| stream |
	stream := self streamOnArray.
	
	self assert: stream peek = 1.
	self deny: stream peek = #(a b c).
	self deny: stream peek = false.
	
	stream next.
	
	self deny: stream peek = 1.
	self assert: stream peek = #(a b c).
	self deny: stream peek = false.
	
	stream next.
	
	self deny: stream peek = 1.
	self deny: stream peek = #(a b c).
	self assert: stream peek = false.
	
	stream next.
	
	"In ANSI Smalltalk Standard Draft, it is said that nil will return nil at the end when using #peek."
	self assert: stream peek isNil.! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'dc 2/28/2007 16:37'!
testPeek2
	| stream |
	stream := self streamOn: #(nil nil nil).
	
	self assert: stream peek isNil.
	stream next.
	self assert: stream peek isNil.
	stream next.
	self assert: stream peek isNil.
	stream next.
	
	"Yes, #peek answers nil when there is no more element to read."
	self assert: stream peek isNil.! !

!ReadStreamTest methodsFor: 'tests - testing' stamp: 'dc 3/1/2007 14:48'!
testPeekFor
	| stream |

	stream := self streamOnArray.
	self assert: (stream peekFor: 1).
	self assert: (stream peekFor: #(a b c)).
	self assert: (stream peekFor: false).

	stream := self streamOnArray.
	self deny: (stream peekFor: #(a b c)).
	self deny: (stream peekFor: false).
	self assert: (stream peekFor: 1).

	self deny: (stream peekFor: 1).
	self deny: (stream peekFor: false).
	self assert: (stream peekFor: #(a b c)).
	
	self deny: (stream peekFor: 1).
	self deny: (stream peekFor: #(a b c)).
	self assert: (stream peekFor: false).
	
	self assert: (stream atEnd).
	self deny: (stream peekFor: nil).
	self deny: (stream peekFor: 1).
	self deny: (stream peekFor: #(a b c)).
	self deny: (stream peekFor: false).! !

!ReadStreamTest methodsFor: 'tests - testing' stamp: 'dc 3/1/2007 14:54'!
testPeekFor2
	| stream negative number |

	stream := self streamOn: '- 145'.
	negative := stream peekFor: $-.
	stream peekFor: Character space.
	number := stream next: 3.
	self assert: negative.
	self assert: number = '145'.

	stream := self streamOn: '-145'.
	negative := stream peekFor: $-.
	stream peekFor: Character space.
	number := stream next: 3.
	self assert: negative.
	self assert: number = '145'.
	
	stream := self streamOn: ' 145'.
	negative := stream peekFor: $-.
	stream peekFor: Character space.
	number := stream next: 3.
	self deny: negative.
	self assert: number = '145'.
	
	stream := self streamOn: '145'.
	negative := stream peekFor: $-.
	stream peekFor: Character space.
	number := stream next: 3.
	self deny: negative.
	self assert: number = '145'.! !

!ReadStreamTest methodsFor: 'tests - positioning' stamp: 'dc 2/28/2007 15:38'!
testPosition
	| stream |
	self assert: self emptyStream position isZero.
	
	stream := self streamOnArray.
	self assert: stream position = 0.
	stream next.
	self assert: stream position = 1.
	stream next.
	self assert: stream position = 2.
	stream next.
	self assert: stream position = 3.
	stream next.
	self assert: stream position = 3.
	stream next.
	self assert: stream position = 3.! !

!ReadStreamTest methodsFor: 'tests - positioning' stamp: 'ul 2/22/2011 16:31'!
testPositionOfSubCollection

	| stream |
	stream := self streamOn: 'xyz'.
	#(
		'' 1
		'q' 0
		'x' 1
		'y' 2
		'z' 3
		'xz' 0
		'xy' 1
		'yz' 2
		'xyz' 1) pairsDo: [ :pattern :expectedPosition |
			self
				assert: expectedPosition equals: (stream positionOfSubCollection: pattern);
				assert: 0 equals: stream position ]! !

!ReadStreamTest methodsFor: 'tests - positioning' stamp: 'dc 2/28/2007 15:44'!
testReset
	| stream |

	stream := self emptyStream.
	stream reset.
	self assert: stream position = 0.
	
	stream := self streamOnArray.
	stream reset.
	self assert: stream position = 0.
	self deny: stream atEnd.
	stream position: 3.
	self assert: stream atEnd.
	stream reset.
	self assert: stream position = 0.! !

!ReadStreamTest methodsFor: 'tests - positioning' stamp: 'dc 2/27/2007 17:41'!
testSetPosition
	| stream |

	stream := self emptyStream.
	self should: [stream position: -2] raise: Error.
	self shouldnt: [stream position: 0] raise: Error.

	stream := self streamOnArray.
	self should: [stream position: -1] raise: Error.
	self shouldnt: [stream position: 0] raise: Error.
	self shouldnt: [stream position: 1] raise: Error.
	self shouldnt: [stream position: 2] raise: Error.

	"According to ANSI Smalltalk Standard 1.9 Draft, the following should be tested too:
	self should: [stream position: 3] raise: Error.
	
	However, I don't see the point of raising an error when positioning at the end.
	
	I prefer testing the absence of error:
	"
	self shouldnt: [stream position: 3] raise: Error.
	self should: [stream position: 4] raise: Error.! !

!ReadStreamTest methodsFor: 'tests - positioning' stamp: 'dc 2/27/2007 17:47'!
testSetToEnd
	| stream |

	stream := self emptyStream.
	stream setToEnd.
	self assert: stream atEnd.
	
	stream := self streamOnArray.
	stream setToEnd.
	self assert: stream atEnd.
	stream position: 1.
	self deny: stream atEnd.
	stream setToEnd.
	self assert: stream atEnd.! !

!ReadStreamTest methodsFor: 'tests - positionning' stamp: 'dc 3/1/2007 15:17'!
testSkipTo
	| stream |
	stream := self emptyStream.
	self deny: (stream skipTo: nil).
	
	stream := self streamOnArray.
	self deny: stream atEnd.
	self deny: (stream skipTo: nil).
	self assert: stream atEnd.
	
	stream := self streamOnArray.
	self assert: stream peek = 1.
	self assert: (stream skipTo: #(a b c)).
	self assert: stream peek = false.
	self assert: (stream skipTo: false).
	self assert: stream atEnd.! !

!ReadStreamTest methodsFor: 'tests - positionning' stamp: 'dc 3/1/2007 15:18'!
testSkipTo2
	| stream |
	
	stream := self streamOnString.
	self assert: (stream skipTo: $b).
	self assert: stream peek = $c.
	self assert: (stream skipTo: $d).
	self assert: stream peek = $e.
	self assert: (stream skipTo: $e).
	self assert: stream atEnd.! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'dc 3/1/2007 12:41'!
testUpTo
	| returnValue stream |
	returnValue := (self emptyStream upTo: nil).
	self assert: returnValue isCollection.
	self assert: returnValue isEmpty.
	
	stream := self streamOnArray.
	returnValue := stream upTo: #(a b c).
	self assert: returnValue = #(1).
	self assert: stream peek = false.
	
	stream := self streamOnArray.
	returnValue := stream upTo: true.
	self assert: returnValue = #(1 #(a b c) false).
	self assert: stream atEnd.! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'dc 3/1/2007 12:42'!
testUpTo2
	| returnValue stream |

	stream := self streamOnString.
	returnValue := stream upTo: $d.
	self assert: returnValue = 'abc'.
	self assert: stream peek = $e.! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'nice 12/9/2009 13:05'!
testUpTo3
	| stream string |
	string := 'XYZabcdUVW'.
	stream := ReadStream on: string from: (string indexOf: $a) to: (string indexOf: $d).
	self assert: stream upToEnd = 'abcd'.
	self assert: stream atEnd.
	stream := ReadStream on: string from: (string indexOf: $a) to: (string indexOf: $d).
	self assert: (stream upTo: $c) = 'ab'.
	self assert: stream next = $d.
	stream := ReadStream on: string from: (string indexOf: $a) to: (string indexOf: $d).
	self assert: (stream upTo: $e) = 'abcd'.
	self assert: stream atEnd.! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'dc 2/28/2007 14:14'!
testUpToAll

	self assert: (self streamOn: 'abcdefgh' upToAll: 'cd') = 'ab'.
	self assert: (self streamOn: 'abcdefgh' upToAll: 'cd' upToAll: 'gh') = 'ef'.

	self assert: (self streamOn: '' upToAll: '') = ''.

	self assert: (self streamOn: 'a' upToAll: '') = ''.
	self assert: (self streamOn: 'a' upToAll: 'a') = ''.
	self assert: (self streamOn: 'a' upToAll: 'b') = 'a'.

	self assert: (self streamOn: 'ab' upToAll: '') = ''.
	self assert: (self streamOn: 'ab' upToAll: 'a') = ''.
	self assert: (self streamOn: 'ab' upToAll: 'b') = 'a'.
	self assert: (self streamOn: 'ab' upToAll: 'c') = 'ab'.
	self assert: (self streamOn: 'ab' upToAll: 'ab') = ''.

	self assert: (self streamOn: 'abc' upToAll: '') = ''.
	self assert: (self streamOn: 'abc' upToAll: 'a') = ''.
	self assert: (self streamOn: 'abc' upToAll: 'b') = 'a'.
	self assert: (self streamOn: 'abc' upToAll: 'c') = 'ab'.
	self assert: (self streamOn: 'abc' upToAll: 'd') = 'abc'.
	self assert: (self streamOn: 'abc' upToAll: 'ab') = ''.
	self assert: (self streamOn: 'abc' upToAll: 'bc') = 'a'.
	self assert: (self streamOn: 'abc' upToAll: 'cd') = 'abc'.
! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'nice 12/7/2009 19:48'!
testUpToAll2
	| string stream array |
	string := 'aababcabcd'.
	stream := self streamOn: string.
	self assert: (stream upToAll: 'a') = ''.
	self assert: (stream upToEnd) = 'ababcabcd'.
	stream := self streamOn: string.
	self assert: (stream upToAll: 'ab') = 'a'.
	self assert: (stream upToEnd) = 'abcabcd'.
	stream := self streamOn: string.
	self assert: (stream upToAll: 'abc') = 'aab'.
	self assert: (stream upToEnd) = 'abcd'.
	stream := self streamOn: string.
	self assert: (stream upToAll: 'abcd') = 'aababc'.
	self assert: (stream upToEnd) = ''.
	stream := self streamOn: string.
	self assert: (stream upToAll: 'abe') = 'aababcabcd'.
	self assert: (stream upToEnd) = ''.
	
	array := #(1 1 2 1 2 3 nil 1 2 3 4).
	stream := self streamOn: array.
	self assert: (stream upToAll: #(1)) = #().
	self assert: (stream upToEnd) = #(1 2 1 2 3 nil 1 2 3 4).
	stream := self streamOn: array.
	self assert: (stream upToAll: #(1 2)) = #(1).
	self assert: (stream upToEnd) = #(1 2 3 nil 1 2 3 4).
	stream := self streamOn: array.
	self assert: (stream upToAll: #(1 2 3)) = #(1 1 2).
	self assert: (stream upToEnd) = #(nil 1 2 3 4).
	stream := self streamOn: array.
	self assert: (stream upToAll: #(1 2 3 4)) = #(1 1 2 1 2 3 nil).
	self assert: (stream upToEnd) = #().
	stream := self streamOn: array.
	self assert: (stream upToAll: #(1 2 5)) = array.
	self assert: (stream upToEnd) = #().! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'nice 12/7/2009 09:57'!
testUpToAnyOf
	| stream |
	stream := 'abcde' readStream.
	self assert: (stream upToAnyOf: String crlf) = 'abcde'.
	self assert: stream atEnd.
	
	stream := 'abcde' readStream.
	self assert: (stream upToAnyOf: 'dc') = 'ab'.
	self assert: stream upToEnd = 'de'.
	
	stream := #( 1 2 nil true false 10 ) readStream.
	self assert: (stream upToAnyOf: #(100 1000)) = #( 1 2 nil true false 10 ).
	self assert: stream atEnd.
	
	stream := #( 1 2 nil true false 10 ) readStream.
	self assert: (stream upToAnyOf: #( true false )) = #( 1 2 nil ).
	self assert: stream upToEnd = #( false 10 ).! !

!ReadStreamTest methodsFor: 'tests - accessing' stamp: 'nice 12/7/2009 10:03'!
testUpToAnyOfDo
	| stream found |
	found := false.
	stream := 'abcde' readStream.
	self assert: (stream upToAnyOf: String crlf do: [:char | found := true]) = 'abcde'.
	self assert: stream atEnd.
	self deny: found.
	
	found := Object new.
	stream := 'abcde' readStream.
	self assert: (stream upToAnyOf: 'dc' do: [:char | found := char]) = 'ab'.
	self assert: found = $c.
	self assert: stream upToEnd = 'de'.
	
	found := false.
	stream := #( 1 2 nil true false 10 ) readStream.
	self assert: (stream upToAnyOf: #(100 1000) do: [:anObject | found := true]) = #( 1 2 nil true false 10 ).
	self deny: found.
	self assert: stream atEnd.
	
	found := Object new.
	stream := #( 1 2 nil true false 10 ) readStream.
	self assert: (stream upToAnyOf: #( true false ) do: [:anObject | found := anObject]) = #( 1 2 nil ).
	self assert: found = true.
	self assert: stream upToEnd = #( false 10 ).! !

ClassTestCase subclass: #ReadWriteStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Streams'!

!ReadWriteStreamTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class ReadWriteStream.
Unit tests are a good way to exercise the
functionality of your system in a repeatable and
automatic manner. They are therefore recommended if
you plan to release anything. For more information,
see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!ReadWriteStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:24'!
testConstructionUsingWith
	"Use the with: constructor."

	| aStream |
	aStream := ReadWriteStream with: #(1 2).
	self assert: (aStream contents = #(1 2)) description: 'Ensure correct initialization.'! !

!ReadWriteStreamTest methodsFor: 'tests - testing' stamp: 'dc 2/27/2007 16:06'!
testIsEmpty
	| stream |
	stream := ReadWriteStream on: String new.
	self assert: stream isEmpty.
	stream nextPut: $a.
	self deny: stream isEmpty.
	stream reset.
	self deny: stream isEmpty.
	stream next.
	self deny: stream isEmpty.! !

!ReadWriteStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:24'!
testNew

	self should: [ReadWriteStream new] raise: Error.! !

ClassTestCase subclass: #SequenceableCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

!SequenceableCollectionTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:21'!
testAfterIfAbsent

	| col |
	col := #(2 3 4).
	self assert: ((col after: 4 ifAbsent: ['block']) = 'block').
	self assert: ((col after: 5 ifAbsent: ['block']) = 'block').
	self assert: ((col after: 2 ifAbsent: ['block']) = 3).! !

!SequenceableCollectionTest methodsFor: 'tests - accessing' stamp: 'dc 3/3/2007 17:12'!
testAtAllPut
	|table|.
	table := Array new: 5.
	table atAllPut: $a.
	self assert: (table allSatisfy: [:elem | elem = $a])! !

!SequenceableCollectionTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:22'!
testBeforeIfAbsent

	| col |
	col := #(2 3 4).	
	self assert: ((col before: 2 ifAbsent: ['block']) = 'block').
	self assert: ((col before: 5 ifAbsent: ['block']) = 'block').
	self assert: ((col before: 3 ifAbsent: ['block']) = 2).! !

!SequenceableCollectionTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:22'!
testBeginsWith
	"We can't test SequenceableCollection directly. However, we can test a sampling of its descendants."

	| la prefix oc |
	la := #(1 2 3 4 5 6).
	oc := OrderedCollection new.
	oc add: 1; add: 2; add: 3.

	self assert: (la beginsWith: #(1)).
	self assert: (la beginsWith: #(1 2)).
	self assert: (la beginsWith: #(1 2 3)).
	self assert: (la beginsWith: oc).
	self deny: (la beginsWith: #()).
	self deny: (la beginsWith: '').
	self deny: (la beginsWith: OrderedCollection new).
	
	self assert: (oc beginsWith: #(1 2)).
	
	prefix := OrderedCollection new.
	self deny: (oc beginsWith: prefix).
	prefix add: 1.
	self assert: (oc beginsWith: prefix).
	prefix add: 2.
	self assert: (oc beginsWith: prefix).
	prefix add: 3.
	self assert: (oc beginsWith: prefix).
	prefix add: 4.
	self deny: (oc beginsWith: prefix).! !

!SequenceableCollectionTest methodsFor: 'tests - testing' stamp: 'ar 2/1/2010 19:14'!
testBeginsWithAnyOf
	"We can't test SequenceableCollection directly. However, we can test a sampling of its descendants."

	| la oc |
	la := #(1 2 3 4 5 6).
	oc := OrderedCollection new.
	oc add: 1; add: 2; add: 3.

	self assert: (la beginsWithAnyOf: #((17) (1) (42))).
	self assert: (la beginsWithAnyOf: #((17) (1 2) (42))).
	self assert: (la beginsWithAnyOf: #((17) (1 2 3) (42))).
	self deny: (la beginsWithAnyOf: #()).
	self deny: (la beginsWithAnyOf: #(())).
	self deny: (la beginsWithAnyOf: #((42))).
! !

!SequenceableCollectionTest methodsFor: 'tests - copying' stamp: 'dc 3/3/2007 17:13'!
testCopyWith
	| table |
	table := Array new: 4 withAll: 3.
	self assert: table = #(3 3 3 3).
	table := table copyWith: 4.
	self assert: table = #(3 3 3 3 4).! !

!SequenceableCollectionTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:22'!
testEndsWith
	"We can't test SequenceableCollection directly. However, we can test a sampling of its descendants."

	| la oc suffix |
	la := #(1 2 3 4 5 6).
	oc := OrderedCollection new.
	oc add: 4; add: 5; add: 6.
	
	self assert: (la endsWith: #(6)).
	self assert: (la endsWith: #(5 6)).
	self assert: (la endsWith: #(4 5 6)).
	self assert: (la endsWith: oc).
	self deny: (la endsWith: #()).
	self deny: (la endsWith: '').
	
	suffix := OrderedCollection new.
	suffix add: 6.
	self assert: (oc endsWith: suffix).
	suffix addFirst: 5.
	self assert: (oc endsWith: suffix).
	suffix addFirst: 4.
	self assert: (oc endsWith: suffix).
	suffix addFirst: 3.
	self deny: (oc endsWith: suffix).! !

!SequenceableCollectionTest methodsFor: 'tests - testing' stamp: 'ar 2/1/2010 19:13'!
testEndsWithAnyOf
	"We can't test SequenceableCollection directly. However, we can test a sampling of its descendants."

	| la oc |
	la := #(6 5 4 3 2 1).
	oc := OrderedCollection new.
	oc add: 3; add: 2; add: 1.

	self assert: (la endsWithAnyOf: #((17) (1) (42))).
	self assert: (la endsWithAnyOf: #((17) (2 1) (42))).
	self assert: (la endsWithAnyOf: #((17) (3 2 1) (42))).
	self deny: (la endsWithAnyOf: #()).
	self deny: (la endsWithAnyOf: #(())).
	self deny: (la endsWithAnyOf: #((42))).
! !

!SequenceableCollectionTest methodsFor: 'tests - converting' stamp: 'dc 3/3/2007 17:16'!
testEvalStrings
	| table |
	table := #('String new' 'Array with: 3 with: $a' '15+4').
	table := table evalStrings.

	self assert: table first isString.
	self assert: table first isEmpty.
	
	self assert: table second isArray.
	self assert: table second first = 3.
	self assert: table second second = $a.
	
	self assert: table third = 19.! !

!SequenceableCollectionTest methodsFor: 'tests - arithmetic' stamp: 'dc 3/3/2007 17:19'!
testPreMultiplyByArray
	| array|.

	array := #(3).
	self assert:(array preMultiplyByArray: 2)=6.
	
	array := Array new: 4.
	self should:[array preMultiplyByArray: 2] raise:Error.! !

!SequenceableCollectionTest methodsFor: 'tests - arithmetic' stamp: 'dc 3/3/2007 17:26'!
testPreMultiplyByMatrix
	| array matrix result|.
	array := #(1 2 3 4 5).
	
	"( 1  2  3  4  5
	  10 20 30 40 50)"
	matrix := Matrix
				rows:2 columns:5 tabulate: [:row :column | row = 1 
											ifTrue: column
											ifFalse: column * 10].
	result := array preMultiplyByMatrix: matrix.
	self assert: result isArray.
	self assert: result size = 2.
	self assert: result first = 55.
	self assert: result second = 550.! !

!SequenceableCollectionTest methodsFor: 'tests - arithmetic' stamp: 'dc 3/3/2007 17:28'!
testPreMultiplyByMatrix2
	| array matrix|.
	
	array := #(1 2 3 4 5).
	matrix := Matrix rows:1 columns:4 tabulate: [:row :column | column].
	
	"Not compatible size"
	self should:[array preMultiplyByMatrix: matrix] raise: Error.! !

!SequenceableCollectionTest methodsFor: 'tests - accessing' stamp: 'dc 3/5/2007 15:57'!
testReplaceFromToWithStartingAt
	| string |

	string := 'abcd' copy.
	string replaceFrom: 1 to: 3 with: 'lmnop' startingAt: 1.
	self assert: string = 'lmnd'.
	
	string := 'abcd' copy.
	string replaceFrom: 1 to: 3 with: 'lmnop' startingAt: 2.
	self assert: string = 'mnod'.
	
	string := 'abcd' copy.
	string replaceFrom: 2 to: 3 with: 'lmnop' startingAt: 1.
	self assert: string = 'almd'.! !

!SequenceableCollectionTest methodsFor: 'tests - converting' stamp: 'ar 2/27/2010 22:25'!
testSplitBy
	"Tests for  splitBy: and splitBy:do:"

	self assert: ('a/b' splitBy: '/') = #('a' 'b').
	self assert: ('a/b/' splitBy: '/') = #('a' 'b' '').
	self assert: ('/a/b/' splitBy: '/') = #('' 'a' 'b' '').
	self assert: ('///' splitBy: '/') = #('' '' '' '').
	self assert: ('/' splitBy: '/') = #('' '').
	self assert: ('a' splitBy: '/') = #('a').
	self assert: ('' splitBy: '/') = #('').

	self assert: ('a//b' splitBy: '//') = #('a' 'b').
	self assert: ('///' splitBy: '//') = #('' '/').

	self assert: ('Hello<p>World<p>' splitBy: '<p>') = #('Hello' 'World' '').
! !

ClassTestCase subclass: #SetTest
	instanceVariableNames: 'full empty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

!SetTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Set. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!SetTest methodsFor: 'running' stamp: 'GL 2/27/2006 21:30'!
setUp
	empty := Set new.
	full := Set with: 5 with: #abc! !

!SetTest methodsFor: 'running' stamp: 'md 4/16/2003 15:03'!
tearDown
	"I am called whenever your test ends. 
I am the place where you release the ressources"! !

!SetTest methodsFor: 'Sunit original tests' stamp: 'GL 2/28/2006 08:26'!
testAdd
	empty add: 5.
	self assert: (empty includes: 5).! !

!SetTest methodsFor: 'tests' stamp: 'GL 3/16/2006 10:36'!
testAddWithOccurences

	empty add: 2 withOccurrences: 3.
	self assert: (empty includes: 2).
	self assert: ((empty occurrencesOf: 2) = 1).! !

!SetTest methodsFor: 'tests' stamp: 'GL 3/16/2006 10:37'!
testAsSet
	"could be moved in Array or Collection"

	| newFull |
	newFull := #(#abc 5) asSet.
	newFull add: 5.
	self assert: (newFull = full).! !

!SetTest methodsFor: 'tests' stamp: 'GL 2/28/2006 09:25'!
testAtRandom
	| rand |
	rand := Random new.
	full add: 3.
	full add: 2.
	full add: 4.
	full add: 1.
	self assert: (full includes: (full atRandom: rand)).
	! !

!SetTest methodsFor: 'tests' stamp: 'GL 3/8/2006 09:09'!
testCollect
	| newFull result |
	newFull := Set withAll: (1 to: 10).
	result := newFull collect: [:each | each >= 1 ifTrue: [each] ifFalse: ['no']].
	self assert: (result = newFull).
	result := newFull collect: [:each | each >= 5 ifTrue: [each] ifFalse: ['no']].
	self assert: (result = ((Set withAll: (5 to: 10)) add: 'no'; yourself)).! !

!SetTest methodsFor: 'tests' stamp: 'GL 2/28/2006 09:46'!
testCopy
	| newFull |
	full add: 3.
	full add: 2.
	newFull := full copy.
	self assert: (full size = newFull size).
	self assert: ((full select: [:each | (newFull includes: each) not]) isEmpty).
	self assert: ((newFull select: [:each | (full includes: each) not]) isEmpty).! !

!SetTest methodsFor: 'tests' stamp: 'GL 3/8/2006 00:06'!
testCopyWithout
	| newFull |
	full add: 3.
	full add: 2.
	newFull := full copyWithout: 3.
	self assert: (newFull size = (full size - 1)).
	self deny: (newFull includes: 3).
	self assert: ((newFull select: [:each | (full includes: each) not]) isEmpty).
	self assert: ((full select: [:each | (newFull includes: each) not]) = (Set with: 3)).
	! !

!SetTest methodsFor: 'tests' stamp: 'GL 3/8/2006 09:23'!
testDo
	| newFull result |
	newFull := Set withAll: (1 to: 5).
	result := 0.
	newFull do: [:each | result := (result + each)].
	self assert: (result = 15).! !

!SetTest methodsFor: 'Sunit original tests' stamp: 'GL 2/28/2006 08:27'!
testGrow
	empty addAll: (1 to: 100).
	self assert: (empty size = 100).
			! !

!SetTest methodsFor: 'Sunit original tests' stamp: 'GL 2/28/2006 08:23'!
testIllegal
	self 
		should: [empty at: 5] raise: TestResult error.
	self 
		should: [empty at: 5 put: #abc] raise: TestResult error.
			! !

!SetTest methodsFor: 'Sunit original tests' stamp: 'GL 2/27/2006 21:42'!
testIncludes
	self assert: (full includes: 5).
	self assert: (full includes: #abc).
	self deny: (full includes: 3).
			! !

!SetTest methodsFor: 'tests' stamp: 'GL 3/8/2006 09:55'!
testIntersection
	| newFull col |
	full add: 3; add: 2.
	col := full intersection: full.
	self assert: (full = col).

	newFull := Set with: 8 with: 9 with: #z.
	col := newFull intersection: full.
	self assert: (col isEmpty).
	
	newFull add: 5; add: #abc; add: 7.
	col := newFull intersection: full.
	self assert: ((full select: [:each | (newFull includes: each)]) = col).
	
	
	! !

!SetTest methodsFor: 'tests' stamp: 'GL 2/28/2006 09:07'!
testLike
	self assert: ((full like: 5) = 5).
	self assert: ((full like: 8) isNil).! !

!SetTest methodsFor: 'Sunit original tests' stamp: 'GL 2/28/2006 08:27'!
testOccurrences
	self assert: ((empty occurrencesOf: 0) = 0).
	self assert: ((full occurrencesOf: 5) = 1).
	full add: 5.
	self assert: ((full occurrencesOf: 5) = 1).! !

!SetTest methodsFor: 'Sunit original tests' stamp: 'GL 3/8/2006 08:20'!
testRemove
	full remove: 5.
	self assert: (full includes: #abc).
	self deny: (full includes: 5).! !

!SetTest methodsFor: 'tests' stamp: 'nice 9/14/2009 21:08'!
testRemoveAll
	"Allows one to remove all elements of a collection" 
	
	| c1 c2 s2 |
	c1 := full.
	c2 := c1 copy.
	s2 := c2 size.
	
	c1 removeAll.
	
	self assert: c1 size = 0.
	self assert: c2 size = s2 description: 'the copy has not been modified'.! !

!SetTest methodsFor: 'tests' stamp: 'GL 3/8/2006 08:27'!
testRemoveIfAbsent
	| result1 result2  |
	result1 := true.
	result2 := true.
	full remove: 8 ifAbsent: [ result1 := false ].
	self assert: (result1 = false).
	full remove: 5 ifAbsent: [ result2 := false ].
	self assert: (result2 = true).
	
	
	! !

!SetTest methodsFor: 'tests' stamp: 'GL 2/28/2006 09:51'!
testSize
	self assert: (empty size = 0).
	self assert: (full size = 2).
	empty add: 2.
	empty add: 1.
	full add: 2.
	self assert: (empty size = 2).
	self assert: (full size = 3).
	empty remove: 2.
	self assert: (empty size = 1).! !

!SetTest methodsFor: 'tests' stamp: 'GL 3/8/2006 00:50'!
testUnion
	| newFull col newCol |
	full add: 3.
	full add: 2.
	col := full union: full.
	self assert: (full = col).

	newFull := Set with: 8 with: 9 with: #z.
	col := newFull union: full.
	self assert: (col size = (full size + newFull size)).
	self assert: ((col select: [:each | (full includes: each) not]) = newFull).
	self assert: ((col select: [:each | (newFull includes: each) not]) = full).

	full add: 9.
	col := newFull union: full.
	newCol := newFull copy.
	newCol remove: 9.
	self assert: (col size = (full size + newFull size - 1)).
	self assert: ((col select: [:each | (full includes: each) not]) = newCol).
	newCol := full copy.
	newCol remove: 9.
	self assert: ((col select: [:each | (newFull includes: each) not]) = newCol).
	
	
	! !

ClassTestCase subclass: #StringTest
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!StringTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class String. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!StringTest methodsFor: 'initialize-release' stamp: 'md 4/18/2003 10:00'!
setUp
	string := 'Hi, I am a String'! !

!StringTest methodsFor: 'tests - converting' stamp: 'ar 8/27/2009 20:43'!
testAsDecomposedUnicode
	"Test the behavior of #asDecomposedUnicode"
	| composed decomposed |

	"Tests single character"
	composed := String with: (Character value: 353).
	decomposed := String with: (Character value: 115) with: (Character value: 780).
	self assert: composed asDecomposedUnicode = decomposed.

	"Test character at end of text"
	composed := 'Hello', (String with: (Character value: 353)).
	decomposed := 'Hello', (String with: (Character value: 115) with: (Character value: 780)).
	self assert: composed asDecomposedUnicode = decomposed.

	"Test character in the midst of text"
	composed := 'Hello', (String with: (Character value: 353)), 'World'.
	decomposed := 'Hello', (String with: (Character value: 115) with: (Character value: 780)), 'World'.
	self assert: composed asDecomposedUnicode = decomposed.

	"Test character in the end of text"
	composed := (String with: (Character value: 353)), 'World'.
	decomposed := (String with: (Character value: 115) with: (Character value: 780)), 'World'.
	self assert: composed asDecomposedUnicode = decomposed.

	"Test string full of characters"
	"Test character in the end of text"
	composed := (String with: (Character value: 353)).
	decomposed := (String with: (Character value: 115) with: (Character value: 780)).
	self assert: (composed, composed, composed) asDecomposedUnicode = (decomposed, decomposed, decomposed).
! !

!StringTest methodsFor: 'tests - converting' stamp: 'sd 6/5/2005 09:26'!
testAsInteger

	self assert: '1796exportFixes-tkMX' asInteger = 1796.
	self assert: 'donald' asInteger isNil.
	self assert: 'abc234def567' asInteger = 234.
	self assert: '-94' asInteger = -94.
	self assert: 'foo-bar-92' asInteger = -92! !

!StringTest methodsFor: 'tests - converting' stamp: 'ar 8/27/2009 20:36'!
testAsPrecomposedUnicode
	"Test the behavior of #asPrecomposedUnicode"
	| composed decomposed |

	"Tests single character"
	composed := String with: (Character value: 353).
	decomposed := String with: (Character value: 115) with: (Character value: 780).
	self assert: decomposed asPrecomposedUnicode = composed.

	"Test character at end of text"
	composed := 'Hello', (String with: (Character value: 353)).
	decomposed := 'Hello', (String with: (Character value: 115) with: (Character value: 780)).
	self assert: decomposed asPrecomposedUnicode = composed.

	"Test character in the midst of text"
	composed := 'Hello', (String with: (Character value: 353)), 'World'.
	decomposed := 'Hello', (String with: (Character value: 115) with: (Character value: 780)), 'World'.
	self assert: decomposed asPrecomposedUnicode = composed.

	"Test character in the end of text"
	composed := (String with: (Character value: 353)), 'World'.
	decomposed := (String with: (Character value: 115) with: (Character value: 780)), 'World'.
	self assert: decomposed asPrecomposedUnicode = composed.

	"Test string full of characters"
	"Test character in the end of text"
	composed := (String with: (Character value: 353)).
	decomposed := (String with: (Character value: 115) with: (Character value: 780)).
	self assert: (decomposed, decomposed, decomposed) asPrecomposedUnicode = (composed, composed, composed).
! !

!StringTest methodsFor: 'tests - converting' stamp: 'sd 6/5/2005 09:27'!
testAsSmalltalkComment

	| exampleStrings  |
	exampleStrings := #(
		''
		' '
		'"'
		'""'
		'"""'
		'abc"abc'
		'abc""abc'
		'abc"hello"abc'
		'abc"'
		'"abc' ).

	"check that the result of scanning the comment is empty"
	exampleStrings do: [ :s |
		| tokens  |
		tokens :=  Scanner new scanTokens: s asSmalltalkComment.
		self assert: (tokens isEmpty) ].

	"check that the result has the same non-quote characters as the original"
	exampleStrings do: [ :s |
		self assert: ( (s copyWithout: $") = (s asSmalltalkComment copyWithout: $"))].

	"finnaly, test for some common kinds of inputs"
	self assert: ( 'abc' asSmalltalkComment = '"abc"').
	self assert: ( 'abc"abc' asSmalltalkComment = '"abc""abc"').
	self assert: ('abc""abc' asSmalltalkComment = '"abc""abc"' ).
		! !

!StringTest methodsFor: 'tests - accessing' stamp: 'sd 6/5/2005 09:27'!
testAt

	self assert: (string at: 1) = $H.! !

!StringTest methodsFor: 'tests - converting' stamp: 'ar 7/22/2009 22:02'!
testBase64
	self assert: 'SGVsbG8gV29ybGQ=' base64Decoded = 'Hello World'.
	self assert: 'Hello World' base64Encoded = 'SGVsbG8gV29ybGQ='.! !

!StringTest methodsFor: 'tests - finding' stamp: 'dtl 11/21/2009 15:57'!
testBeginsWith

	self assert: ('Elvis' beginsWith: 'Elv').
	self assert: ('Elvis' beginsWith: 'Elvis').
	self assert: ('Elvis' beginsWith: 'E').
	self deny: ('Elvis' beginsWith: 'e').
	self deny: ('Elvis' beginsWith: 'Elvis ').
	self deny: ('' beginsWith: '').
	self deny: ('abc' beginsWith: '')! !

!StringTest methodsFor: 'tests - converting' stamp: 'sd 6/5/2005 09:27'!
testCapitalized

	| uc lc empty |		
	uc := 'MElViN'.
	lc := 'mElViN'.
	empty := ' '.
	self assert:  lc capitalized = uc.
	self assert: uc capitalized = uc.
	"the string gets copied"
	self deny: uc capitalized == uc.
	self deny: empty capitalized == empty.! !

!StringTest methodsFor: 'tests - comparing' stamp: 'lr 7/7/2006 11:32'!
testComparing
	self assert: 'foo' < 'foo:'.
	self assert: 'foo' < 'fooBar'.
	self assert: 'foo' <= 'foo:'.
	self assert: 'foo' <= 'fooBar'.
	self assert: 'foo:' > 'foo'.
	self assert: 'fooBar' > 'foo'.
	self assert: 'foo:' >= 'foo'.
	self assert: 'fooBar' >= 'foo'! !

!StringTest methodsFor: 'tests - converting' stamp: 'nice 11/17/2009 11:16'!
testEncodeForHTTP

	self assert: 'aa aa  aa aa' encodeForHTTP = 'aa%20aa%20%C3%A9%C3%A9%20aa%20aa'! !

!StringTest methodsFor: 'tests - finding' stamp: 'dtl 11/21/2009 15:55'!
testEndsWith

	self assert: ('Elvis' endsWith: 'vis').
	self assert: ('Elvis' endsWith: 'Elvis').
	self assert: ('Elvis' endsWith: 's').
	self deny: ('Elvis' endsWith: 'S').
	self deny: ('Elvis' endsWith: ' Elvis').
	self deny: ('' endsWith: '').
	self deny: ('abc' endsWith: '')! !

!StringTest methodsFor: 'tests - comparing' stamp: 'nice 1/18/2010 14:54'!
testEquality

	self assert: 'abc' = 'abc' asWideString.
	self assert: 'abc' asWideString = 'abc'.
	self assert: (#[ 97 0 0 0 ] asString ~= 'a000' asWideString).
	self assert: ('a000' asWideString ~= #[ 97 0 0 0 ] asString).

	self assert: ('abc' sameAs: 'aBc' asWideString).
	self assert: ('aBc' asWideString sameAs: 'abc').
	self assert: (#[ 97 0 0 0 ] asString 
						sameAs: 'Abcd' asWideString) not.
	self assert: ('a000' asWideString sameAs: 
					#[ 97 0 0 0 ] asString) not.! !

!StringTest methodsFor: 'tests - formatting' stamp: 'ul 2/10/2011 22:53'!
testExpandMacrosWithArguments

	| patterns |
	patterns := {
		"Input - Expected output"
		#('' '').
		#('foo' 'foo').
		{ '<N>'. String cr }.
		{ '<T>'. String tab }.
		{ '<n>'. String cr }.
		{ '<t>'. String tab }.
		#('%>' '>').
		#('%<' '<').
		#('%%' '%').
		#('%a' 'a').
		#('%A' 'A').
		#('<1S>' 'foo').
		#('<1s>' 'foo').
		#('<10S>' '10').
		#('<10s>' '10').
		#('<2P>' '1').
		#('<3p>' 'true').
		#('<10P>' '''10''').
		#('<11p>' '11').
		#('<3?ab:cde>' 'ab').
		#('<4?ab:cde>' 'cde') }.
	"Test all possible 3 long sequences of the above patterns. Since the patterns include
	the empty string, all 1 and 2 long sequences will be tested too."
	patterns do: [ :first |
		patterns do: [ :second |
			patterns do: [ :third |
				| result |
				result := first first, second first, third first expandMacrosWithArguments: #('foo' 1 true false 5 6 7 8 9 '10' 11).
				self assert: first second, second second, third second equals: result ] ] ].
	"Test some malformed inputs too."
	#('%' '<' '<N' '<T' '<1T' '<>' '<N><' '<1st' '<A' '<1Phello>' '<1?abcd>' '<?a:b>') do: [ :each |
		self should: [ each expandMacrosWithArguments: #() ] raise: Error ]! !

!StringTest methodsFor: 'tests - accessing' stamp: 'nice 10/5/2009 03:44'!
testFindLastOccurenceOfStringStartingAt

	self assert: ('Smalltalk' findLastOccurrenceOfString: 'al' startingAt: 2) = 7.
	self assert: ('aaa' findLastOccurrenceOfString: 'aa' startingAt: 1) = 2.
	self assert: ('Smalltalk' asWideString findLastOccurrenceOfString: 'al' startingAt: 2) = 7.
	self assert: ('Smalltalk' asWideString findLastOccurrenceOfString: 'al' asWideString startingAt: 2) = 7.
	self assert: (('Smalltalk' copyWith: 835 asCharacter) findLastOccurrenceOfString: 'al' asWideString startingAt: 2) = 7.! !

!StringTest methodsFor: 'tests - finding' stamp: 'ul 9/14/2009 03:16'!
testFindStringStartingAt
	"Run the tests with both String's and ByteString's method"

	{
		"text pattern startIndex expectedResult"
		#('Smalltalk' 'al' 2 3).
		{ (Character value: 12345) asString, 'foo'. 'foo'. 1. 2 }.
		{ (Character value: 12345) asString, 'foo'. (Character value: 12345) asString, 'foo'. 1. 1 }.
		{ 'foo', (Character value: 12345) asString. (Character value: 12345) asString. 1. 4 }.
		#('ababab' 'ab' 2 3).
		#('abc' 'a' 2 0) } do: [ :input |
			#(
				(yourself yourself)
				"the (yourself asWideString) pair would sometimes result 0 
				(like here: 'Smalltalk' findString: 'al' asWideString startingAt: 1)
				if the text is a ByteString, because of the optimized version,
				but normally ByteStrings never ''contain'' WideStrings"
				(asWideString yourself)
				(asWideString asWideString)) do: [ :modifiers |
					| text pattern startIndex expectedResult result |
					text := input first perform: modifiers first.
					pattern := input second perform: modifiers second.
					startIndex := input third.
					expectedResult := input fourth.
					self assert: (result := text findString: pattern startingAt: startIndex) = expectedResult ] ]! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 9/11/2011 16:54'!
testFindTokens

	#("receiver (argument expectedResult argument expectedResult ...) ..."
		'' ('' () $a () 'a' () 'ab' () 'aaa' ())
		'a' ('' ('a') $a () 'a' () 'ab' () 'aaa' () $b ('a') 'b' ('a') 'bc' ('a'))
		'aa' ('' ('aa') $a () 'a' () 'ab' () 'aaa' () $b ('aa') 'b' ('aa') 'bc' ('aa'))
		'ab' ('' ('ab') $a ('b') 'a' ('b') 'ab' () 'aaa' ('b') $b ('a') 'b' ('a') 'bc' ('a'))
		'ababa' ('' ('ababa') $a ('b' 'b') 'a' ('b' 'b') 'ab' () 'aaa' ('b' 'b') $b ('a' 'a' 'a') 'b' ('a' 'a' 'a') 'bc' ('a' 'a' 'a'))
		'abbccba' ('' ('abbccba') $a ('bbccb') 'a' ('bbccb') 'ab' ('cc') 'aaa' ('bbccb') $b ('a' 'cc' 'a') 'b' ('a' 'cc' 'a') 'bc' ('a' 'a')))
		pairsDo: [ :receiver :argumentsAndExpectedResults |
			argumentsAndExpectedResults pairsDo: [ :argument :expectedResult |
				self assert: expectedResult asOrderedCollection equals: (
					receiver findTokens: argument) ] ]
! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:05'!
testFindTokensEscapedBy01

	| tokens |
	string := 'this, is, "a, test"'.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens size = 3! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'fbs 2/13/2006 22:20'!
testFindTokensEscapedBy02

	| tokens |
	string := ''.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens isEmpty! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:05'!
testFindTokensEscapedBy03

	| tokens |
	string := 'this, is, a, test'.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens size = 4! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:05'!
testFindTokensEscapedBy04

	| tokens |
	string := 'this, is, a"," test'.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens size = 3.
	self assert: tokens third = ' a, test'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'fbs 2/13/2006 22:19'!
testFindTokensEscapedBy05

	| tokens |
	string := 'this, /is, a"," test/'.
	tokens := string findTokens: ',#' escapedBy: '"/'.
	self assert: tokens size = 2.
	self assert: tokens first = 'this'.
	self assert: tokens second = ' is, a"," test'.! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy06

	| tokens |
	string := 'this, is, "a, test'.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens size = 3.
	self assert: tokens third = ' a, test'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy07

	| tokens |
	string := 'a:b::c'.
	tokens := string findTokens: ':' escapedBy: '"'.
	self assert: tokens size = 4.
	self assert: tokens first = 'a'.
	self assert: tokens second = 'b'.
	self assert: tokens third = ''.
	self assert: tokens fourth = 'c'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy08

	| tokens |
	string := 'this, is, ##a, test'.
	tokens := string findTokens: ',' escapedBy: '#'.
	self assert: tokens size = 4.
	self assert: tokens third = ' a'.
	self assert: tokens fourth = ' test'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy09

	| tokens |
	string := 'this, is, ###a, test#'.
	tokens := string findTokens: ',' escapedBy: '#'.
	self assert: tokens size = 3.
	self assert: tokens third = ' #a, test'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy10

	| tokens |
	string := 'this, is, ###a, test'.
	tokens := string findTokens: ',' escapedBy: '#'.
	self assert: tokens size = 3.
	self assert: tokens third = ' #a, test'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy11

	| tokens |
	string := 'this, is, """a, test"'.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens size = 3.
	self assert: tokens third = ' "a, test'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy12

	| tokens |
	string := 'one, two# three; four. five'.
	tokens := string findTokens: ',#;.' escapedBy: '"'.
	self assert: tokens size = 5.
	self assert: tokens third = ' three'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy13

	| tokens |
	string := 'one, two# three; four. five'.
	tokens := string findTokens: ',#;.' escapedBy: nil.
	self assert: tokens size = 5.
	self assert: tokens third = ' three'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy14

	| tokens |
	string := 'one, "two# three"; &four. five&'.
	tokens := string findTokens: ',#;.' escapedBy: '"&'.
	self assert: tokens size = 3.
	self assert: tokens second = ' two# three'.
	self assert: tokens third = ' four. five'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'fbs 2/13/2006 22:19'!
testFindTokensEscapedBy15

	| tokens |
	string := 'one, "two# three"; &four. five&'.
	tokens := string findTokens: nil escapedBy: '"&'.
	self assert: tokens size = 1.
	self assert: tokens first = 'one, two# three; four. five'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'fbs 2/13/2006 22:19'!
testFindTokensEscapedBy16

	| tokens |
	string := 'one, "two# three"; &four. five&'.
	tokens := string findTokens: nil escapedBy: nil.
	self assert: tokens size = 1.
	self assert: tokens first = string! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy21

	| tokens |
	string := 'this, is, "a, test"'.
	tokens := string findTokens: $, escapedBy: $".
	self assert: tokens size = 3! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'fbs 2/13/2006 22:19'!
testFindTokensEscapedBy22

	| tokens |
	string := ''.
	tokens := string findTokens: $, escapedBy: $".
	self assert: tokens size = 0! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy23

	| tokens |
	string := 'this, is, a, test'.
	tokens := string findTokens: $, escapedBy: $".
	self assert: tokens size = 4! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy24

	| tokens |
	string := 'this, is, a"," test'.
	tokens := string findTokens: $, escapedBy: $".
	self assert: tokens size = 3.
	self assert: tokens third = ' a, test'! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'fbs 2/13/2006 22:19'!
testFindTokensEscapedBy25

	| tokens |
	string := 'this, /is, a"," test/'.
	tokens := string findTokens: $, escapedBy: $/.
	self assert: tokens size = 2.
	self assert: tokens first = 'this'.
	self assert: tokens second = ' is, a"," test'.! !

!StringTest methodsFor: 'tests - tokenizing' stamp: 'ul 11/15/2010 12:06'!
testFindTokensEscapedBy26

	| tokens |
	string := 'this, is, "a, test'.
	tokens := string findTokens: $, escapedBy: $".
	self assert: tokens size = 3.
	self assert: tokens third = ' a, test'! !

!StringTest methodsFor: 'tests - formatting' stamp: 'ul 10/13/2009 01:23'!
testFormat

	{ 
		[ '\{ \} \\ foo {1} bar {2}' format: { 12. 'string' } ] -> '{ } \ foo 12 bar string'.
		[ '\{ \} \\ foo {2} bar {1}' format: { 'string'. 12 } ] -> '{ } \ foo 12 bar string'.
		[ '\{1}' format: {} ] -> '{1}'.
		[ '\{1}{1}' format: { $a } ] -> '{1}a'.
	} do: [ :each |
		self assert: each key value = each value ]! !

!StringTest methodsFor: 'tests - indexOf' stamp: 'nice 3/15/2007 21:11'!
testIndexOf
	
	"test for http://bugs.impara.de/view.php?id=3574"
	self assert: ('abc-' asWideString indexOfAnyOf: (CharacterSet newFrom: ' -0123456789')) = 4.
	self assert: ('ab7' asWideString indexOfAnyOf: (CharacterSet newFrom: ' -0123456789')) = 3.
	self assert: ('a2c' asWideString indexOfAnyOf: (CharacterSet newFrom: ' -0123456789')) = 2.
	self assert: ('3bc' asWideString indexOfAnyOf: (CharacterSet newFrom: ' -0123456789')) = 1.
	self assert: ('abc' asWideString indexOfAnyOf: (CharacterSet newFrom: ' -0123456789')) = 0.
	
	"extension to wide characters"
	self assert: ((String with: 803 asCharacter with: 811 asCharacter) indexOfAnyOf: (CharacterSet newFrom: (String with: 811 asCharacter with: 812 asCharacter))) = 2.
	
	self assert: ('abc' indexOfAnyOf: (CharacterSet newFrom: (String with: 811 asCharacter with: 812 asCharacter))) = 0.
	
	self assert: ('abc' indexOfAnyOf: (CharacterSet newFrom: (String with: 811 asCharacter with: $c))) = 3.
	
	"make sure start index is used in wide string algorithm"
	self assert: ('ab bcd abc' copyWith: 811 asCharacter) substrings = {'ab'. 'bcd'. 'abc' copyWith: 811 asCharacter}.! !

!StringTest methodsFor: 'tests - lines' stamp: 'nice 11/16/2009 13:22'!
testLineCorrespondingToIndex
	| sampleCRString sampleLFString sampleCRLFString anIndex |
	sampleCRString := 'Fred', String cr , 'the' , String cr , 'Bear'.
	sampleLFString := 'Fred', String lf , 'the' , String lf , 'Bear'.
	sampleCRLFString := 'Fred', String crlf , 'the' , String crlf , 'Bear'.
	
	anIndex := sampleCRString indexOf: $h.
	self assert: (sampleCRString lineCorrespondingToIndex: anIndex) = 'the'.
	anIndex := sampleLFString indexOf: $h.
	self assert: (sampleLFString lineCorrespondingToIndex: anIndex) = 'the'.
	anIndex := sampleCRLFString indexOf: $h.
	self assert: (sampleCRLFString lineCorrespondingToIndex: anIndex) = 'the'.
	
	anIndex := sampleCRString indexOf: $B.
	self assert: (sampleCRString lineCorrespondingToIndex: anIndex) = 'Bear'.
	anIndex := sampleLFString indexOf: $B.
	self assert: (sampleLFString lineCorrespondingToIndex: anIndex) = 'Bear'.
	anIndex := sampleCRLFString indexOf: $B.
	self assert: (sampleCRLFString lineCorrespondingToIndex: anIndex) = 'Bear'.
	
	anIndex := sampleCRString indexOf: $d.
	self assert: (sampleCRString lineCorrespondingToIndex: anIndex) = 'Fred'.
	anIndex := sampleLFString indexOf: $d.
	self assert: (sampleLFString lineCorrespondingToIndex: anIndex) = 'Fred'.
	anIndex := sampleCRLFString indexOf: $d.
	self assert: (sampleCRLFString lineCorrespondingToIndex: anIndex) = 'Fred'.! !

!StringTest methodsFor: 'tests - lines' stamp: 'nice 11/16/2009 13:13'!
testLineCount
	| sampleCRString sampleLFString sampleCRLFString |
	sampleCRString := 'Fred', String cr , 'the' , String cr , 'Bear'.
	sampleLFString := 'Fred', String lf , 'the' , String lf , 'Bear'.
	sampleCRLFString := 'Fred', String crlf , 'the' , String crlf , 'Bear'.
	
	self assert: sampleCRString lineCount = 3.
	self assert: sampleLFString lineCount = 3.
	self assert: sampleCRLFString lineCount = 3.! !

!StringTest methodsFor: 'tests - lines' stamp: 'nice 11/16/2009 13:14'!
testLineNumber
	| sampleCRString sampleLFString sampleCRLFString |
	sampleCRString := 'Fred', String cr , 'the' , String cr , 'Bear'.
	sampleLFString := 'Fred', String lf , 'the' , String lf , 'Bear'.
	sampleCRLFString := 'Fred', String crlf , 'the' , String crlf , 'Bear'.
	
	self assert: (sampleCRString lineNumber: 2) = 'the'.
	self assert: (sampleLFString lineNumber: 2) = 'the'.
	self assert: (sampleCRLFString lineNumber: 2) = 'the'.! !

!StringTest methodsFor: 'tests - lines' stamp: 'nice 11/16/2009 13:19'!
testLinesDo
	| sampleCRString sampleLFString sampleCRLFString lines |
	sampleCRString := 'Fred', String cr , 'the' , String cr , 'Bear'.
	sampleLFString := 'Fred', String lf , 'the' , String lf , 'Bear'.
	sampleCRLFString := 'Fred', String crlf , 'the' , String crlf , 'Bear'.
	
	lines := OrderedCollection new.
	sampleCRString linesDo: [:aLine |
		self deny: (aLine includes: Character cr).
		self deny: (aLine includes: Character lf).
		lines add: aLine].
	self assert: lines asArray = #('Fred' 'the' 'Bear').
	
	lines := OrderedCollection new.
	sampleLFString linesDo: [:aLine |
		self deny: (aLine includes: Character cr).
		self deny: (aLine includes: Character lf).
		lines add: aLine].
	self assert: lines asArray = #('Fred' 'the' 'Bear').

	lines := OrderedCollection new.
	sampleCRLFString linesDo: [:aLine |
		self deny: (aLine includes: Character cr).
		self deny: (aLine includes: Character lf).
		lines add: aLine].
	self assert: lines asArray = #('Fred' 'the' 'Bear').

! !

!StringTest methodsFor: 'tests - comparing' stamp: 'ul 1/15/2011 13:30'!
testMatch

	#('**' 'f**'  'f**o' 'f*' '*f*' 'f#*' 'f##' '*oo' '#oo' '*o*' '#o#' '#o*' '*o#' 'fo*' 'fo#' '*foo*' '###' '*#'  'f*#' 'f*#o')
		do: [ :each | self assert: (each match: 'foo') ].
	#('bar' 'foo#' '#foo' '*foo#' '#foo*' '*bar*') do: [ :each |
		self deny: (each match: 'foo') ]! !

!StringTest methodsFor: 'tests - converting' stamp: 'KR 06/24/2005 11:21'!
testPercentEncodingJa
	| leading hiraA hiraO hiraAO encodedHiraA encodedHiraO encodedHiraAO |

    "Make Japanese String from unicode. see http://www.unicode.org/charts/PDF/U3040.pdf"
     leading := JapaneseEnvironment leadingChar.
	hiraA := (Character leadingChar: leading code: 16r3042) asString.  "HIRAGANA LETTER A"
	hiraO := (Character leadingChar: leading code: 16r304A) asString.  "HIRAGANA LETTER O"
	hiraAO := hiraA, hiraO.

	"Percent Encoded Japanese String"
	encodedHiraA := hiraA encodeForHTTP.
	self assert: encodedHiraA = '%E3%81%82'.
	encodedHiraO := hiraO encodeForHTTP.
	self assert: encodedHiraO = '%E3%81%8A'.
	encodedHiraAO := hiraAO encodeForHTTP.
	self assert: encodedHiraAO =  '%E3%81%82%E3%81%8A'.

     "without percent encoded string"
	self assert: '' unescapePercents = ''.
	self assert: 'abc' unescapePercents = 'abc'.	"latin1 character"
	self assert: hiraAO unescapePercents = hiraAO.  "multibyte character"

	"encoded latin1 string"
	self assert: '%61' unescapePercents = 'a'.
	self assert: '%61%62%63' unescapePercents = 'abc'.

	"encoded multibyte string"
	Locale currentPlatform: (Locale isoLanguage: 'ja') during: [ 
		self assert: encodedHiraA unescapePercents = hiraA.
		self assert: encodedHiraAO unescapePercents = hiraAO].

	"mixed string"
	Locale currentPlatform: (Locale isoLanguage: 'ja') during: [ 
		self assert: (encodedHiraAO,'a') unescapePercents = (hiraAO, 'a').
		self assert: ('a', encodedHiraA) unescapePercents = ('a', hiraA).
		self assert: ('a', encodedHiraA, 'b')  unescapePercents = ('a', hiraA, 'b').
		self assert: ('a', encodedHiraA, 'b', encodedHiraO) unescapePercents = ('a', hiraA, 'b', hiraO).
		self assert: (encodedHiraA, encodedHiraO, 'b', encodedHiraA) unescapePercents = (hiraA, hiraO, 'b', hiraA)].


	"for Seaside"
	Locale currentPlatform: (Locale isoLanguage: 'ja') during: [ 
		self assert: (encodedHiraA, '+', encodedHiraO) unescapePercents = (hiraA, ' ', hiraO)].

! !

!StringTest methodsFor: 'tests - instance creation' stamp: 'dc 3/21/2007 11:22'!
testSpace
	"self debug: #testSpace"
	
	string := String new.
	self assert: string size = 0. "instead of #isEmpty to be consistent with the following test"
	
	string := String space.
	self assert: string size = 1.
	self assert: string = ' '! !

!StringTest methodsFor: 'tests - converting' stamp: 'ul 9/11/2011 16:57'!
testSubStrings

	#("receiver (argument expectedResult argument expectedResult ...) ..."
		'' ('' () ($a) () 'a' () 'ab' () 'aaa' ())
		'a' ('' ('a') ($a) () 'a' () 'ab' () 'aaa' () ($b) ('a') 'b' ('a') 'bc' ('a'))
		'aa' ('' ('aa') ($a) () 'a' () 'ab' () 'aaa' () ($b) ('aa') 'b' ('aa') 'bc' ('aa'))
		'ab' ('' ('ab') ($a) ('b') 'a' ('b') 'ab' () 'aaa' ('b') ($b) ('a') 'b' ('a') 'bc' ('a'))
		'ababa' ('' ('ababa') ($a) ('b' 'b') 'a' ('b' 'b') 'ab' () 'aaa' ('b' 'b') ($b) ('a' 'a' 'a') 'b' ('a' 'a' 'a') 'bc' ('a' 'a' 'a'))
		'abbccba' ('' ('abbccba') ($a) ('bbccb') 'a' ('bbccb') 'ab' ('cc') 'aaa' ('bbccb') ($b) ('a' 'cc' 'a') 'b' ('a' 'cc' 'a') 'bc' ('a' 'a')))
		pairsDo: [ :receiver :argumentsAndExpectedResults |
			argumentsAndExpectedResults pairsDo: [ :argument :expectedResult |
				self assert: expectedResult equals: (
					receiver subStrings: argument) ] ]
! !

!StringTest methodsFor: 'tests - converting' stamp: 'ul 8/20/2012 02:10'!
testUnescapePercents

	#(
		''	''
		'x'	'x'
		'+'	' '
		'x+'	'x '
		'+x'	' x'
		'x+x'	'x x'
		'%'	'%'
		'%3'	'%3'
		'%3C'	'<'
		'%3C%'	'%3C%'
		'%3C%3'	'%3C%3'
		'%3C%3C'	'<<'
		'%0G'	'%0G'
		'%3C%0G'	'%3C%0G'
		'+%3'	'+%3'
		'%3C+%'	'%3C+%'
		'%3C%+'	'%3C%+'
		'%3Cx%3E4%3C%2Fx%3E' '<x>4</x>'
		'!!@#$%25%5E&*()%7B%7D%5B%5D=:/;?+''%22'	'!!@#$%^&*(){}[]=:/;? ''"'
		'!!%40%23%24%25%5E%26*()%7B%7D%5B%5D%3D%3A%2F%3B%3F%2B''%22'	'!!@#$%^&*(){}[]=:/;?+''"'
		'%21@%23%24%25%5E%26*%28%29%7B%7D%5B%5D%3D%3A/%3B%3F+%27%22'	'!!@#$%^&*(){}[]=:/;? ''"'
	) pairsDo: [ :from :to |
		self assert: to equals: from unescapePercents ]! !

!StringTest methodsFor: 'tests - converting' stamp: 'ky 7/8/2006 18:01'!
testUnescapePercentsWithTextEncoding
	| leading kataTe kataSu kataTo |
	leading := JapaneseEnvironment leadingChar.
	"Katakana letter Te"
	kataTe := (Character leadingChar: leading code: 12486) asString.
	"Katakana letter Su"
	kataSu := (Character leadingChar: leading code: 12473) asString.
	"Katakana letter To"
	kataTo := (Character leadingChar: leading code: 12488) asString.
	self assert: ('%83e%83X%83g' unescapePercentsWithTextEncoding: 'shift_jis')
			= (kataTe , kataSu , kataTo).
	self assert: ('%83e%83X%83g%20and%20%83e%83X%83g' unescapePercentsWithTextEncoding: 'shift_jis')
			= (kataTe , kataSu , kataTo , ' and ' , kataTe , kataSu , kataTo)! !

!StringTest methodsFor: 'tests - converting' stamp: 'dc 4/10/2007 09:35'!
testUpTo
	"self debug: #testUpTo"
	self assert: #up:to: keywords = #(up: to:).
	self assert: #copy:from:to: keywords = #(copy: from: to:).
	self assert: #up keywords = #(up).
	self assert: #at: keywords = #(at:).
	! !

!StringTest methodsFor: 'tests - converting' stamp: 'ul 1/29/2010 02:12'!
testUtf8ToSqueakByteOrderMark
	"Ensure that utf8ToSqueak ignores Byte order mark (BOM) just like UTF8TextConverter does"

	{
		#('﻿' '').
		#('﻿abc' 'abc').
		"Make sure that we remove inner BOMs for maximal compatibility with UTF8TextConverter"
		#('abc﻿' 'abc').
		{ 'ab﻿cӒ'. 'abc', (Unicode value: 1234) asString }
	} do: [ :each |
		self assert: each first utf8ToSqueak = each second ]! !

!StringTest methodsFor: 'tests - converting' stamp: 'nice 1/18/2010 18:10'!
testUtf8ToSqueakLeadingChar
	"Ensure utf8ToSqueak inserts the leading char just like UTF8TextConverter does"
	| data |
	data := #[ 227 129 130 227 129 132 227 129 134 227 129 136 227 129 138 ] asString "aiueo in Japanese".
	self assert: data utf8ToSqueak = (data convertFromEncoding: #utf8)

! !

!StringTest methodsFor: 'tests - converting' stamp: 'ul 10/13/2009 00:55'!
testWithBlanksTrimmed

	| s |
	self assert: ' abc  d   ' withBlanksTrimmed = 'abc  d'.
	self assert: 'abc  d   ' withBlanksTrimmed = 'abc  d'.
	self assert: ' abc  d' withBlanksTrimmed = 'abc  d'.
	self assert: (((0 to: 255) collect: [ :each | each asCharacter ] thenSelect: [ :each | each isSeparator ]) as: String) withBlanksTrimmed = ''.
	s := 'abcd'.
	self assert: s withBlanksTrimmed = s.
	self assert: s withBlanksTrimmed ~~ s.! !

!StringTest methodsFor: 'tests - converting' stamp: 'sd 6/5/2005 09:27'!
testWithFirstCharacterDownshifted

	| uc lc empty |		
	uc := 'MElViN'.
	lc := 'mElViN'.
	empty := ' '.
	self assert:  uc withFirstCharacterDownshifted = lc.
	self assert: lc withFirstCharacterDownshifted = lc.
	"the string gets copied"
	self deny: lc withFirstCharacterDownshifted == lc.
	self deny: empty withFirstCharacterDownshifted == empty.! !

!StringTest methodsFor: 'tests - internet' stamp: 'ul 11/23/2009 14:39'!
testWithInternetLineEndings

	{
		'abc' -> 'abc'.
		'abc', String cr -> ('abc', String crlf).
		'abc', String lf -> ('abc', String crlf).
		'abc', String crlf -> ('abc', String crlf).
		String cr, 'abc' -> (String crlf, 'abc').
		String lf, 'abc' -> (String crlf, 'abc').
		String crlf, 'abc' -> (String crlf, 'abc').
		'abc', String cr, String cr, 'abc' -> ('abc', String crlf, String crlf, 'abc').
		'abc', String lf, String lf, 'abc' -> ('abc', String crlf, String crlf, 'abc').
		'abc', String crlf, String crlf, 'abc' -> ('abc', String crlf, String crlf, 'abc').
		String cr, 'abc', String cr, String crlf, 'abc', String lf -> (String crlf, 'abc', String crlf, String crlf, 'abc', String crlf).
		String lf, 'abc', String lf, String crlf, 'abc', String cr -> (String crlf, 'abc', String crlf, String crlf, 'abc', String crlf).
	} do: [ :each |
		self assert: each key withInternetLineEndings = each value ]! !

!StringTest methodsFor: 'tests - converting' stamp: 'nice 3/25/2011 22:49'!
testWithNoLineLongerThan
	"self run: #testWithNoLineLongerThan"
	self assert: ('Hello World' withNoLineLongerThan: 5) = ('Hello', String cr, 'World').
	self shouldnt: [String crlf withNoLineLongerThan: 5] raise: Error.
	self shouldnt: [('Hello', String cr, String cr,'World') withNoLineLongerThan: 5] raise: Error.
	self assert: ('Sample text to line break at some point' withNoLineLongerThan:  28)
		= ('Sample text to line break at', String cr, 'some point').
	self assert: ('Sample text', String cr, 'to line break', String cr, 'at some point' withNoLineLongerThan:  28)
		= ('Sample text', String cr, 'to line break', String cr, 'at some point').
! !

!StringTest methodsFor: 'tests - internet' stamp: 'nice 1/18/2010 23:33'!
testWithSqueakLineEndings

	{
		'abc' -> 'abc'.
		'abc', String cr -> ('abc', String cr).
		'abc', String lf -> ('abc', String cr).
		'abc', String crlf -> ('abc', String cr).
		String cr, 'abc' -> (String cr, 'abc').
		String lf, 'abc' -> (String cr, 'abc').
		String crlf, 'abc' -> (String cr, 'abc').
		'abc', String cr, String cr, 'abc' -> ('abc', String cr, String cr, 'abc').
		'abc', String lf, String lf, 'abc' -> ('abc', String cr, String cr, 'abc').
		'abc', String crlf, String crlf, 'abc' -> ('abc', String cr, String cr, 'abc').
		String cr, 'abc', String cr, String crlf, 'abc', String lf -> (String cr, 'abc', String cr, String cr, 'abc', String cr).
		String lf, 'abc', String lf, String crlf, 'abc', String cr -> (String cr, 'abc', String cr, String cr, 'abc', String cr).
	} do: [ :each |
		self assert: each key withSqueakLineEndings = each value ]! !

!StringTest methodsFor: 'tests - internet' stamp: 'nice 1/18/2010 23:35'!
testWithUnixLineEndings

	{
		'abc' -> 'abc'.
		'abc', String cr -> ('abc', String lf).
		'abc', String lf -> ('abc', String lf).
		'abc', String crlf -> ('abc', String lf).
		String cr, 'abc' -> (String lf, 'abc').
		String lf, 'abc' -> (String lf, 'abc').
		String crlf, 'abc' -> (String lf, 'abc').
		'abc', String cr, String cr, 'abc' -> ('abc', String lf, String lf, 'abc').
		'abc', String lf, String lf, 'abc' -> ('abc', String lf, String lf, 'abc').
		'abc', String crlf, String crlf, 'abc' -> ('abc', String lf, String lf, 'abc').
		String cr, 'abc', String cr, String crlf, 'abc', String lf -> (String lf, 'abc', String lf, String lf, 'abc', String lf).
		String lf, 'abc', String lf, String crlf, 'abc', String cr -> (String lf, 'abc', String lf, String lf, 'abc', String lf).
	} do: [ :each |
		self assert: each key withUnixLineEndings = each value ]! !

!StringTest methodsFor: 'tests - internet' stamp: 'ul 11/16/2010 06:29'!
testWithoutQuoting

	#(
		'"foo"' 'foo'
		'''foo''' 'foo'
		'"foo''' '"foo'''
		'''foo"' '''foo"'
		'"foo' '"foo'
		'foo"' 'foo"'
		'foo' 'foo') pairsDo: [ :before :after |
			self assert: before withoutQuoting = after ]! !

!StringTest methodsFor: 'tests - lines' stamp: 'nice 11/16/2009 22:26'!
testWthNoLineLongerThan
	| phrases |
	phrases := {
		'Fred the bear went down to the brook to read his book in silence'.
		'Fred the bear went down' , String cr , ' to the brook to ' , String cr , 'read his book in silence'.
		'Fred the bear went down' , String lf , '  to the brook to ' , String lf , 'read his book in silence'.
		'Fred the bear went down' , String crlf , '  to the brook to ' , String crlf , 'read his book in silence'.
		}.
	#(5 7 9) do: [:length |
		phrases do: [:phrase |
			| split |
			split := phrase withNoLineLongerThan: length.
			self
				assert: (split copyWithoutAll: CharacterSet separators) = (phrase copyWithoutAll: CharacterSet separators)
				description: 'no information should be lost'.
			split linesDo: [:line |
				self assert: line size <= length description: 'lines should not be longer than prescribed'].
			phrase lineCount = 1 ifTrue: [| lastLineSize |
				lastLineSize := length.
				split linesDo: [:line |
					line isEmpty ifFalse: [| firstWordSize |
						firstWordSize := line indexOfAnyOf: CharacterSet separators startingAt: 1 ifAbsent: [line size + 1].
						self assert: (lastLineSize + firstWordSize) > length description: 'split was too short'.
						lastLineSize := line size]]]]]! !

!StringTest methodsFor: 'tests - converting' stamp: 'md 2/5/2007 15:21'!
testZipped
	| compressed |
	
	compressed := 'hello' zipped.
	self assert: (compressed unzipped = 'hello').! !

ClassTestCase subclass: #SymbolTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!SymbolTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Symbol. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!SymbolTest methodsFor: 'tests' stamp: 'md 9/6/2005 20:02'!
testAsMutator

	self assert: #x asMutator = #x:.
	self assert: #x asMutator isSymbol! !

!SymbolTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:29'!
testCapitalized

	| uc lc |		
	uc := #MElViN.
	lc := #mElViN.
	self assert:  lc capitalized = uc.
	self assert: uc capitalized = uc.
! !

!SymbolTest methodsFor: 'testing - testing' stamp: 'fbs 8/26/2012 20:41'!
testIsBinaryReturnsFalseForKeywordSelectors
	self deny: #do: isBinary.
	self deny: #ifTrue:ifFalse: isBinary.
	self deny: #with:with:with:with: isBinary.! !

!SymbolTest methodsFor: 'testing - testing' stamp: 'fbs 8/26/2012 20:42'!
testIsBinaryReturnsFalseForUnarySelectors
	self deny: #timesRepeat isBinary.
	self deny: #once isBinary.
	self deny: #isBinary isBinary.! !

!SymbolTest methodsFor: 'testing - testing' stamp: 'fbs 8/26/2012 20:33'!
testIsBinaryReturnsTrueForOperatorLikeSymbols
	self assert: #+ isBinary.
	self assert: #>>= isBinary.
	self assert: #| isBinary.! !

!SymbolTest methodsFor: 'as yet unclassified' stamp: 'md 2/16/2006 17:17'!
testNumArgs2
    "TODO: need to be extended to support shrinking and for selectors like #+ " 
	
	self assert: (#test numArgs: 0) = #test.
	self assert: (#test numArgs: 1) = #test:.
	self assert: (#test numArgs: 2) = #test:with:.
	self assert: (#test numArgs: 3) = #test:with:with:.
	

	self assert: (#test: numArgs: 0) = #test:.
	self assert: (#test: numArgs: 1) = #test:.
	self assert: (#test: numArgs: 2) = #test:with:.
	self assert: (#test: numArgs: 3) = #test:with:with:.
	
	self assert: (#test:with: numArgs: 0) = #test:with:.
	self assert: (#test:with: numArgs: 1) = #test:with:.
	self assert: (#test:with: numArgs: 2) = #test:with:.
	self assert: (#test:with: numArgs: 3) = #test:with:with:.
	self assert: (#test:with: numArgs: 4) = #test:with:with:with:.
	
	self assert: (#test:with:with: numArgs: 0) = #test:with:with:.
	self assert: (#test:with:with: numArgs: 1) = #test:with:with:.
	self assert: (#test:with:with: numArgs: 2) = #test:with:with:.
	self assert: (#test:with:with: numArgs: 3) = #test:with:with:.
	self assert: (#test:with:with: numArgs: 4) = #test:with:with:with:.! !

!SymbolTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:29'!
testWithFirstCharacterDownshifted

	| uc lc empty |		
	uc := #MElViN.
	lc := #mElViN.
	empty := #' '.
	self assert:  uc withFirstCharacterDownshifted = lc.
	self assert: lc withFirstCharacterDownshifted = lc.
	
! !

ClassTestCase subclass: #TextTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Text. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

ClassTestCase subclass: #WeakIdentityKeyDictionaryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Weak'!

!WeakIdentityKeyDictionaryTest methodsFor: 'coverage' stamp: 'ul 6/12/2010 21:14'!
classToBeTested 

	^WeakIdentityKeyDictionary! !

!WeakIdentityKeyDictionaryTest methodsFor: 'tests' stamp: 'ul 6/12/2010 21:14'!
testFinalizeValuesWhenLastChainContinuesAtFront

	| objectWithHashModulo dictionary capacity a b c |
	objectWithHashModulo := [ :requestedHash :modulo |
		| object |
		[ 
			object := Object new.
			object hash \\ modulo = requestedHash ] whileFalse.
		object ].
	dictionary := self classToBeTested new.
	capacity := dictionary capacity.
	a := objectWithHashModulo value: capacity - 2 value: capacity.
	dictionary at: a put: 1.
	b := objectWithHashModulo value: capacity - 1 value: capacity.
	dictionary at: b put: 2.
	c := objectWithHashModulo value: capacity - 2 value: capacity.
	dictionary at: c put: 3.
	self assert: dictionary capacity = capacity.
	self assert: (dictionary array at: capacity - 1) key == a.
	self assert: (dictionary array at: capacity) key == b.
	self assert: (dictionary array at: 1) key == c.
	a := nil.
	Smalltalk garbageCollect.
	dictionary finalizeValues.
	self assert: (dictionary includesKey: b).
	self assert: (dictionary includesKey: c).
	self assert: dictionary slowSize = 2.
! !

!WeakIdentityKeyDictionaryTest methodsFor: 'tests' stamp: 'md 7/15/2006 18:02'!
testNoNils
	| d |
	d := WeakIdentityKeyDictionary new
	at: 'hello' copy put: 'world';
		yourself.
	Smalltalk garbageCollect.
	self deny: (d keys includes: nil)! !

ClassTestCase subclass: #WeakRegistryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Weak'!

!WeakRegistryTest methodsFor: 'tests' stamp: 'ar 4/6/2010 22:31'!
testFinalization

	| w finalized block object |
	w := WeakRegistry new: 1.
	finalized := false.
	block := [ :v | finalized := v ].
	object := ObjectFinalizer "an object that responds to #finalize"
		receiver: block
		selector: #value:
		argument: true.
	w add: object.
	object := nil. "let it go"
	Smalltalk garbageCollect. "finalize it"

	"This is an odd issue. It seems that in some situations the finalization
	process doesn't run 'in time' for the isEmpty assertion below to succeed.
	This really *shouldn't* happen since isEmpty is full of real sends and
	there ought to be an interrupt check in there somewhere. However,
	since there are no real-time guarantees about finalization, it's fair to
	just wait a little to ensure that the finalization process has been run."
	(Delay forMilliseconds: 100) wait.

	self assert: w isEmpty.
	self assert: finalized! !

!WeakRegistryTest methodsFor: 'tests' stamp: 'ul 2/26/2010 13:59'!
testFinalizationWithMultipleFinalizersPerObject

	| object registry counter |
	registry := WeakRegistry new.
	object := Object new.
	counter := 0.
	5 timesRepeat: [
		registry add: object executor: (ObjectFinalizer
			receiver: [ counter := counter + 1 ]
			selector: #value) ].
	self assert: registry size = 1.
	object := nil. 
	Smalltalk garbageCollect.
	registry finalizeValues.
	self assert: registry isEmpty.
	self assert: counter = 5
! !

!WeakRegistryTest methodsFor: 'tests' stamp: 'ul 2/26/2010 14:50'!
testGrowingByAHighPriorityProcessDoesntLeak

	| w finalized block object executor semaphore |
	w := WeakRegistry new: 1.
	finalized := false.
	block := [ :v | finalized := v ].
	object := Object new.
	executor := ObjectFinalizer
		receiver: block
		selector: #value:
		argument: true.
	w add: object executor: executor.
	semaphore := Semaphore new.
	[ 
		object := nil. "let it go"
		w addAll: (1 to: 1000). "force growing"
		semaphore signal ] 
			forkAt: WeakArray runningFinalizationProcess priority + 1.
	semaphore wait.
	Smalltalk garbageCollect. "finalize it"
	self assert: w size = 1000.
	self assert: finalized! !

!WeakRegistryTest methodsFor: 'tests' stamp: 'ul 2/26/2010 14:50'!
testGrowingDoesntLeak

	| w finalized block object executor |
	w := WeakRegistry new: 1.
	finalized := false.
	block := [ :v | finalized := v ].
	object := Object new.
	executor := ObjectFinalizer
		receiver: block
		selector: #value:
		argument: true.
	w add: object executor: executor.
	object := nil. "let it go"
	w addAll: (1 to: 1000). "force growing"
	Smalltalk garbageCollect. "finalize it"
	self assert: w size = 1000.
	self assert: finalized! !

!WeakRegistryTest methodsFor: 'tests' stamp: 'ul 2/26/2010 14:50'!
testRemovingByAHighPriorityProcessDoesntLeak

	| w finalized block hash object executor semaphore |
	w := WeakRegistry new: 1.
	finalized := false.
	block := [ :v | finalized := v ].
	object := Object new.
	executor := ObjectFinalizer
		receiver: block
		selector: #value:
		argument: true.
	hash := object hash.
	w add: hash.
	w add: object executor: executor.
	semaphore := Semaphore new.
	[ 
		object := nil. "let it go"
		w remove: hash.
		semaphore signal ] 
			forkAt: WeakArray runningFinalizationProcess priority + 1.
	semaphore wait.
	Smalltalk garbageCollect. "finalize it"
	self assert: w isEmpty.
	self assert: finalized! !

!WeakRegistryTest methodsFor: 'tests' stamp: 'ul 2/26/2010 14:46'!
testRemovingDoesntLeak

	| w finalized block hash object executor |
	w := WeakRegistry new: 1.
	finalized := false.
	block := [ :v | finalized := v ].
	object := Object new.
	executor := ObjectFinalizer
		receiver: block
		selector: #value:
		argument: true.
	hash := object hash.
	w add: hash.
	w add: object executor: executor.
	object := nil. "let it go"
	w remove: hash.
	Smalltalk garbageCollect. "finalize it"
	self assert: w isEmpty.
	self assert: finalized! !

!WeakRegistryTest methodsFor: 'tests' stamp: 'Igor.Stasenko 10/11/2010 13:32'!
testVileFinalizer
	" this test checks that manipulation (or accessing) weak registry 
	does not leads to deadlock, when performed from within #finalize implementation"
	
	| reg villian proc locked |
	
	reg := WeakRegistry new.
	WeakArray removeWeakDependent: reg.  "to prevent test interference with finalization process"
	
	villian := VileFinalizer new initialize: reg.  "add object with vile finalization to registry"
	
	locked := true. "be pessimistic"

	proc := [ reg finalizeValues. locked := false ] newProcess.
	villian := nil.	
	Smalltalk garbageCollect.
	
	proc resume.
	100 milliSeconds asDelay wait. "give chance for created process to run "
	
	proc isTerminated ifFalse: [ proc terminate ].

	self assert: locked == false.
! !

ClassTestCase subclass: #WeakSetTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Weak'!

!WeakSetTest methodsFor: 'testing' stamp: 'nice 5/7/2009 01:06'!
testAddIncludesSizeReclaim
	| ws o2 o3 |
	o2 := 1/2.
	o3 := '123' copy.
	ws := WeakSet new.
	ws add: o2.
	ws add: o3.
	self assert: ws size = 2.
	self assert: (ws includes: o2).
	self assert: (ws includes: o3).
	
	"inclusion test does use equality, not identity"
	self assert: (ws includes: o3 copy) description: 'WeakSet are not WeakIdentitySet'.
	
	"only one copy is added"
	ws add: o3 copy.
	self assert: ws size = 2.
	
	"reclame objects so that slots of ws are nilled out"
	o2 := o3 := nil.
	Smalltalk garbageCollect.
	self deny: (ws includes: 1/2).
	self deny: (ws includes: '123' copy).
	
	"fast #size is not updated automatically by dead object reclamation
	But there is a slowSize trying to tell the truth"
	self assert: ws slowSize = 0.! !

!WeakSetTest methodsFor: 'testing' stamp: 'ul 11/15/2010 12:15'!
testCollisions

	| ws o1 o2 o5 on remember forget |

	"create a weak set"
	ws := WeakSet new: 15.
	
	"select some fractions wanting same place in ws array"
	o1 := (2 to: 100) select: [:i | (ws scanFor: 1 / i) = 1].
	o2 := (2 to: 100) select: [:i | (ws scanFor: 1 / i) = 2].
	o5 := (2 to: 100) select: [:i | (ws scanFor: 1 / i) = 5].
	on := (2 to: 100) select: [:i | (ws scanFor: 1 / i) = (ws array size - 1)].
	
	"Add some fractions to the weak set, and remember a pointer for a few of them"
	remember := OrderedCollection new.
	forget := OrderedCollection new.
	ws add: (remember add: 1 / o1 first).
	ws add: (forget add: 1 / on second).
	ws add: (forget add: 1 / o1 second).
	ws add: (forget add: 1 / o5 second).
	ws add: (forget add: 1 / o2 second).
	ws add: (forget add: 1 / o1 third).
	ws add: (remember add: 1 / o2 first).
	ws add: (forget add: 1 / o5 third).
	ws add: (forget add: 1 / on third).
	ws add: (remember add: 1 / o2 fourth).
	ws add: (remember add: 1 / on first).
	ws add: (remember add: 1 / o5 first).
	
	"forget and reclaim all entries but those in remember"
	forget := nil.
	Smalltalk garbageCollect.
	
	remember do: [:m | self assert: (ws includes: m)].
	ws add: 1/on second.
	remember do: [:m | self assert: (ws includes: m)].
	
	ws add: (remember add: 1 / o1 fourth).
	ws add: (remember add: 1 / on fourth).
	remember remove: (ws remove: (1 / o5 first)).
	remember remove: (ws remove: (1 / on first)).
	remember remove: (ws remove: (1 / o2 first)).
	remember remove: (ws remove: (1 / o1 first)).
	remember do: [:m | self assert: (ws includes: m)].
	ws add: 1/on second.
	ws add: 1/o5 second.
	remember do: [:m | self assert: (ws includes: m)].
	! !

!WeakSetTest methodsFor: 'tests' stamp: 'ul 5/3/2010 20:13'!
testDoAfter

	| input weakSet array |
	input := (1 to: 11) collect: [ :each | each asString asSymbol ]. "Some symbols might be garbage collected without this variable"
	weakSet := WeakSet withAll: input.
	array := weakSet asArray. "Assume that the elements will have the same order as the internal array of the weakset"
	0 to: array size do: [ :index |
		| element result |
		element := array at: index ifAbsent: nil.
		result := Array new: weakSet size - index streamContents: [ :stream |
			weakSet
				do: [ :each | stream nextPut: each ]
				after: element ].
		self assert: result sort = (array allButFirst: index) sort ]! !

!WeakSetTest methodsFor: 'testing' stamp: 'nice 5/7/2009 01:20'!
testDoDontIterateOnReclaimedObjects

	| ws o1 o2 size |
	ws := WeakSet new.
	2 to: 20 do: [:i | ws add: 1 / i].
	
	o1 := 3.0.
	o2 := 4.0.
	ws add: o1; add: o2.
	
	"Reclaim memory"
	Smalltalk garbageCollect.
	
	size := 0.
	ws do: [:each |
		size := size + 1.
		self deny: each isNil].
	
	self assert: size = 2! !

!WeakSetTest methodsFor: 'testing' stamp: 'nice 5/7/2009 01:15'!
testGrowWhenNecessary
	| ws objs initialSize |
	objs := (2 to: 20) collect: [:i | 1 / i].
	ws := WeakSet new: 5.
	initialSize := ws array size.
	1 to: objs size do: [:k | ws add: (objs at: k)].
	self assert: ws array size > initialSize
		description: 'The WeakSet grown because not enough room were preallocated'.! !

!WeakSetTest methodsFor: 'tests' stamp: 'ul 11/20/2009 22:53'!
testIncludes

	| weakSet transientFakeNilObject |
	weakSet := WeakSet new.
	#(true nil 1) do: [ :each |
		self deny: (weakSet includes: each) ].
	weakSet add: true.
	self assert: (weakSet includes: true).
	weakSet remove: true.
	self deny: (weakSet includes: true).
	transientFakeNilObject := ((1 to: 1000) detect: [ :each | each asString hash - nil hash \\ weakSet capacity = 0 ]) asString. "this string will occupy the same slot as nil would"
	weakSet add: transientFakeNilObject.
	transientFakeNilObject := transientFakeNilObject copy.
	Smalltalk garbageCollect. "get rid of transientFakeNilObject"
	self deny: (weakSet includes: transientFakeNilObject).
	self deny: (weakSet includes: nil)
	
	
	! !

!WeakSetTest methodsFor: 'testing' stamp: 'nice 5/7/2009 01:11'!
testIncludesNil

	| ws |
	ws := WeakSet new.
	self deny: (ws includes: nil).
	
	"After reclamation, should not includes nil: nil counts for nothing"
	ws add: 1/2.
	ws add: 1/3.
	Smalltalk garbageCollect.
	self deny: (ws includes: nil).! !

ClassTestCase subclass: #WideStringTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!WideStringTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class String. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!WideStringTest methodsFor: 'tests - converting' stamp: 'ar 4/12/2005 17:36'!
testAsInteger
	self assert: '1796exportFixes-tkMX' asWideString asInteger = 1796.
	self assert: 'donald' asWideString asInteger isNil.
	self assert: 'abc234def567' asWideString asInteger = 234.
	self assert: '-94' asWideString asInteger = -94.
	self assert: 'foo-bar-92' asWideString asInteger = -92.

	self assert: '1796exportFixes-tkMX' asWideString asSignedInteger = 1796.
	self assert: 'donald' asWideString asSignedInteger isNil.
	self assert: 'abc234def567' asWideString asSignedInteger = 234.
	self assert: '-94' asWideString asSignedInteger = -94.
	self assert: 'foo-bar-92' asWideString asSignedInteger = -92.

	self assert: '1796exportFixes-tkMX' asWideString asUnsignedInteger = 1796.
	self assert: 'donald' asWideString asUnsignedInteger isNil.
	self assert: 'abc234def567' asWideString asUnsignedInteger = 234.
	self assert: '-94' asWideString asUnsignedInteger = 94.
	self assert: 'foo-bar-92' asWideString asUnsignedInteger = 92! !

!WideStringTest methodsFor: 'testing' stamp: 'nice 4/2/2008 23:49'!
testAtPut
	"Non regression test for http://bugs.squeak.org/view.php?id=6998"
	
	| w1 |
	w1 := WideString with: (Unicode value: 402) with: $a with: (Unicode value: 400) with: $b.
	self assert: (w1 at: 2 put: $b) = $b description: 'at:put: should return the put-object'
! !

!WideStringTest methodsFor: 'tests - beginsWith' stamp: 'nice 7/28/2007 23:29'!
testBeginsWith
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
	
	| w1 w2 |
	self assert: ('abc' beginsWith: 'ab').
	self assert: ('abc' beginsWith: 'ab' asWideString).
	self assert: ('abc' asWideString beginsWith: 'ab').
	self assert: ('abc' beginsWith: 'aX') not .
	self assert: ('abc' beginsWith: 'AB') not.
	self assert: ('abc' beginsWith: 'AB' asWideString) not .
	self assert: ('ABC' asWideString beginsWith: 'ab') not.

	w1 := WideString with: (Unicode value: 402) with: $a with: (Unicode value: 400) with: $b.
	w2 := WideString with: (Unicode value: 402).
	w1 beginsWith: w2.
! !

!WideStringTest methodsFor: 'tests - match' stamp: 'nice 7/28/2007 23:22'!
testCharactersExactlyMatching
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
	
	self assert: ('abc' charactersExactlyMatching: 'abc') = 3.
	self assert: ('abd' charactersExactlyMatching: 'abc') = 2.
	self assert: ('abc' charactersExactlyMatching: 'abc' asWideString) = 3.
	self assert: ('abd' charactersExactlyMatching: 'abc' asWideString) = 2.
	self assert: ('abc' asWideString charactersExactlyMatching: 'abc') = 3.
	self assert: ('abd' asWideString charactersExactlyMatching: 'abc') = 2.
	self assert: ('abc' asWideString charactersExactlyMatching: 'abc' asWideString) = 3.
	self assert: ('abd' asWideString charactersExactlyMatching: 'abc' asWideString)= 2.
	self assert: ('abc' charactersExactlyMatching: 'ABC') = 0.

! !

!WideStringTest methodsFor: 'tests - compare' stamp: 'nice 7/28/2007 23:19'!
testCompare
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
	
	self assert: ('abc' compare: 'abc') = 2.
	self assert: ('abc' compare: 'abd') = 1.
	self assert: ('abd' compare: 'abc') = 3.
	self assert: ('abc' compare: 'abC') = 2.
	self assert: ('abc' compare: 'abD') = 1.
	self assert: ('abd' compare: 'abC') = 3.
	self assert: ('aBc' compare: 'abC') = 2.
	self assert: ('aBc' compare: 'abD') = 1.
	self assert: ('aDd' compare: 'abC') = 3.
	
	
	self assert: ('abc' compare: 'abc' asWideString) = 2.
	self assert: ('abc' compare: 'abd' asWideString) = 1.
	self assert: ('abd' compare: 'abc' asWideString) = 3.
	self assert: ('abc' compare: 'abC' asWideString) = 2.
	self assert: ('abc' compare: 'abD' asWideString) = 1.
	self assert: ('abd' compare: 'abC' asWideString) = 3.
	self assert: ('aBc' compare: 'abC' asWideString) = 2.
	self assert: ('aBc' compare: 'abD' asWideString) = 1.
	self assert: ('aDd' compare: 'abC' asWideString) = 3.
	
	self assert: ('abc' asWideString compare: 'abc') = 2.
	self assert: ('abc' asWideString compare: 'abd') = 1.
	self assert: ('abd' asWideString compare: 'abc') = 3.
	self assert: ('abc' asWideString compare: 'abC') = 2.
	self assert: ('abc' asWideString compare: 'abD') = 1.
	self assert: ('abd' asWideString compare: 'abC') = 3.
	self assert: ('aBc' asWideString compare: 'abC') = 2.
	self assert: ('aBc' asWideString compare: 'abD') = 1.
	self assert: ('aDd' asWideString compare: 'abC') = 3.
	
	self assert: ('abc' asWideString compare: 'abc' asWideString) = 2.
	self assert: ('abc' asWideString compare: 'abd' asWideString) = 1.
	self assert: ('abd' asWideString compare: 'abc' asWideString) = 3.
	self assert: ('abc' asWideString compare: 'abC' asWideString) = 2.
	self assert: ('abc' asWideString compare: 'abD' asWideString) = 1.
	self assert: ('abd' asWideString compare: 'abC' asWideString) = 3.
	self assert: ('aBc' asWideString compare: 'abC' asWideString) = 2.
	self assert: ('aBc' asWideString compare: 'abD' asWideString) = 1.
	self assert: ('aDd' asWideString compare: 'abC' asWideString) = 3.
	
	self assert: ('abc' compare: 'abc' caseSensitive: true) = 2.
	self assert: ('abc' compare: 'abC' caseSensitive: false) = 2.
	self assert: ('abc' compare: 'abc' asWideString caseSensitive: true) = 2.
	self assert: ('abc' compare: 'abC' asWideString caseSensitive: false) = 2.
	self assert: ('abc' asWideString compare: 'abc' caseSensitive: true) = 2.
	self assert: ('abc' asWideString compare: 'abC' caseSensitive: false) = 2.
	self assert: ('abc' asWideString compare: 'abc' asWideString caseSensitive: true) = 2.
	self assert: ('abc' asWideString compare: 'abC' asWideString caseSensitive: false) = 2.! !

!WideStringTest methodsFor: 'tests - endsWith' stamp: 'nice 10/3/2009 21:18'!
testEndsWith
	"Mix of tests from http://bugs.squeak.org/view.php?id=6366
	#endsWith: was broken because using broken findSubstring
	and tests from johnmci at http://bugs.squeak.org/view.php?id=5331"

	| ws |	
	
	self assert: ('abc' endsWith: 'bc').
	self assert: ('abc' endsWith: 'bc' asWideString).
	self assert: ('abc' asWideString endsWith: 'bc').
	self assert: ('abc' endsWith: 'bX') not.
	self assert: ('abc' endsWith: 'BC') not.
	self assert: ('abc' endsWith: 'BC' asWideString) not.
	self assert: ('ABC' asWideString endsWith: 'bc') not.
	
	self assert: ('Elvis' endsWith: 'vis').
	self assert: ('Elvis' asWideString endsWith: 'vis').
	self assert: ((WideString with: (Unicode value: 530)) , 'Elvis' endsWith: 'vis').
	
	self deny: ('Elvis' endsWith: 'Vis').
	self deny: ('Elvis' asWideString endsWith: 'vIs').
	self deny: ((WideString with: (Unicode value: 530)) , 'Elvis' endsWith: 'viS').
	
	ws := 'vis' , (WideString with: (Unicode value: 530)).
	self deny: ('Elvis' endsWith: ws).
	self assert: ('Elvis' , ws endsWith: ws).
	self deny: ((WideString with: (Unicode value: 530)) , 'Elvis' endsWith: ws).

! !

!WideStringTest methodsFor: 'tests - compare' stamp: 'nice 1/18/2010 14:55'!
testEqual
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
	
	self assert: 'abc' = 'abc'.
	self assert: 'abc' = 'abc' asWideString.
	self assert: 'abc' asWideString = 'abc'.
	self assert: 'abc' asWideString = 'abc' asWideString.
	self assert: ('abc' = 'ABC') not.
	self assert: ('abc' = 'ABC' asWideString) not.
	self assert: ('abc' asWideString = 'ABC') not.
	self assert: ('abc' asWideString = 'abc' asWideString).
	self assert: (#[ 97 0 0 0 ] asString ~= 'a000' asWideString).
	self assert: ('a000' asWideString ~= #[ 97 0 0 0 ] asString).! !

!WideStringTest methodsFor: 'tests - substrings' stamp: 'nice 7/28/2007 00:46'!
testFindSubstring
	"This is related to http://bugs.squeak.org/view.php?id=6366
	finding substring in a WideString was broken because matchTable are byte-wise"
	
	| ws1 ws2 |
	
	self assert: ('abcd' findString: 'bc' startingAt: 1) = 2.
	self assert: ('abcd' asWideString findString: 'bc' startingAt: 1) = 2.
	self assert: ('abcd' findString: 'bc' asWideString startingAt: 1) = 2.
	self assert: ('abcd' asWideString findString: 'bc' asWideString startingAt: 1) = 2.
	
	ws1 := 'A' , (WideString with: (Unicode value: 530)) , 'BCD'.
	self assert: (ws1 findString: 'bc' startingAt: 1 caseSensitive: true) = 0.
	self assert: (ws1 findString: 'bc' startingAt: 1 caseSensitive: false) = 3.
	
	ws2 := (WideString with: (Unicode value: 530)) , 'b'.
	self assert: (ws1 findString: ws2 startingAt: 1 caseSensitive: true) = 0.
	self assert: (ws1 findString: ws2 startingAt: 1 caseSensitive: false) = 2.
	
	self assert: ('abc' findString: ws2 startingAt: 1 caseSensitive: true) = 0.
	self assert: ('abc' findString: ws2 startingAt: 1 caseSensitive: false) = 0.! !

!WideStringTest methodsFor: 'tests - match' stamp: 'nice 7/28/2007 23:16'!
testMatch
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
	
	self assert: ('*baz' match: 'mobaz' ).
	self assert: ('*foo#zort' match: 'afoo3zortthenfoo3zort' ).
	self assert: ('*baz' match: 'mobaz' ).
	self assert: ('*foo#zort' match: 'afoo3zortthenfoo3zort' ).
	
	self assert: ('*baz' match: 'mobaz' asWideString).
	self assert: ('*foo#zort' match: 'afoo3zortthenfoo3zort' asWideString).
	self assert: ('*baz' match: 'mobaz' asWideString).
	self assert: ('*foo#zort' match: 'afoo3zortthenfoo3zort' asWideString).
	
	self assert: ('*baz' asWideString match: 'mobaz' ).
	self assert: ('*foo#zort' asWideString match: 'afoo3zortthenfoo3zort' ).
	self assert: ('*baz' asWideString match: 'mobaz' ).
	self assert: ('*foo#zort' asWideString match: 'afoo3zortthenfoo3zort' ).
	
	self assert: ('*baz' asWideString match: 'mobaz' asWideString).
	self assert: ('*foo#zort' asWideString match: 'afoo3zortthenfoo3zort' asWideString).
	self assert: ('*baz' asWideString match: 'mobaz' asWideString).
	self assert: ('*foo#zort' asWideString match: 'afoo3zortthenfoo3zort' asWideString).! !

!WideStringTest methodsFor: 'tests - relation order' stamp: 'nice 7/28/2007 23:14'!
testRelationOrder
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"

	self assert: ('aa' < 'ab').
	self assert: ('aa' <= 'ab').
	self assert: ('aa' <= 'aa').
	self assert: ('ab' > 'aa').
	self assert: ('ab' >= 'aa').
	self assert: ('aa' >= 'aa').
	
	self assert: ('aa' < 'ab' asWideString).
	self assert: ('aa' <= 'ab' asWideString).
	self assert: ('aa' <= 'aa' asWideString).
	self assert: ('ab' > 'aa' asWideString).
	self assert: ('ab' >= 'aa' asWideString).
	self assert: ('aa' >= 'aa' asWideString).
	
	self assert: ('aa' asWideString < 'ab').
	self assert: ('aa' asWideString <= 'ab').
	self assert: ('aa' asWideString <= 'aa').
	self assert: ('ab' asWideString > 'aa').
	self assert: ('ab' asWideString >= 'aa').
	self assert: ('aa' asWideString >= 'aa').
	
	self assert: ('aa' asWideString< 'ab' asWideString).
	self assert: ('aa' asWideString<= 'ab' asWideString).
	self assert: ('aa' asWideString<= 'aa' asWideString).
	self assert: ('ab' asWideString> 'aa' asWideString).
	self assert: ('ab' asWideString >= 'aa' asWideString).
	self assert: ('aa' asWideString>= 'aa' asWideString).! !

!WideStringTest methodsFor: 'tests - relation order' stamp: 'nice 7/28/2007 23:21'!
testRelationOrderWithCase
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
		
	self assert: ('ABC' caseInsensitiveLessOrEqual: 'abc').
	self assert: ('ABC' caseInsensitiveLessOrEqual: 'abd').
	self assert: ('ABD' caseInsensitiveLessOrEqual: 'abc') not.
	self assert: ('ABC' caseInsensitiveLessOrEqual: 'abc' asWideString).
	self assert: ('ABC' caseInsensitiveLessOrEqual: 'abd' asWideString).
	self assert: ('ABD' caseInsensitiveLessOrEqual: 'abc' asWideString) not.
	self assert: ('ABC' asWideString caseInsensitiveLessOrEqual: 'abc').
	self assert: ('ABC' asWideString caseInsensitiveLessOrEqual: 'abd').
	self assert: ('ABD' asWideString caseInsensitiveLessOrEqual: 'abc') not.
	self assert: ('ABC' asWideString caseInsensitiveLessOrEqual: 'abc' asWideString).
	self assert: ('ABC' asWideString caseInsensitiveLessOrEqual: 'abd' asWideString).
	self assert: ('ABD' asWideString caseInsensitiveLessOrEqual: 'abc' asWideString) not.
	
	
	self assert: ('abc' caseSensitiveLessOrEqual: 'abc').
	self assert: ('abc' caseSensitiveLessOrEqual: 'abd').
	self assert: ('abd' caseSensitiveLessOrEqual: 'abc') not.
	self assert: ('abc' caseSensitiveLessOrEqual: 'abc' asWideString).
	self assert: ('abc' caseSensitiveLessOrEqual: 'abd' asWideString).
	self assert: ('abd' caseSensitiveLessOrEqual: 'abc' asWideString) not.
	self assert: ('abc' asWideString caseSensitiveLessOrEqual: 'abc').
	self assert: ('abc' asWideString caseSensitiveLessOrEqual: 'abd').
	self assert: ('abd' asWideString caseSensitiveLessOrEqual: 'abc') not.
	self assert: ('abc' caseSensitiveLessOrEqual: 'ABC') not.
	! !

!WideStringTest methodsFor: 'tests - compare' stamp: 'nice 1/18/2010 14:56'!
testSameAs
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"

	self assert: ('abc' sameAs: 'aBc' asWideString).
	self assert: ('aBc' asWideString sameAs: 'abc').
	self assert: (#[ 97 0 0 0 ] asString sameAs: 'Abcd' asWideString) not.
	self assert: ('a000' asWideString sameAs: #[ 97 0 0 0 ] asString) not.
	! !

!WideStringTest methodsFor: 'tests - substrings' stamp: 'nice 3/23/2007 01:52'!
testSubstrings
	"this is related to http://bugs.squeak.org/view.php?id=6367"
	
	| w1 w2 |
	w1 := WideString with: 401 asCharacter with: $a with: 402 asCharacter with: $b.
	w2 := WideString with: 403 asCharacter with: 404 asCharacter.

	self assert: w1 substrings first = w1.
	self assert: (w1 , ' ' , w2) substrings size = 2.
	self assert: (w1 , ' ' , w2) substrings last = w2.! !

ClassTestCase subclass: #WriteStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Streams'!

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'dc 2/14/2007 14:22'!
testCr
	"self debug: #testCr"
	
	| stream |
	stream := WriteStream on: 'stream'.
	stream cr.
	self assert: stream last = Character cr.! !

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'ul 12/2/2009 05:53'!
testCrLf
	"self debug: #testCrLf"
	
	| stream |
	stream := WriteStream on: 'stream'.
	stream crlf.
	self assert: (stream contents last: 2) = String crlf.! !

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'dc 2/14/2007 14:26'!
testCrTab
	"self debug: #testCrTab"
	
	| stream |
	stream := WriteStream on: 'stream'.
	stream crtab.
	self assert: (stream contents last: 2) = (String with: Character cr with: Character tab)! !

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'dc 2/14/2007 14:28'!
testCrTabs
	"self debug: #testCrTabs"
	
	| stream |
	stream := WriteStream on: 'stream'.
	stream crtab: 2.
	self assert: (stream contents last: 3) = (String with: Character cr with: Character tab with: Character tab)! !

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'dc 2/11/2007 14:14'!
testEnsureASpace
	"self debug: #testEnsureASpace"
	| stream |
	stream := WriteStream on: String new.
	stream nextPutAll: 'this is a test'.
	stream ensureASpace.
	stream nextPutAll: 'for WriteStreamTest'.
	self assert: stream contents = 'this is a test for WriteStreamTest'.
	
	"Manually put a space and verify there are no 2 consecutive spaces"
	stream := WriteStream on: String new.
	stream nextPutAll: 'this is a test '.
	stream ensureASpace.
	stream nextPutAll: 'for WriteStreamTest'.
	self assert: stream contents = 'this is a test for WriteStreamTest'.! !

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'dc 2/11/2007 14:17'!
testEnsureASpace2
	"self debug: #testEnsureASpace2"
	| stream |
	stream := WriteStream on: String new.
	stream ensureASpace.
	self assert: stream contents = ' '.
	! !

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'dc 2/14/2007 14:29'!
testEnsureNoSpace
	"self debug: #testEnsureNoSpace"
	
	| stream |

	stream := WriteStream with: 'stream'.
	stream ensureNoSpace.
	self assert: stream contents = 'stream'.
	
	stream := WriteStream with: 'stream '.
	stream ensureNoSpace.
	self assert: stream contents = 'stream'.
	
	stream := WriteStream with: ' '.
	stream ensureNoSpace.
	self assert: stream contents = ''.! !

!WriteStreamTest methodsFor: 'tests - instance creation' stamp: 'dc 2/11/2007 13:59'!
testInstanciationUsingOn
	"self debug: #testInstanciationUsingOn"
	| stream |
	stream := WriteStream on: #(1 2).
	stream nextPut: 3.
	self assert: stream contents = #(3)! !

!WriteStreamTest methodsFor: 'tests - instance creation' stamp: 'dc 2/11/2007 13:59'!
testInstanciationUsingWith
	"self debug: #testInstanciationUsingWith"
	| stream |
	stream := WriteStream with: #(1 2).
	stream nextPut: 3.
	self assert: stream contents = #(1 2 3)! !

!WriteStreamTest methodsFor: 'tests - testing' stamp: 'dc 2/27/2007 16:20'!
testIsEmpty
	| stream |
	stream := WriteStream on: String new.
	self assert: stream isEmpty.
	stream nextPut: $a.
	self deny: stream isEmpty.
	stream reset.
	self deny: stream isEmpty.! !

!WriteStreamTest methodsFor: 'tests - testing' stamp: 'edc 4/12/2007 07:01'!
testIsEmpty2
    self assert: (WriteStream on: (String new: 100)) isEmpty! !

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'ul 12/2/2009 05:52'!
testLf
	"self debug: #testLf"
	
	| stream |
	stream := WriteStream on: 'stream'.
	stream lf.
	self assert: stream last = Character lf.! !

!WriteStreamTest methodsFor: 'tests - instance creation' stamp: 'sd 6/5/2005 09:24'!
testNew

	self should: [WriteStream new] raise: Error. ! !

!WriteStreamTest methodsFor: 'tests - accessing' stamp: 'dc 2/14/2007 14:12'!
testNextPut
	"self debug: #testNextPut"
	| stream |
	stream := WriteStream on: String new.
	stream
		nextPut: $t;
		nextPut: $e;
		nextPut: $s;
		nextPut: $t.
	self assert: stream contents = 'test'! !

!WriteStreamTest methodsFor: 'tests - accessing' stamp: 'dc 2/14/2007 14:13'!
testNextPut2
	"self debug: #testNextPut2"
	| stream |
	stream := WriteStream with: 'test'.
	stream nextPut: $s.
	self assert: stream contents = 'tests'! !

!WriteStreamTest methodsFor: 'tests - accessing' stamp: 'dc 2/14/2007 14:13'!
testNextPutAll
	"self debug: #testNextPutAll"
	| stream |
	stream := WriteStream on: String new.
	stream
		nextPutAll: #($t $e $s $t).
	self assert: stream contents = 'test'! !

!WriteStreamTest methodsFor: 'tests - instance creation' stamp: 'dc 2/24/2007 18:15'!
testNextPutAllDifferentFromNextPuts
	"self debug: #testNextPutAllDifferentFromNextPuts"
	
	"When a stream is created on a collection, it tries to keep using that collection instead of copying. See thread with title 'Very strange bug on Streams and probably compiler' (Feb 14 2007) on the squeak-dev mailing list."
	
	"nextPutAll verifies the size of the parameter and directly grows the underlying collection of the required size."
	|string stream|
	
	string := String withAll: 'z'.
	stream := WriteStream on: string.
	stream nextPutAll: 'abc'.
	self assert: string = 'z'. "string hasn't been modified because #nextPutAll: detects that 'abc' is bigger than the underlying collection. Thus, it starts by creating a new collection and doesn't modify our variable."
	
	string := String withAll: 'z'.
	stream := WriteStream on: string.
	stream nextPut: $a; nextPut: $b; nextPut: $c.
	self assert: string = 'a'. "The first #nextPut: has no problem and replaces $z by $a in the string. Others will detect that string is too small."! !

!WriteStreamTest methodsFor: 'tests - positioning' stamp: 'dc 2/14/2007 14:48'!
testPosition
	"self debug: #testPosition"
	
	| stream |
	stream := WriteStream with: 'an elephant'.
	stream position: 6.
	self assert: stream contents = 'an ele'.

	stream nextPutAll: 'vator'.
	stream assert: stream contents = 'an elevator'! !

!WriteStreamTest methodsFor: 'tests - positioning' stamp: 'dc 2/14/2007 14:52'!
testPosition2
	"self debug: #testPosition2"
	
	| stream |

	stream := WriteStream with: ''.
	self should: [stream position: 2] raise: Error.
	self should: [stream position: -2] raise: Error.

	stream := WriteStream with: 'a test'.
	self shouldnt: [stream position: 2] raise: Error.
	self should: [stream position: 7] raise: Error.
	self should: [stream position: -2] raise: Error.! !

!WriteStreamTest methodsFor: 'tests - positioning' stamp: 'dc 2/24/2007 17:55'!
testReset
	"self debug: #testReset"
	|stream stream2|
	stream := WriteStream with: 'a test ' copy.
	stream reset.
	stream nextPutAll: 'to test'.
	self assert: stream contents = 'to test'.

	stream2 := WriteStream with: 'a test ' copy.
	stream2 nextPutAll: 'to test'.
	self assert: stream2 contents = 'a test to test'! !

!WriteStreamTest methodsFor: 'tests - accessing' stamp: 'sd 6/5/2005 09:24'!
testSetToEnd

	| string stream |
	string := 'hello'.
	stream := WriteStream with: ''.
	stream nextPutAll: string.
	self assert: stream position = string size.
	stream setToEnd.
	self assert: stream position = string size.
	self assert: stream contents = string! !

!WriteStreamTest methodsFor: 'tests - accessing' stamp: 'dc 2/14/2007 14:19'!
testSize
	"self debug: #testSize"

	| string streamEmpty streamFull |
	string := 'a string'.
	streamEmpty := WriteStream on: string.
	streamFull := WriteStream with: 'a string'.
	
	self assert: streamEmpty size = 0.
	self assert: streamFull size = 8.
	
	streamEmpty nextPut: $..
	streamFull nextPut: $..
	self assert: streamEmpty size = 1.
	self assert: streamFull size = (string size + 1).! !

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'dc 2/14/2007 14:39'!
testSpace
	"self debug: #testSpace"
	
	| stream |
	stream := WriteStream on: 'stream'.
	stream space.
	self assert: stream last = Character space.! !

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'dc 2/14/2007 14:40'!
testSpaces
	"self debug: #testSpaces"
	
	| stream |
	stream := WriteStream on: 'stream'.
	stream space: 3.
	self assert: (stream contents last: 3) = '   '! !

!WriteStreamTest methodsFor: 'tests - instance creation' stamp: 'dc 2/24/2007 18:05'!
testStreamUseGivenCollection
	"self debug: #testStreamUseGivenCollection"
	
	"When a stream is created on a collection, it tries to keep using that collection instead of copying. See thread with title 'Very strange bug on Streams and probably compiler' (Feb 14 2007) on the squeak-dev mailing list."
	
	|string stream|
	
	string := String withAll: 'erased'.
	stream := WriteStream on: string.
	self assert: string = 'erased'.
	
	stream nextPutAll: 'test'.
	self assert: string = 'tested'. "Begining of 'erased' has been replaced by 'test'".! !

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'dc 2/14/2007 14:42'!
testTab
	"self debug: #testTab"
	
	| stream |
	stream := WriteStream on: 'stream'.
	stream tab.
	self assert: (stream contents last) = Character tab! !

!WriteStreamTest methodsFor: 'tests - character writing' stamp: 'dc 2/14/2007 14:41'!
testTabs
	"self debug: #testTabs"
	
	| stream |
	stream := WriteStream on: 'stream'.
	stream tab: 3.
	self assert: (stream contents last: 3) = (String with: Character tab with: Character tab with: Character tab)! !

HashAndEqualsTestCase subclass: #TextAlignmentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextAlignmentTest methodsFor: 'initialize-release' stamp: 'mjr 8/20/2003 18:55'!
setUp
	super setUp.
	prototypes add: TextAlignment centered;
		 add: TextAlignment justified;
		 add: TextAlignment leftFlush;
		 add: TextAlignment rightFlush ! !

HashAndEqualsTestCase subclass: #TextEmphasisTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextEmphasisTest methodsFor: 'initialize-release' stamp: 'mjr 8/20/2003 18:55'!
setUp
	super setUp.
	prototypes add: TextEmphasis bold;
		 add: TextEmphasis italic;
		 add: TextEmphasis narrow;
		 add: TextEmphasis normal;
		 add: TextEmphasis struckOut;
		 add: TextEmphasis underlined ! !

!TextEmphasisTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:03'!
testAppendString
	"tests the Text>>prepend: method when appending a String " 
	"self run: # testAppendString"

	| receiver argument result expectedResult |

	receiver := 'xxx' asText  
		addAttribute: TextEmphasis bold from: 1 to: 3.
	argument := 'yyy'.
	expectedResult := 'xxxyyy' asText 
		addAttribute: TextEmphasis bold from: 1 to: 3.
	result := receiver append: argument.
	self assert: (result == receiver). 
	self assert: (result string = expectedResult string).
	self assert: (result runs = expectedResult runs)! !

!TextEmphasisTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:04'!
testAppendText
	"tests the Text>>prepend: method when appending a Text " 
	| receiver argument result expectedResult |

	receiver := 'xxx' asText  
		addAttribute: TextEmphasis bold from: 1 to: 3.
	argument := 'yyy' asText
		addAttribute: TextEmphasis italic from: 1 to: 3.		.
	expectedResult := 'xxxyyy' asText 
		addAttribute: TextEmphasis bold from: 1 to: 3;
		addAttribute: TextEmphasis italic from: 4 to: 6.
	result := receiver append: argument.
	self assert: (result == receiver). 
	self assert: (result string = expectedResult string).
	self assert: (result runs = expectedResult runs)

	
	! !

!TextEmphasisTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:04'!
testPrependString
	"tests the Text>>prepend: method when prepending a String " 
	| receiver argument result expectedResult |

	receiver := 'xxx' asText  
		addAttribute: TextEmphasis bold from: 1 to: 3.
	argument := 'yyy'.
	expectedResult := 'yyyxxx' asText 
		addAttribute: TextEmphasis bold from: 4 to: 6.
	result := receiver prepend: argument.
	self assert: (result == receiver). 
	self assert: (result string = expectedResult string).
	self assert: (result runs = expectedResult runs)

	! !

!TextEmphasisTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:04'!
testPrependText
	"tests the Text>>prepend: method when prepending Text " 
	| receiver argument result expectedResult |

	receiver := 'xxx' asText  
		addAttribute: TextEmphasis bold from: 1 to: 3.
	argument := 'yyy' asText 
		addAttribute: TextEmphasis italic from: 1 to: 3.
	expectedResult := 'yyyxxx' asText 
		addAttribute: TextEmphasis italic from: 1 to: 3;
		addAttribute: TextEmphasis bold from: 4 to: 6.
	result := receiver prepend: argument.
	self assert: (result == receiver). 
	self assert: (result string = expectedResult string).
	self assert: (result runs = expectedResult runs)! !

HashAndEqualsTestCase subclass: #TextFontChangeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextFontChangeTest methodsFor: 'initialize-release' stamp: 'mjr 8/20/2003 18:55'!
setUp
	"create the prototypes for testing"
	super setUp.
	prototypes add: TextFontChange defaultFontChange.
	prototypes add: TextFontChange font1.
	prototypes add: TextFontChange font2.
	prototypes add: TextFontChange font3.
	prototypes add: TextFontChange font4.
	prototypes
		add: (TextFontChange fontNumber: 6) ! !

!TextFontChangeTest methodsFor: 'tests' stamp: 'mjr 8/17/2003 20:29'!
testEquality
	"Check that different instances of the same TextFontChange are equal"
	self assert: TextFontChange defaultFontChange = TextFontChange defaultFontChange.
	self assert: TextFontChange font1 = TextFontChange font1.
	self assert: TextFontChange font2 = TextFontChange font2.
	self assert: TextFontChange font3 = TextFontChange font3.
	self assert: TextFontChange font4 = TextFontChange font4.
	self assert: (TextFontChange fontNumber: 6)
			= (TextFontChange fontNumber: 6)! !

!TextFontChangeTest methodsFor: 'tests' stamp: 'stephaneducasse 2/4/2006 20:10'!
testHash
	"test that different instances of the same TextFontChange hash to the 
	same value"
	| hashes hash |
	hashes := OrderedCollection new.
	1
		to: 100
		do: [:i | hashes add: TextFontChange defaultFontChange hash].
	hash := hashes at: 1.
	2
		to: 100
		do: [:i | self assert: (hashes at: i)
					= hash]! !

HashAndEqualsTestCase subclass: #TextFontReferenceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextFontReferenceTest methodsFor: 'tests' stamp: 'mjr 8/20/2003 18:55'!
setUp
	super setUp.
	prototypes
		add: (TextFontReference
				toFont: (StrikeFont familyName: 'NewYork' size: 15)) ! !

HashAndEqualsTestCase subclass: #TextKernTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextKernTest methodsFor: 'tests' stamp: 'mjr 8/20/2003 18:55'!
setUp
	super setUp.
	prototypes
		add: (TextKern kern: 1) ! !

Object subclass: #VileFinalizer
	instanceVariableNames: 'weakRegistry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Weak'!

!VileFinalizer methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 10/11/2010 13:29'!
finalize
	weakRegistry remove: self ifAbsent: [].! !

!VileFinalizer methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 10/11/2010 13:36'!
initialize: wr
	weakRegistry := wr.
	weakRegistry add: self.! !

TestCase subclass: #BagTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

!BagTest methodsFor: 'basic tests' stamp: 'sd 3/21/2006 22:28'!
testAdd
	"self run: #testAdd"
	"self debug: #testAdd"

	| aBag |
	aBag := Bag new.
	aBag add: 'a'.
	aBag add: 'b'.
	
	self assert: aBag size = 2.
	aBag add: 'a'.
	self assert: aBag size = 3.
	self assert: (aBag occurrencesOf: 'a') = 2
	
! !

!BagTest methodsFor: 'basic tests' stamp: 'sd 3/21/2006 22:28'!
testAddWithOccurrences
	"self debug:#testAddWithOccurrences"
	
	| aBag |
 	aBag := Bag new.	
	aBag add: 'a' withOccurrences: 3.
	self assert: (aBag size = 3).
	
	
	
	
		
! !

!BagTest methodsFor: 'basic tests' stamp: 'TJ 3/8/2006 08:42'!
testAsBag

	| aBag |
 
	aBag := Bag new.	
	
	self assert: aBag asBag = aBag.! !

!BagTest methodsFor: 'basic tests' stamp: 'sd 3/21/2006 22:29'!
testAsSet

	| aBag aSet |
	aBag := Bag new.	
	aBag add:'a' withOccurrences: 4.
	aBag add:'b' withOccurrences: 2.
	aSet := aBag asSet.
	self assert: aSet size = 2.
	self assert: (aSet occurrencesOf: 'a') = 1 
	! !

!BagTest methodsFor: 'basic tests' stamp: 'sd 3/21/2006 22:30'!
testCopy
	"self run: #testCopy"
	
	| aBag newBag |
	aBag := Bag new.
	aBag add:'a' withOccurrences: 4.
	aBag add:'b' withOccurrences: 2.
	newBag := aBag copy.
	self assert: newBag = newBag.
	self assert: newBag asSet size = 2.! !

!BagTest methodsFor: 'tests' stamp: 'EP 2/28/2006 09:56'!
testCreation
	"self run: #testCreation"
	"self debug: #testCreation"
		
	| bag |
	bag := Bag new.
	self assert: (bag size) = 0.
	self assert: (bag isEmpty).
	
! !

!BagTest methodsFor: 'tests' stamp: 'EP 2/28/2006 10:05'!
testCumulativeCounts
	"self run: #testCumulativeCounts"
	"self debug: #testCumulativeCounts"
		
	| bag cumulativeCounts |
	bag := Bag new.
	bag add: '1' withOccurrences: 50.
	bag add: '2' withOccurrences: 40.
	bag add: '3' withOccurrences: 10.
	
	cumulativeCounts := bag cumulativeCounts.
	
	self assert: cumulativeCounts size = 3.
	self assert: cumulativeCounts first = (50 -> '1').
	self assert: cumulativeCounts second = (90 -> '2').
	self assert: cumulativeCounts third = (100 -> '3').
! !

!BagTest methodsFor: 'tests' stamp: 'EP 3/8/2006 08:39'!
testEqual
	"(self run: #testEqual)"
	"(self debug: #testEqual)"
	| bag1 bag2 |
	bag1 := Bag new.
	bag2 := Bag new.
	self assert: bag1 = bag2.
	bag1 add: #a;
		 add: #b.
	bag2 add: #a;
		 add: #a.
	self deny: bag1 = bag2.
	self assert: bag1 = bag1.
	bag1 add: #a.
	bag2 add: #b.
	self assert: bag1 = bag2.
	bag1 add: #c.
	self deny: bag1 = bag2.
	bag2 add: #c.
	self assert: bag1 = bag2! !

!BagTest methodsFor: 'basic tests' stamp: 'sd 3/21/2006 22:32'!
testOccurrencesOf
	"self debug: #testOccurrencesOf"

	| aBag |
 	aBag := Bag new.	
	aBag add: 'a' withOccurrences: 3.
	aBag add: 'b'.
	aBag add: 'b'.
	aBag add: 'b'.
	aBag add: 'b'.	
	self assert: (aBag occurrencesOf:'a') = 3.
	self assert: (aBag occurrencesOf:'b') = 4.
	self assert: (aBag occurrencesOf:'c') = 0.
	self assert: (aBag occurrencesOf: nil) =0.
	aBag add: nil.
	self assert: (aBag occurrencesOf: nil) =1.
	! !

!BagTest methodsFor: 'tests' stamp: 'EP 2/28/2006 09:57'!
testRemove
	"self run: #testRemove"
	"self debug: #testRemove"
	
	| bag item |
	item := 'test item'.
	bag := Bag new.
	
	bag add: item.
	self assert: (bag size) = 1.
	bag remove: item.
	self assert: bag isEmpty.
	
	bag add: item withOccurrences: 2.
	bag remove: item.
	bag remove: item.
	self assert: (bag size) = 0.
	
	self should: [bag remove: item.] raise: Error.! !

!BagTest methodsFor: 'tests' stamp: 'nice 9/14/2009 21:05'!
testRemoveAll
	"Allows one to remove all elements of a collection" 
	
	| c1 c2 s2 |
	c1 := #(10 9 8 7 5 4 4 2) asBag.
	c2 := c1 copy.
	s2 := c2 size.
	
	c1 removeAll.
	
	self assert: c1 size = 0.
	self assert: c2 size = s2 description: 'the copy has not been modified'.! !

!BagTest methodsFor: 'tests' stamp: 'EP 2/28/2006 09:48'!
testSortedCounts
	"self run: #testSortedCounts"
	"self debug: #testSortedCounts"
	
	| bag sortedCounts|
	bag := Bag new.
	bag add: '1' withOccurrences: 10.
	bag add: '2' withOccurrences: 1.
	bag add: '3' withOccurrences: 5.
	
	sortedCounts := bag sortedCounts.
	self assert: sortedCounts size = 3.
	
	self assert: sortedCounts first = (10->'1').
	self assert: sortedCounts second =  (5->'3').
	self assert: sortedCounts third =  (1->'2').	! !

!BagTest methodsFor: 'tests' stamp: 'EP 2/28/2006 09:48'!
testSortedElements
	"self run: #testSortedElements"
	"self debug: #testSortedElements"
	
	| bag sortedElements|
	bag := Bag new.
	bag add: '2' withOccurrences: 1.
	bag add: '1' withOccurrences: 10.
	bag add: '3' withOccurrences: 5.
	
	sortedElements := bag sortedElements.
	
	self assert: sortedElements size = 3.
	
	self assert: sortedElements first = ('1'->10).
	self assert: sortedElements second =  ('2'->1).
	self assert: sortedElements third =  ('3'->5).
	! !

TestCase subclass: #Base64MimeConverterTest
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Streams'!

!Base64MimeConverterTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Base64MimeConverter. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!Base64MimeConverterTest methodsFor: 'initialize-release' stamp: 'jcg 9/1/2009 00:21'!
setUp
	message := 'Hi There!!' readStream.
! !

!Base64MimeConverterTest methodsFor: 'tests' stamp: 'jcg 9/1/2009 00:23'!
testBase64Encoded
	| encoded |
	encoded := (Base64MimeConverter mimeEncode: message) contents.
	self assert: encoded = 'Hi There!!' base64Encoded.
	! !

!Base64MimeConverterTest methodsFor: 'tests' stamp: 'ar 3/9/2010 22:19'!
testMimeEncodeDecode

	| encoded |
	encoded := Base64MimeConverter mimeEncode: message.
	self assert: (encoded contents = 'SGkgVGhlcmUh').
     self assert: ((Base64MimeConverter mimeDecodeToChars: encoded) contents = message contents).

	"Encoding should start from the beginning of the stream."
	message reset.
	message skip: 2.
	encoded := Base64MimeConverter mimeEncode: message.
	self assert: (encoded contents = 'SGkgVGhlcmUh').
	
	"Encoding should start from the current position of the stream."
	message reset.
	message skip: 2.
	encoded := Base64MimeConverter mimeEncodeContinue: message.
	self assert: (encoded contents = 'IFRoZXJlIQ==').! !

!Base64MimeConverterTest methodsFor: 'tests' stamp: 'ar 3/9/2010 22:21'!
testMimeEncodeDecodeMultiLine

	| encoded |
	encoded := (Base64MimeConverter mimeEncode: (String new: 100 withAll: $a) readStream) contents.
	self assert: encoded = 
'YWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFh
YWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYQ=='.

	encoded := (Base64MimeConverter mimeEncode: (String new: 100 withAll: $a) readStream multiLine: false) contents.
	self assert: encoded = 
'YWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYWFhYQ=='.
! !

!Base64MimeConverterTest methodsFor: 'tests' stamp: 'jcg 9/1/2009 00:38'!
testOnByteArray
	self assert: ('Hi There!!' base64Encoded = 'Hi There!!' asByteArray base64Encoded)! !

TestCase subclass: #CharacterSetTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Support'!

!CharacterSetTest commentStamp: 'nice 11/20/2007 00:35' prior: 0!
CharacterSetTest holds tests for CharacterSet!

!CharacterSetTest methodsFor: 'testing' stamp: 'nice 11/20/2007 00:38'!
testCopy
    | theOriginal theCopy |
    theOriginal := CharacterSet newFrom: 'abc'.
    theCopy := theOriginal copy.
    theCopy remove: $a.
    ^self should: [theOriginal includes: $a] description: 'Changing the copy should not change the original'.! !

TestCase subclass: #CollectionTest
	instanceVariableNames: 'empty nonEmpty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Abstract'!

!CollectionTest commentStamp: '<historical>' prior: 0!
A TestCase is a Command representing the future running of a test case. Create one with the class method #selector: aSymbol, passing the name of the method to be run when the test case runs.

When you discover a new fixture, subclass TestCase, declare instance variables for the objects in the fixture, override #setUp to initialize the variables, and possibly override# tearDown to deallocate any external resources allocated in #setUp.

When you are writing a test case method, send #assert: aBoolean when you want to check for an expected value. For example, you might say "self assert: socket isOpen" to test whether or not a socket is open at a point in a test.!

!CollectionTest methodsFor: 'initialize-release' stamp: 'st 10/7/2004 16:23'!
setUp
	empty := Set new.
	nonEmpty := OrderedCollection with: #x! !

!CollectionTest methodsFor: 'testing' stamp: 'fbs 1/14/2005 10:19'!
testAsCommaString
	{OrderedCollection new. Set new.} do:
		[ :coll |
		self assert: coll asCommaString = ''.

		coll add: 1.
		self assert: coll asCommaString = '1'.

		coll add: 2; add: 3.
		self assert: coll asCommaString = '1, 2, 3'].! !

!CollectionTest methodsFor: 'testing' stamp: 'fbs 1/14/2005 10:19'!
testAsCommaStringAnd
	{OrderedCollection new. Set new.} do:
		[ :coll |
		self assert: coll asCommaStringAnd = ''.

		coll add: 1.
		self assert: coll asCommaStringAnd = '1'.

		coll add: 2; add: 3.
		self assert: coll asCommaStringAnd = '1, 2 and 3'].! !

!CollectionTest methodsFor: 'testing' stamp: 'nice 12/27/2009 03:11'!
testAsStringOnDelimiter
	| delim |
	delim := ', '.
	{OrderedCollection new. Set new.} do:
		[ :coll | | oneItemStream emptyStream multiItemStream |
		emptyStream := ReadWriteStream on: ''.
		coll asStringOn: emptyStream delimiter: delim.
		self assert: emptyStream contents = ''.

		coll add: 1.
		oneItemStream := ReadWriteStream on: ''.
		coll asStringOn: oneItemStream delimiter: delim.
		self assert: oneItemStream contents = '1'.

		coll add: 2; add: 3.
		multiItemStream := ReadWriteStream on: ''.
		coll asStringOn: multiItemStream delimiter: ', '.
		self assert: multiItemStream contents = '1, 2, 3'.]! !

!CollectionTest methodsFor: 'testing' stamp: 'nice 12/27/2009 03:11'!
testAsStringOnDelimiterLast
	| delim last |
	delim := ', '.
	last := ' & '.
	{OrderedCollection new. Set new.} do:
		[ :coll | | multiItemStream emptyStream oneItemStream |
		emptyStream := ReadWriteStream on: ''.
		coll asStringOn: emptyStream delimiter: delim last: last.
		self assert: emptyStream contents = ''.

		coll add: 1.
		oneItemStream := ReadWriteStream on: ''.
		coll asStringOn: oneItemStream delimiter: delim last: last.
		self assert: oneItemStream contents = '1'.

		coll add: 2; add: 3.
		multiItemStream := ReadWriteStream on: ''.
		coll asStringOn: multiItemStream delimiter: ', ' last: last.
		self assert: multiItemStream contents = '1, 2 & 3'.]! !

!CollectionTest methodsFor: 'tests' stamp: 'ar 11/21/2009 01:00'!
testIfEmpty

	nonEmpty ifEmpty: [self assert: false].
	self assert: (empty ifEmpty: [self]) == self.
	self assert: (nonEmpty ifEmpty: [self]) == nonEmpty.
! !

!CollectionTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:12'!
testIfEmptyifNotEmpty

	self assert: (empty ifEmpty: [true] ifNotEmpty: [false]).
	self assert: (nonEmpty ifEmpty: [false] ifNotEmpty: [true]).
	self assert: (nonEmpty ifEmpty: [false] ifNotEmpty: [:s | s first = #x])! !

!CollectionTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:12'!
testIfEmptyifNotEmptyDo

	self assert: (empty ifEmpty: [true] ifNotEmptyDo: [:s | false]).
	self assert: (nonEmpty ifEmpty: [false] ifNotEmptyDo: [:s | s first = #x])! !

!CollectionTest methodsFor: 'tests' stamp: 'ar 11/21/2009 00:58'!
testIfNotEmpty

	empty ifNotEmpty: [self assert: false].
	self assert: (nonEmpty ifNotEmpty: [self]) == self.
	self assert: (nonEmpty ifNotEmpty: [:s | s first]) = #x.
	self assert: (empty ifNotEmpty: [self]) == empty.
! !

!CollectionTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:13'!
testIfNotEmptyDo

	empty ifNotEmptyDo: [:s | self assert: false].
	self assert: (nonEmpty ifNotEmptyDo: [:s | s first]) = #x
! !

!CollectionTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:13'!
testIfNotEmptyDoifNotEmpty

	self assert: (empty ifNotEmptyDo: [:s | false] ifEmpty: [true]).
	self assert: (nonEmpty ifNotEmptyDo: [:s | s first = #x] ifEmpty: [false])! !

!CollectionTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:13'!
testIfNotEmptyifEmpty

	self assert: (empty ifEmpty: [true] ifNotEmpty: [false]).
	self assert: (nonEmpty ifEmpty: [false] ifNotEmpty: [true]).
	self assert: (nonEmpty ifEmpty: [false] ifNotEmpty: [:s | s first = #x])! !

!CollectionTest methodsFor: 'tests' stamp: 'klc 3/7/2010 18:31'!
testMinMax
	self assert: #(5 6 3 -3 2 1 0 4) minMax = #(-3 6).
	self assert: #($f $g $a $g $u $z $s $t) minMax = #($a $z).
	self assert: #(2834.83 -293482.28 99283 23 959323) minMax = #(-293482.28 959323).! !

!CollectionTest methodsFor: 'testing' stamp: 'nice 12/27/2009 03:11'!
testPrintOnDelimiter
	| delim |
	delim := ', '.
	{OrderedCollection new. Set new.} do:
		[ :coll | | emptyStream oneItemStream multiItemStream |
		emptyStream := ReadWriteStream on: ''.
		coll printOn: emptyStream delimiter: delim.
		self assert: emptyStream contents = ''.

		coll add: 1.
		oneItemStream := ReadWriteStream on: ''.
		coll printOn: oneItemStream delimiter: delim.
		self assert: oneItemStream contents = '1'.

		coll add: 2; add: 3.
		multiItemStream := ReadWriteStream on: ''.
		coll printOn: multiItemStream delimiter: ', '.
		self assert: multiItemStream contents = '1'', ''2'', ''3'.]! !

!CollectionTest methodsFor: 'testing' stamp: 'nice 12/27/2009 03:11'!
testPrintOnDelimiterLast
	| delim last |
	delim := ', '.
	last := ' & '.
	{OrderedCollection new. Set new.} do:
		[ :coll | | emptyStream oneItemStream multiItemStream |
		emptyStream := ReadWriteStream on: ''.
		coll printOn: emptyStream delimiter: delim last: last.
		self assert: emptyStream contents = ''.

		coll add: 1.
		oneItemStream := ReadWriteStream on: ''.
		coll printOn: oneItemStream delimiter: delim last: last.
		self assert: oneItemStream contents = '1'.

		coll add: 2; add: 3.
		multiItemStream := ReadWriteStream on: ''.
		coll printOn: multiItemStream delimiter: ', ' last: last.
		self assert: multiItemStream contents = '1'', ''2'' & ''3'.]! !

!CollectionTest methodsFor: 'testing' stamp: 'wbk 7/26/2007 12:41'!
testPrintingArrayWithMetaclass
	self assert: {Array class} printString = '{Array class}'! !

!CollectionTest methodsFor: 'tests' stamp: 'ar 5/4/2010 08:52'!
testReduce
	"self run: #testReduce"

	| array set bag |
	array := Array with: 1 with: 2 with: 3.
	self assert: (array reduce:[:a :b| a + b]) = 6.
	set := array asSet.
	self assert: (set reduce:[:a :b| a + b]) = 6.

	array := Array with: Color red with: Color green with: Color blue.
	self assert: (array reduce:[:a :b| a + b]) = Color white.
	set := array asSet.
	self assert: (set reduce:[:a :b| a + b]) = Color white.

	array := Array with: 1 with: 1 with: 1.
	self assert: (array reduce:[:a :b| a + b]) = 3.
	bag := array asBag.
	self assert: (bag reduce:[:a :b| a + b]) = 3.
! !

TestCase subclass: #DictionaryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

DictionaryTest class
	instanceVariableNames: 'testToto pt1'!

DictionaryTest class
	instanceVariableNames: 'testToto pt1'!

!DictionaryTest methodsFor: 'tests' stamp: 'zz 12/7/2005 19:59'!
testAdd
	
	| dict |
	dict := Dictionary new.
	dict add: #a -> 1.
	dict add: #b -> 2.
	self assert: (dict at: #a) = 1.
	self assert: (dict at: #b) = 2! !

!DictionaryTest methodsFor: 'tests' stamp: 'ar 8/24/2010 20:16'!
testAddAll
	
	| dict1 dict2 |
	dict1 := Dictionary new.
	dict1 at: #a put:1 ; at: #b put: 2. 
	dict2 := Dictionary new.
	dict2 at: #a put: 3 ; at: #c put: 4.
	dict1 addAll: dict2.
	self assert: (dict1 at: #a) = 3.
	self assert: (dict1 at: #b) = 2.
	self assert: (dict1 at: #c) = 4.

	dict1 := Dictionary new.
	dict1 at: #a put:1 ; at: #b put: 2. 
	dict1 addAll: {#a -> 3. #c -> 4}.
	self assert: (dict1 at: #a) = 3.
	self assert: (dict1 at: #b) = 2.
	self assert: (dict1 at: #c) = 4.! !

!DictionaryTest methodsFor: 'association tests' stamp: 'NDCC 3/8/2006 08:14'!
testAddAssociation
	"self run:#testAddAssociation"
	"self debug:#testAddAssociation"
	
	| dict |
	dict := Dictionary new.
	dict at: #a put: 1.
	dict at: #b put: 2.
	self assert: (dict at: #a) = 1.
	self assert: (dict at: #b) = 2.
	
	dict at: #a put: 10.
	dict at: #c put: 2.
	
	self assert: (dict at: #a) = 10.
	self assert: (dict at: #b) = 2.
	self assert: (dict at: #c) = 2
	
	! !

!DictionaryTest methodsFor: 'tests' stamp: 'nice 10/6/2009 14:21'!
testAsSet
	"Non regression test for http://bugs.squeak.org/view.php?id=7258"
	
	| aDictionary aSet assoc0 assoc1 |
	
	"Create a dictionary"
	aDictionary := Dictionary new.
	
	"Convert it to a Set"
	aSet := aDictionary asSet.

	"Add two associations to it"	
	assoc0 := #first -> 0.
	assoc1 := #first -> 1.
	aSet add: assoc0 copy; add: assoc1.
	
	"Check if the two associations were added (that should happen if they are different)"
	self
		assert: (assoc0 copy ~= assoc1) ==> (aSet size > 1)
		description:
  			'When adding two different elements, the set size should be greater than one'! !

!DictionaryTest methodsFor: 'association tests' stamp: 'ul 11/15/2010 11:38'!
testAssociationsSelect
	| answer d |
	d := Dictionary new.
	d at: (Array with: #hello with: #world)
	  put: #fooBar.
	d at: Smalltalk put: #'Smalltalk is the key'.
	d at: #Smalltalk put: Smalltalk.
	answer := d
				associationsSelect: [:assoc | assoc key == #Smalltalk
						and: [assoc value == Smalltalk]].
	self
		should: [answer isKindOf: Dictionary].
	self
		should: [answer size = 1].
	self
		should: [(answer at: #Smalltalk)
				== Smalltalk].
	answer := d
				associationsSelect: [:assoc | assoc key == #NoSuchKey
						and: [assoc value == #NoSuchValue]].
	self
		should: [answer isKindOf: Dictionary].
	self
		should: [answer size = 0]! !

!DictionaryTest methodsFor: 'basic tests' stamp: 'ar 3/9/2010 22:10'!
testAtError
	"self run: #testAtError"
	
	| dict |
	dict := Dictionary new.
	dict at: #a put: 666.
	self shouldnt: [ dict at: #a ] raise: KeyNotFound.
	self should: [ dict at: #b ] raise: KeyNotFound.
	
	! !

!DictionaryTest methodsFor: 'basic tests' stamp: 'NDCC 3/1/2006 14:27'!
testAtIfAbsent
	"self run: #testAtIfAbsent"
	
	| dict |
	dict := Dictionary new.
	dict at: #a put: 666.
	
	self assert: (dict at: #a ifAbsent: [nil]) = 666.
	
	self assert: (dict at: #b ifAbsent: [nil]) isNil.

	! !

!DictionaryTest methodsFor: 'basic tests' stamp: 'ar 3/4/2010 20:31'!
testAtIfPresentIfAbsent
	"Test at:ifPresent:ifAbsent:"
	| dict present absent |
	dict := Dictionary new.
	present := absent := false.
	dict at: #foo ifPresent:[:v| present := true] ifAbsent:[absent := true].
	self deny: present.
	self assert: absent.

	dict at: #foo put: #bar.
	present := absent := false.
	dict at: #foo ifPresent:[:v| present := true] ifAbsent:[absent := true].
	self assert: present.
	self deny: absent.

	present := absent := false.
	dict at: #foo ifPresent:[:v| present := true. nil] ifAbsent:[absent := true].
	self assert: present.
	self deny: absent.
! !

!DictionaryTest methodsFor: 'implementation tests' stamp: 'stephaneducasse 9/18/2005 10:48'!
testAtNil
	"(self run: #testAtNil)"
	"nil is a valid key in squeak. In VW nil is not a valid key"
	"Ansi 1.9 p, 168
    		5.7.2.5 Message: at: key put: newElement
    		Synopsis
    			Store newElement at key in the receiver. Answer newElement.
    		Definition: <abstractDictionary>
    		If lookup succeeds for key, then newElement replaces the element previously stored at key.
    		Otherwise, the newElement is stored at the new key. In either case, subsequent successful
    		lookups for key will answer newElement.  Answer newElement.

    		The result is undefined if the key is nil.

		This clearly indicates that different smalltalks where doing different assumptions."
	
	
	
	| dict1  |
	dict1 := Dictionary new.
	self shouldnt: [ dict1 at: nil put: #none] raise: Error.
	self assert: (dict1 at: nil) = #none. 
	! !

!DictionaryTest methodsFor: 'basic tests' stamp: 'NDCC 3/8/2006 09:28'!
testAtPut
	"self run: #testAtPut"
	"self debug: #testAtPut"
	
	| adictionary |
	adictionary := Dictionary new.
	adictionary at: #a put: 3.
	self assert: (adictionary at: #a) = 3.
	adictionary at: #a put: 3.
	adictionary at: #a put: 4.
	self assert: (adictionary at: #a) = 4.
	adictionary at: nil put: 666.
	self assert: (adictionary at: nil) = 666! !

!DictionaryTest methodsFor: 'basic tests' stamp: 'NDCC 3/8/2006 09:53'!
testAtPutNil
	"self run: #testAtPut"
	"self debug: #testAtPut"
	
	| dict |
	dict := Dictionary new.
	dict at: nil put: 1.
	self assert: (dict at: nil) = 1.
	dict at: #a put: nil.
	self assert: (dict at: #a) = nil.
	dict at: nil put: nil.
	self assert: (dict at: nil) = nil.
	
	
	! !

!DictionaryTest methodsFor: 'testing' stamp: 'ar 6/13/2008 00:14'!
testCollect
	"Ensure that Dictionary>>collect: answers a dictionary not something else"
	| dict expected result |
	dict := Dictionary newFromPairs:{
		#first.		1.
		#second.	2.
		#third.		3.
		#fourth.	4.
		#fifth.		5.
	}.
	result := dict collect:[:each| each asWords].
	expected := Dictionary newFromPairs:{
		#first.		'one'.
		#second.	'two'.
		#third.		'three'.
		#fourth.	'four'.
		#fifth.		'five'.
	}.
	self assert: result = expected.! !

!DictionaryTest methodsFor: 'tests' stamp: 'zz 12/7/2005 19:57'!
testComma
	
	| dict1 dict2 dict3 |
	dict1 := Dictionary new.
	dict1 at: #a put:1 ; at: #b put: 2. 
	dict2 := Dictionary new.
	dict2 at: #a put: 3 ; at: #c put: 4.
	dict3 := dict1, dict2.
	self assert: (dict3 at: #a) = 3.
	self assert: (dict3 at: #b) = 2.
	self assert: (dict3 at: #c) = 4.! !

!DictionaryTest methodsFor: 'keys and value tests' stamp: 'NDCC 3/8/2006 08:16'!
testDictionaryConcatenation
	"self run: #testDictionaryConcatenation"
	
	
	| dict1 dict2 dict3 |
	dict1 := Dictionary new.
	dict1 at: #a put: 'Nicolas' ; at: #b put: 'Damien'. 
	
	dict2 := Dictionary new.
	dict2 at: #a put: 'Christophe' ; at: #c put: 'Anthony'.
	dict3 := dict1, dict2.
	
	self assert: (dict3 at: #a) = 'Christophe'.
	self assert: (dict3 at: #b) = 'Damien'.
	self assert: (dict3 at: #c) = 'Anthony'.
	

	! !

!DictionaryTest methodsFor: 'association tests' stamp: 'NDCC 3/8/2006 09:20'!
testIncludesAssociation
	"self run:#testIncludesAssociation"
	
	| dict |
	dict := Dictionary new.
	dict at: #a put: 1.
	dict at: #b put: 2.
	self assert: (dict includesAssociation: (#a -> 1)).
	self assert: (dict includesAssociation: (#b -> 2)).
	
	! !

!DictionaryTest methodsFor: 'association tests' stamp: 'ndCollectionsTests-Unordered 3/16/2006 10:25'!
testIncludesAssociationNoValue
	"self run:#testIncludesAssociationNoValue"
	"self debug:#testIncludesAssociationNoValue"
	
	| dict a1 a3 |
	a1 := Association key: #Italie.
	a3 := Association key: #France value: 'Paris'.
	
	self assert: (a1 key = #Italie).
	self assert: (a1 value isNil).
	
	dict := Dictionary new.
	dict add: a1.
	dict add: a3.
	self assert: (dict includesKey: #France).
	self assert: (dict includesKey: #Italie).
	self assert: (dict at: #Italie) isNil.
	self assert: (dict at: #France) = 'Paris'

	
	
	! !

!DictionaryTest methodsFor: 'basic tests' stamp: 'ndCollectionsTests-Unordered 3/16/2006 10:24'!
testIncludesKey
	"self run:#testIncludesKey"
	"self debug:#testIncludesKey"
	
	| dict a1 a2 a3 |
	a1 := Association key: 'Italie'.
	a2 := Association new.
	a3 := Association key: 'France' value: 'Paris'.
	
	dict := Dictionary new.
	dict add: a1 .
	dict add: a2.
	dict add: a3.
	self assert: (dict includesKey: #France).
	self assert: (dict includesKey: 'France').
	self assert: (dict includesKey: #Italie).
	self assert: (dict includesKey: nil).	
		
	self assert: (dict at: 'France' ) = 'Paris'.
! !

!DictionaryTest methodsFor: 'keys and value tests' stamp: 'ndCollectionsTests-Unordered 3/16/2006 10:24'!
testKeyAtValue
	"self run: #testKeyAtValue"
	"self debug: #testKeyAtValue"
	
	| dict |
	dict := Dictionary new.
	dict at: #a put: 1.
	dict at: #b put: 2.
	dict at: #c put: 1.
	
	self assert: (dict keyAtValue: 2) = #b.
	self assert: (dict keyAtValue: 1) = #c.
	"ugly may be a bug, why not having a set #a and #c"
	
	self should: [dict keyAtValue: 0] raise: Error
	
	
! !

!DictionaryTest methodsFor: 'keys and value tests' stamp: 'ndCollectionsTests-Unordered 3/16/2006 10:23'!
testKeys

	"self run:#testKeys "
	
	| a1 a2  dict | 
	a1 := Association key: 'France' value: 'Paris'.
	a2 := Association key: 'Italie' value: 'Rome'.
	dict := Dictionary new.
	dict add: a1.
	dict add: a2.
	 		
	self assert: (dict keys size) = 2.
	
	self assert: (dict keys includes: #France)
	
	


	
	! !

!DictionaryTest methodsFor: 'keys and value tests' stamp: 'NDCC 3/8/2006 09:13'!
testKeysDo
	"self run: #testKeysDo"
	"self debug: #testKeysDo"
	
	| dict res |
	dict := Dictionary new.
	
	dict at: #a put: 33.
	dict at: #b put: 66.
	
	res := OrderedCollection new.
	dict keysDo: [ :each | res add: each].
	
	self assert: res asSet = #(a b) asSet.


	
	! !

!DictionaryTest methodsFor: 'basic tests' stamp: 'ael 7/30/2012 14:30'!
testNewFromIsolation
	"self run:#testNewFromIsolation"
	"self debug:#testNewFromIsolation"
	
	| dict1 dict2 |
	dict1 := Dictionary new.
	dict1 at: #one put: 'foo'; at: #two put: 0@0.
	dict2 := Dictionary newFrom: dict1.
	dict2 at: #one put: 'bar'.

	self assert: (dict1 at: #one) = 'foo'.
! !

!DictionaryTest methodsFor: 'tests' stamp: 'ar 11/12/2009 21:58'!
testNilHashCollision
	"Ensures that fixCollisionsFrom: does the right thing in the presence of a nil key"
	| dict key |
	dict := Dictionary new.
	key := nil hash. "any key with same hash as nil"
	dict at: key hash put: 1.
	dict at: nil put: 2.
	self assert: (dict includesKey: nil).
	dict removeKey: key.
	self assert: (dict includesKey: nil).
! !

!DictionaryTest methodsFor: 'basic tests' stamp: 'NDCC 3/8/2006 09:41'!
testOccurrencesOf
	"self run:#testOccurrencesOf"
	
	| dict |
	dict := Dictionary new.
	dict at: #a put: 1.
	dict at: #b put: 2.
	dict at: #c put: 1.
	dict at: #d put: 3.
	dict at: nil put: nil.
	dict at: #z put: nil.
	
	
	self assert: (dict occurrencesOf: 1 ) = 2.
	self assert: (dict occurrencesOf: nil ) = 2.
	
	
	
	! !

!DictionaryTest methodsFor: 'tests' stamp: 'stephaneducasse 9/18/2005 10:45'!
testPseudo
	"(self run: #testPseudo)"
	"true and false are valid keys"
	
	| dict1  |
	dict1 := Dictionary new.
	self shouldnt: [dict1 at: true put: #true] raise: Error.
	self assert: (dict1 at: true) = #true.
		
	self shouldnt: [dict1 at: false put: #false] raise: Error.
	self assert: (dict1 at: false) = #false.! !

!DictionaryTest methodsFor: 'implementation tests' stamp: 'ndCollectionsTests-Unordered 3/16/2006 10:29'!
testPseudoVariablesAreValidKeys
	"(self run: #testPseudoVariablesAreValidKeys)"
	"true and false are valid keys"
	
	| dict1  |
	dict1 := Dictionary new.
	self shouldnt: [dict1 at: true put: #true] raise: Error.
	self assert: (dict1 at: true) = #true.
		
	self shouldnt: [dict1 at: false put: #false] raise: Error.
	self assert: (dict1 at: false) = #false.! !

!DictionaryTest methodsFor: 'testing' stamp: 'ar 6/13/2008 00:13'!
testReject
	"Ensure that Dictionary>>reject: answers a dictionary not something else"
	| dict expected result |
	dict := Dictionary newFromPairs:{
		#first.		1.
		#second.	2.
		#third.		3.
		#fourth.	4.
		#fifth.		5.
	}.
	result := dict reject:[:each| each odd].
	expected := Dictionary newFromPairs:{
		#second.	2.
		#fourth.	4.
	}.
	self assert: result = expected.! !

!DictionaryTest methodsFor: 'tests' stamp: 'nice 9/14/2009 21:07'!
testRemoveAll
	"Allows one to remove all elements of a collection" 
	
	| dict1 dict2 s2 |
	dict1 := Dictionary new.
	dict1 at: #a put:1 ; at: #b put: 2. 
	dict2 := dict1 copy.
	s2 := dict2 size.
	
	dict1 removeAll.
	
	self assert: dict1 size = 0.
	self assert: dict2 size = s2 description: 'the copy has not been modified'.! !

!DictionaryTest methodsFor: 'keys and value tests' stamp: 'NDCC 3/8/2006 08:56'!
testRemoveKey
		"self run:#testRemoveKey "

	| dict | 
	dict := Dictionary new.
	dict at: #a put: 1.
	dict at: #b put: 2.
	 
	self assert: (dict keys size) = 2.
	dict removeKey: #a.
	self assert: dict keys size  = 1.

	self should: [dict at: #a] raise: Error.
	self assert: (dict at: #b) = 2


	
	! !

!DictionaryTest methodsFor: 'testing' stamp: 'ul 9/30/2009 14:29'!
testSelect
	"Ensure that Dictionary>>select: answers a dictionary not something else"
	| dict expected result |
	dict := Dictionary newFromPairs:{
		#first.		1.
		#second.	2.
		#third.		3.
		#fourth.	4.
		#fifth.		5.
	}.
	result := dict select:[:each| each odd].
	expected := Dictionary newFromPairs:{
		#first.		1.
		#third.		3.
		#fifth.		5.
	}.
	self assert: result = expected.
	result at: #first put: 0.
	self assert: (dict at: #first) = 1 "No side effects"! !

!DictionaryTest methodsFor: 'keys and value tests' stamp: 'ndCollectionsTests-Unordered 3/16/2006 10:23'!
testValues
	"self run:#testValues "
	
	| a1 a2 a3 dict | 
	a1 := Association key: 'France' value: 'Paris'.
	a2 := Association key: 'Italie' value: 'Rome'.
	dict := Dictionary new.
	dict add: a1.
	dict add: a2.
	 
	self assert: (dict values size ) = 2.
	self assert: (dict values includes: 'Paris').
	
	a3 := Association new.
	dict add: a3.
	self assert: (dict values size ) = 3.
	self assert: (dict values includes: nil).
	
	
	
	
	
	


	
	! !

!DictionaryTest methodsFor: 'tests' stamp: 'ar 8/24/2010 20:17'!
testWithAll
	
	| dict1 dict2 |
	dict1 := Dictionary withAll: {#a -> 1. #b -> 2}.
	self assert: (dict1 at: #a) = 1.
	self assert: (dict1 at: #b) = 2.

	dict2 := Dictionary withAll: dict1.
	self assert: (dict2 at: #a) = 1.
	self assert: (dict2 at: #b) = 2.
! !

TestCase subclass: #GeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Streams'!

!GeneratorTest class methodsFor: 'accessing' stamp: 'lr 2/10/2010 08:34'!
packageNamesUnderTest
	^ #('Generator')! !

!GeneratorTest methodsFor: 'generators' stamp: 'lr 1/8/2009 16:29'!
fibonacciSequence
	"Yields an infinite sequence of fibonacci numbers."
	
	^ Generator on: [ :generator |
		| a b |
		a := 0. b := 1.
		[ a := b + (b := a).
		  generator yield: a ]
			repeat ]! !

!GeneratorTest methodsFor: 'generators' stamp: 'lr 1/8/2009 15:49'!
numbersBetween: aStartInteger and: aStopInteger
	"Yields the nubmers between aStartInteger and aStopInteger."
	
	^ Generator on: [ :generator |
		aStartInteger to: aStopInteger 
			do: [ :value | generator yield: value ] ]! !

!GeneratorTest methodsFor: 'testing' stamp: 'lr 1/8/2009 16:33'!
testAtEnd
	| generator |
	generator := self numbersBetween: 1 and: 3.
	self deny: generator atEnd.
	generator next.
	self deny: generator atEnd.
	generator next.
	self deny: generator atEnd.
	generator next.
	self assert: generator atEnd! !

!GeneratorTest methodsFor: 'testing' stamp: 'lr 2/10/2010 09:18'!
testClose
	| generator doEnsure notEnsure |
	doEnsure := notEnsure := 0.
	[ generator := Generator on: [ :g |
		[ g yield: 1; yield: 2 ]
			ensure: [ doEnsure := doEnsure + 1 ] ].
	self assert: doEnsure = 0; assert: notEnsure = 0.
	self assert: generator peek = 1.
	self assert: doEnsure = 0; assert: notEnsure = 0.
	generator close.
	self assert: doEnsure = 1; assert: notEnsure = 0 ]
		ensure: [ notEnsure := notEnsure + 1 ].
	self assert: doEnsure = 1; assert: notEnsure = 1! !

!GeneratorTest methodsFor: 'testing' stamp: 'lr 4/26/2009 11:51'!
testContents
	| generator |
	generator := self numbersBetween: 1 and: 3.
	self assert: generator contents = #(1 2 3)! !

!GeneratorTest methodsFor: 'testing' stamp: 'lr 1/8/2009 16:45'!
testEmpty
	| generator |
	generator := Generator on: [ :g | ].
	self assert: generator atEnd.
	self assert: generator peek isNil.
	self assert: generator next isNil! !

!GeneratorTest methodsFor: 'testing' stamp: 'lr 2/10/2010 09:02'!
testEnsure
	| generator |
	generator := Generator on: [ :g |
		[ g yield: 1; yield: 2 ]
			ensure: [ g yield: 3 ] ].
	self assert: generator upToEnd asArray = #( 1 2 3 )! !

!GeneratorTest methodsFor: 'testing' stamp: 'ar 2/10/2010 21:01'!
testErrorPropagation
	"Ensure that errors in the generator block are properly propagated"
	| generator |
	self shouldnt:[generator := Generator on: [ :g | g yield: 1. g error: 'yo']] raise: Error.
	self should:[generator next] raise: Error.
! !

!GeneratorTest methodsFor: 'testing' stamp: 'lr 1/8/2009 16:50'!
testFibonacci
	| generator |
	generator := self fibonacciSequence.
	self assert: (generator next: 10) asArray = #( 1 1 2 3 5 8 13 21 34 55 )! !

!GeneratorTest methodsFor: 'testing' stamp: 'lr 1/8/2009 16:33'!
testNext
	| generator |
	generator := self numbersBetween: 1 and: 3.
	self assert: generator next = 1.
	self assert: generator next = 2.
	self assert: generator next = 3.
	self assert: generator next isNil! !

!GeneratorTest methodsFor: 'testing' stamp: 'lr 1/8/2009 16:45'!
testPeek
	| generator |
	generator := self numbersBetween: 1 and: 3.
	self assert: generator peek = 1.
	self assert: generator peek = 1.
	generator next.
	self assert: generator peek = 2! !

!GeneratorTest methodsFor: 'testing' stamp: 'lr 1/8/2009 16:35'!
testReset
	| generator |
	generator := self numbersBetween: 1 and: 3.
	self assert: generator next = 1.
	self assert: generator next = 2.
	generator reset.
	self assert: generator next = 1.
	self assert: generator next = 2.
	self assert: generator next = 3.
	self assert: generator next = nil.
	generator reset.
	self assert: generator next = 1! !

!GeneratorTest methodsFor: 'testing' stamp: 'ar 2/10/2010 21:03'!
testResetUnwind
	"Just like close, just using reset"
	| generator doEnsure notEnsure |
	doEnsure := notEnsure := 0.
	[ generator := Generator on: [ :g |
		[ g yield: 1; yield: 2 ]
			ensure: [ doEnsure := doEnsure + 1 ] ].
	self assert: doEnsure = 0; assert: notEnsure = 0.
	self assert: generator peek = 1.
	self assert: doEnsure = 0; assert: notEnsure = 0.
	generator reset.
	self assert: doEnsure = 1; assert: notEnsure = 0 ]
		ensure: [ notEnsure := notEnsure + 1 ].
	self assert: doEnsure = 1; assert: notEnsure = 1! !

!GeneratorTest methodsFor: 'testing' stamp: 'lr 1/8/2009 16:46'!
testSimple
	| generator |
	generator := Generator on: [ :g | g yield: 1; yield: 2 ].
	self assert: generator upToEnd asArray = #( 1 2 )! !

TestCase subclass: #HashedCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Abstract'!

!HashedCollectionTest methodsFor: 'test - class - sizing' stamp: 'ul 11/25/2009 03:55'!
testGoodPrimeAtLeast

	| primes |
	primes := HashedCollection goodPrimes.
	primes do: [ :each |
		self assert: (HashedCollection goodPrimeAtLeast: each) = each.
		self assert: (HashedCollection goodPrimeAtLeast: each - 1) = each.
		self assert: (HashedCollection goodPrimeAtLeast: each + 1) > each ].
	(0 to: 1000) do: [ :each |
		| goodPrime |
		goodPrime := HashedCollection goodPrimeAtLeast: each.
		self assert: goodPrime isPrime.
		self assert: goodPrime >= each.
		self assert: (primes detect: [ :ea | ea >= each ] ifNone: [ each + 1 ]) = goodPrime ].
	self assert: (HashedCollection goodPrimeAtLeast: primes last + 1) odd! !

!HashedCollectionTest methodsFor: 'test - class - sizing' stamp: 'ul 3/19/2010 14:20'!
testGoodPrimes

	self testPrimes: HashedCollection goodPrimes! !

!HashedCollectionTest methodsFor: 'test - class - sizing' stamp: 'ul 4/7/2010 00:18'!
testPrimes: primes

	| badPrimes |
	badPrimes := #(3 5 71 139 479 5861 277421). "These primes are less than the hashMultiply constant (1664525) and 1664525 \\ prime is close to 0 (mod prime). The following snippet reproduces these numbers: 
	| hashMultiplyConstant |
	hashMultiplyConstant := 1 hashMultiply.
	(Integer primesUpTo: hashMultiplyConstant) select: [ :each |
		| remainder |
		remainder := hashMultiplyConstant \\ each.
		remainder <= 1 or: [ remainder + 1 = each ] ]."
	self assert: primes isSorted.
	primes do: [ :each |
		self assert: each isPrime.
		self deny: (each > 2069 and: [ badPrimes includes: each ]) ].
	self assert: (
		primes select: [ :p |
			| result |
			result := false.
			p > 2069 ifTrue: [
			1 to: 8 do: [ :k |
				1 to: 32 do: [ :a |
					(p gcd: (256 raisedTo: k) + a) = 1 ifFalse: [
						result := true ].
					(p gcd: (256 raisedTo: k) - a) = 1 ifFalse: [
						result := true ] ] ] ].
			result ]) isEmpty.! !

!HashedCollectionTest methodsFor: 'test - class - sizing' stamp: 'ul 11/25/2009 04:23'!
testSizeFor

	#(
		0 1 5 10 30 57 89 578 1234 1912 2411 2591 5627 7849
		10999 61356 68602 73189 79868 86789 239984 239985 501175 661865 841558
		9669391 15207345 19827345 23469817 27858432 65223175 106650047
		157687845 190892299 234947087 264782114 269617510 270995400 
		392236508 456647275 468699153 606865011 606997796 617927086
		837938371 880614337 989233852 1000473294 1060034095) do: [ :each |
			| size |
			size := HashedCollection sizeFor: each.
			size <= HashedCollection goodPrimes last ifTrue: [
				self assert: size isPrime ].
			self assert: size odd.
			self assert: size * 3 // 4 >= each ]! !

TestCase subclass: #IdentityBagTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

!IdentityBagTest methodsFor: 'tests' stamp: 'EP 3/8/2006 09:50'!
testIdentity
	"self run:#testIdentity"
	"self debug:#testIdentity"
	| bag identityBag aString anOtherString |
	
	aString := 'hello'.
	anOtherString := aString copy.
	
	self assert: (aString = anOtherString).
	self assert: (aString == anOtherString) not.

	bag := Bag new.
	bag add: aString.
	bag add: aString.
	bag add: anOtherString.
	self assert: (bag occurrencesOf: aString) = 3.
	self assert: (bag occurrencesOf: anOtherString) = 3.
	
	identityBag := IdentityBag new.
	identityBag add: aString.
	identityBag add: aString.
	identityBag add: anOtherString.
	
	self assert: (identityBag occurrencesOf: aString) = 2.
	self assert: (identityBag occurrencesOf: anOtherString) = 1.



! !

TestCase subclass: #RWBinaryOrTextStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Streams'!

!RWBinaryOrTextStreamTest methodsFor: 'testing' stamp: 'ul 9/16/2012 21:01'!
expectedFailures

	^#(
		testExisting "This is more like a request for new behavior than a bug."
	)! !

!RWBinaryOrTextStreamTest methodsFor: 'tests' stamp: 'topa 5/23/2011 13:49'!
testExisiting

	| string stream |
	
	string := 'hello'.
	
	stream := RWBinaryOrTextStream on: string.
	self assert: stream contents = string.

	stream := RWBinaryOrTextStream on: string.
	stream setToEnd.
	self assert: stream position = string size.
! !

!RWBinaryOrTextStreamTest methodsFor: 'tests' stamp: 'ar 9/1/2009 00:43'!
testNextPutAll
	
	| stream |
	stream := RWBinaryOrTextStream on: String new.
	stream nextPutAll: 'hello'.
	self assert: stream contents = 'hello'.

	stream := RWBinaryOrTextStream on: String new.
	stream nextPutAll: 'hello' asArray.
	self assert: stream contents = 'hello'.

	stream := RWBinaryOrTextStream on: String new.
	stream nextPutAll: 'hello' asByteArray.
	self assert: stream contents = 'hello'
! !

!RWBinaryOrTextStreamTest methodsFor: 'tests' stamp: 'ar 9/1/2009 00:45'!
testUpTo

	| stream |
	stream := RWBinaryOrTextStream on: String new.
	stream nextPutAll: 'world'; reset.
	self assert: (stream upTo: $r) = 'wo'.
	self assert: stream next = $l.
	
	stream := RWBinaryOrTextStream on: String new.
	stream nextPutAll: 'hello'; reset.
	self assert: (stream upTo: $x) = 'hello'.
	self assert: stream atEnd.
	
	stream := RWBinaryOrTextStream on: String new.
	stream binary.
	stream nextPutAll: 'world'; reset.
	self assert: (stream upTo: $r asciiValue) = 'wo' asByteArray.
	self assert: stream next = $l asciiValue.
	
	stream := RWBinaryOrTextStream on: String new.
	stream binary.
	stream nextPutAll: 'hello'; reset.
	self assert: (stream upTo: $x asciiValue) = 'hello' asByteArray.
	self assert: stream atEnd.
! !

!RWBinaryOrTextStreamTest methodsFor: 'tests' stamp: 'ar 9/1/2009 00:46'!
testUpToEnd
	
	| stream |
	stream := RWBinaryOrTextStream on: String new.
	stream nextPutAll: 'world'; reset.
	self assert: stream upToEnd = 'world'.
	self assert: stream atEnd.

	self assert: stream upToEnd = ''.
	self assert: stream atEnd.
	
	stream reset.
	stream upTo: $r.
	self assert: stream upToEnd = 'ld'.
	self assert: stream atEnd.
	
	stream := RWBinaryOrTextStream on: String new.
	stream binary.
	stream nextPutAll: 'world'; reset.
	self assert: stream upToEnd = 'world' asByteArray.
	self assert: stream atEnd.

	self assert: stream upToEnd = ByteArray new.
	self assert: stream atEnd.
	
	stream reset.
	stream upTo: $r asciiValue.
	self assert: stream upToEnd = 'ld' asByteArray.
	self assert: stream atEnd! !

TestCase subclass: #RunArrayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Arrayed'!

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/15/2007 10:25'!
testAt
	"self debug: #testAt"
	| array |
	array := RunArray new: 5 withAll: 2.
	self assert: (array at: 3) = 2.
	
	array at: 3 put: 5.
	self assert: (array at: 3) = 5
! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/15/2007 10:26'!
testAtPut
	"self debug: #testAtPut"
	| array |
	array := RunArray new: 5 withAll: 2.
	
	array at: 3 put: 5.
	self assert: array = #(2 2 5 2 2).
	
	array at: 1 put: 1.
	self assert: array = #(1 2 5 2 2).! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/15/2007 10:29'!
testAtPut2
	"self debug: #testAtPut2"
	| array |
	array := RunArray new: 5 withAll: 2.
	
	self should: [array at: 0 put: 5] raise: Error.
	self should: [array at: 6 put: 5] raise: Error.
	self should: [array at: $b put: 5] raise: Error.! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/12/2007 09:49'!
testFirst
	"self debug: #testFirst"
	| array |
	array := RunArray new: 5 withAll: 2.
	self assert: array first = 2.
	
	array := #($a $b $c $d) as: RunArray.
	self assert: array first = $a.! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/12/2007 09:49'!
testLast
	"self debug: #testLast"
	| array |
	array := RunArray new: 5 withAll: 2.
	self assert: array last = 2.
	
	array := #($a $b $c $d) as: RunArray.
	self assert: array last = $d.! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'dc 2/12/2007 09:22'!
testNew
	"self debug: #testNew"
	| array |
	array := RunArray new.
	self assert: array size = 0.! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'dc 2/12/2007 09:24'!
testNewFrom
	"self debug: #testNewFrom"
	| array |
	array := RunArray newFrom: #($a $b $b $b $b $c $c $a).
	self assert: array size = 8.
	self assert: array = #($a $b $b $b $b $c $c $a).! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'dc 2/12/2007 09:15'!
testNewWithAll
	"self debug: #testNewWithAll"
	| array |
	array := RunArray new: 5 withAll: 2.
	self assert: array size = 5.
	self assert: array = #(2 2 2 2 2)! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'dc 5/24/2007 10:56'!
testNewWithSize
	|array|
	array := RunArray new: 5.
	self assert: array size = 5.
	1 to: 5 do: [:index | self assert: (array at: index) isNil]! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/12/2007 09:58'!
testRunLengthAt
	"self debug: #testRunLengthAt"
	| array |
	array := #($a $b $b $c $c $c $d $d) as: RunArray.
	self assert: (array runLengthAt: 1) = 1.
	self assert: (array runLengthAt: 2) = 2.
	self assert: (array runLengthAt: 3) = 1.
	self assert: (array runLengthAt: 4) = 3.
	self assert: (array runLengthAt: 5) = 2.
	self assert: (array runLengthAt: 6) = 1.
	self assert: (array runLengthAt: 7) = 2.
	self assert: (array runLengthAt: 8) = 1.! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'dc 2/12/2007 09:30'!
testRunsValues
	"self debug: #testRunsValues"
	| array |
	array := RunArray runs: #(1 4 2 1) values: #($a $b $c $a).
	self assert: array size = 8.
	self assert: array = #($a $b $b $b $b $c $c $a).! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'sd 6/5/2005 09:13'!
testScanFromANSICompatibility
	"self run: #testScanFromANSICompatibility"

	RunArray scanFrom: (ReadStream on: '()f1dNumber new;;').
	RunArray scanFrom: (ReadStream on: '()a1death;;').
	RunArray scanFrom: (ReadStream on: '()F1death;;').! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'ar 2/24/2010 18:05'!
testScanFromTrailer
	"This test documents an ancient bug in RunArray class>>scanFrom:."

	self assert: (RunArray scanFrom: '(8 4 5),b,!!' readStream) = 
		(RunArray runs: #(8 4 5) values: { {}. {TextEmphasis bold}. {} }).! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/12/2007 10:12'!
testWithStartStopAndValueDo
	"self debug: #testWithStartStopAndValueDo"
	| array elements startStops |
	array := #($a $b $b $c $c $c $d $d) as: RunArray.
	elements := OrderedCollection new.
	startStops := OrderedCollection new.
	array withStartStopAndValueDo: [:start :stop :value | elements add: value. startStops add: start->stop].
	
	self assert: elements asArray = #($a $b  $c  $d).
	self assert: startStops asArray = {1->1 . 2->3 . 4->6 . 7->8}! !

TestCase subclass: #SetWithNilTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

!SetWithNilTest methodsFor: 'tests' stamp: 'ar 2/1/2010 21:43'!
runSetWithNilTestOf: newSet
	"Run the common tests for the given set class"
	self shouldnt:[newSet value add: nil] raise: Error.
	self shouldnt:[newSet value addAll: #(nil nil nil)] raise: Error.

	self assert: (newSet value add: nil; yourself) size = 1.
	self assert: (newSet value addAll: #(nil nil nil); yourself) size = 1.

	self assert: ((newSet value add: nil; yourself) includes: nil).
	self assert: ((newSet value addAll: #(nil nil nil); yourself) includes: nil).

	self assert: (newSet value add: nil; yourself) anyOne = nil.
	self assert: ((newSet value add: nil; yourself) remove: nil) == nil.
	self assert: ((newSet value add: nil; yourself) remove: nil; yourself) isEmpty.

	self assert: (newSet value addAll: #(1 nil foo); yourself) size = 3.
	self assert: ((newSet value addAll: #(1 nil foo); yourself) remove: nil; yourself) size = 2.

	self assert: ((newSet value add: nil; yourself) collect:[:x| x]) = (newSet value add: nil; yourself).
	self assert: ((newSet value add: nil; yourself) collect:[:x| x] as: Array) = #(nil).

	self deny: ((newSet value) includes: nil).
	self deny: ((newSet value add: 3; yourself) includes: nil).
	self deny: ((newSet value add: 3; remove: 3; yourself) includes: nil).
! !

!SetWithNilTest methodsFor: 'tests' stamp: 'ar 2/1/2010 21:24'!
testIdentitySetWithNil
	self runSetWithNilTestOf: [IdentitySet new]! !

!SetWithNilTest methodsFor: 'tests' stamp: 'ar 2/1/2010 21:28'!
testKeyedIdentitySetWithNil
	| set |
	self runSetWithNilTestOf: [KeyedIdentitySet keyBlock:[:o| o]].
	set := KeyedIdentitySet keyBlock:[:o| o].
	set add: nil.
	self assert: (set at: nil) == nil.
! !

!SetWithNilTest methodsFor: 'tests' stamp: 'ul 2/19/2010 15:59'!
testKeyedSetWithNil
	| set |
	self runSetWithNilTestOf: [KeyedSet keyBlock:[:o| o]].
	set := KeyedSet keyBlock:[:o| o].
	set add: nil.
	self assert: (set at: nil) == nil.
	self assert: (set includes: nil)
! !

!SetWithNilTest methodsFor: 'tests' stamp: 'ar 2/1/2010 21:24'!
testPluggableSetWithNil
	self runSetWithNilTestOf: [PluggableSet new]! !

!SetWithNilTest methodsFor: 'tests' stamp: 'ar 2/1/2010 21:24'!
testSetWithNil
	self runSetWithNilTestOf: [Set new]! !

!SetWithNilTest methodsFor: 'tests' stamp: 'ar 2/1/2010 21:40'!
testWeakSetWithNil
	| set |
	self runSetWithNilTestOf: [WeakSet new].
	"Ensure that GCed entries don't count"
	set := WeakSet with: Object new.
	Smalltalk garbageCollect.
	set do:[:x| self assert: false]. 	"test for de facto emptyness"
	self deny: (set includes: nil).
	set add: nil.
	self assert: (set includes: nil).
	set remove: nil.
	self deny: (set includes: nil).! !

TestCase subclass: #SharedQueue2Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

!SharedQueue2Test methodsFor: 'testing' stamp: 'ls 6/25/2005 13:49'!
testBasics
	| q |
	q := SharedQueue2 new.

	self should: [ q nextOrNil = nil ].

	q nextPut: 5.
	self should: [ q nextOrNil = 5 ].
	self should: [ q nextOrNil = nil ].

! !

!SharedQueue2Test methodsFor: 'testing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testContention1
	"here is a test case that breaks the standard SharedQueue from Squeak 3.8"

	| q r1 r2 |
	q := SharedQueue2 new.
	q nextPut: 5.
	q nextPut: 10.

	self should: [ q nextOrNil = 5 ].

	[ r1 := q next ] fork.
	[ r2 := q next ] fork.
	Processor  yield.   "let the above two threads block"
	
	q nextPut: 10.
	Processor yield.

	self should: [ r1 = 10 ].
	self should: [ r2 = 10 ].
	self should: [ q nextOrNil = nil ].
! !

!SharedQueue2Test methodsFor: 'testing' stamp: 'ls 6/25/2005 14:15'!
testNextOrNilSuchThat
	| q item |
	q := SharedQueue2 new.
	q nextPut: 5.
	q nextPut: 6.

	item := q nextOrNilSuchThat: [ :x | x even ].
	self should: [ item = 6 ].

	self should: [ q nextOrNil = 5 ].
	self should: [ q nextOrNil = nil ].
! !

TestCase subclass: #SortedCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:27'!
testAdd
	"self run: #testAdd"
	"self debug: #testAdd"
	
	| collection |
	collection := #(10 9 8 7 5 4 4 2) asSortedCollection.
	self assert: collection first = 2.
	self assert: collection last = 10.
	self assert: (collection size = 8).
	collection add:1.
	self assert: (collection size = 9).
	collection add: 6.
	self assert: ((collection at: 5) = 5).
	self assert: (collection size = 10).
	collection add: 3.
	! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:30'!
testAddAll
	"self run: #testAddAll"
	"self debug: #testAddAll"
	
	| sorted2 sorted|
	sorted2 := SortedCollection new.
	sorted2 add: 'brochet'; add:'truitelle'.
	sorted := SortedCollection new.
	sorted addAll: sorted2.
	self assert: (sorted hasEqualElements: sorted2).
	 ! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:32'!
testAddAll2
	"self run: #testAddAll2"
	"self debug: #testAddAll2"
	
	| sorted2 sorted|
	sorted2 := SortedCollection new.
	sorted2 add: 'brochet'; add:'truitelle'.
	sorted := SortedCollection new.
	sorted add: 'perche'.
	sorted addAll: sorted2.
	self assert: (sorted size = (sorted2 size + 1)).
	sorted2 do: 
			[ :each | self assert: (sorted includes: each)]
	 ! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:33'!
testCollect
	"self run: #testCollect"
	
	|result aSortedCollection|
	aSortedCollection := SortedCollection new.
	result := OrderedCollection new.
	result add:true ; add: true ; add: true ;add: false ; add: false.
	aSortedCollection := (1 to: 5) asSortedCollection.
	self assert: (result = (aSortedCollection collect: [:each | each < 4])).
	! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:39'!
testCopy
	"self run: #testCopy"
	"self debug: #testCopy"
	
	|aSortedCollection copySorted|
	aSortedCollection := SortedCollection new.
	aSortedCollection sortBlock: [:a :b | a < b].
	aSortedCollection add: 'truite' ; add: 'brochet'.
	self assert: aSortedCollection first = 'brochet'.
	
	copySorted := aSortedCollection copy.
	
	self assert: (copySorted  hasEqualElements: aSortedCollection).
	self assert: (copySorted  species = aSortedCollection species).
	self assert: (copySorted  sortBlock = aSortedCollection sortBlock).
	self assert: copySorted first = 'brochet'.! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:30'!
testCreation
	"self run: #testCreation"
	"self debug: #testCreation"
	
	| collection |
	collection := #(10 9 3 6 1 8 7 5 4 2) asSortedCollection.
	self assert: collection = (1 to: 10) asSortedCollection.
	! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:42'!
testEquals
	"self run: #testEquals"
	"self debug: #testEquals"
	
	|aSortedCollection|
	aSortedCollection := SortedCollection new.
	aSortedCollection add:'truite' ; add: 'brochet'.
	self assert: aSortedCollection copy = aSortedCollection.! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:43'!
testMedian
	"self run: #testMedian"
	"self debug: #testMedian"
	
	|aSortedCollection|
	aSortedCollection := (1 to: 10) asSortedCollection.
	self assert: aSortedCollection median=5.
	
	aSortedCollection := SortedCollection new.
	aSortedCollection add:'truite' ; add:'porcinet' ; add:'carpe'.
	self assert: (aSortedCollection median = 'porcinet').
	! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'nice 9/14/2009 21:02'!
testRemoveAll
	"Allows one to remove all elements of a collection" 
	
	| c1 c2 s2 |
	c1 := #(10 9 8 7 5 4 4 2) asSortedCollection: [:a :b | a >= b].
	c2 := c1 copy.
	s2 := c2 size.
	
	c1 removeAll.
	
	self assert: c1 size = 0.
	self assert: c2 size = s2 description: 'the copy has not been modified'.
	
	c1 add: 13; add: 14.
	self assert: (c1 first = 14 and: [c1 second = 13])  description: 'the sortBlock has been preserved'.! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'nice 2/26/2008 22:52'!
testReversed
	
	| sc1 sc2 sc3 |
	sc1 := #(1 2 3 4) asSortedCollection.
	self assert: sc1 reversed asArray = sc1 asArray reversed.
	
	self
		assert: sc1 reversed class = SortedCollection
		description: 'reversing a SortedCollection should answer a SortedCollection'.
	
	sc1 removeFirst; removeLast.
	sc2 := sc1 reversed.
	self assert: sc2 reversed asArray = sc1 asArray.
	
	sc2 add: 3/2; add: 1/2; add: 7/2.
	self assert: sc2 asArray = {7/2. 3. 2. 3/2. 1/2}.
	
	
	sc3 := #(1 2 3 3.0 4) asSortedCollection.
	self assert: sc3 reversed asArray = #(4 3.0 3 2 1).
	self assert: (sc3 reversed at: 2) class = Float.
	! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:46'!
testShouldPrecede
	"self run: #testShouldPrecede"
	
	|aSortedCollection|
	aSortedCollection := SortedCollection new.
	self assert: (aSortedCollection should: 'za' precede: 'zb').
	self assert: (aSortedCollection isEmpty).
	self assert: (aSortedCollection should: 1 precede: 2).
	
	aSortedCollection sortBlock: [:a :b | a > b].
	aSortedCollection reSort.
	self assert: (aSortedCollection should: 'zb' precede: 'za').
	self assert: (aSortedCollection isEmpty).
	self assert: (aSortedCollection should: 2 precede: 1).
		! !

!SortedCollectionTest methodsFor: 'tests - sorting' stamp: 'ul 12/23/2009 12:20'!
testSort

	| s |
	s := SortedCollection withAll: #(3 2 1).
	self assert: s asArray = #(1 2 3).
	s sort: #>=. "This should change the sortBlock too"
	self assert: s asArray = #(3 2 1).
	s add: 4.
	self assert: s asArray = #(4 3 2 1).
	s := SortedCollection new 
		sortBlock: #>=;
		addAll: #(1 2 3);
		yourself.
	self assert: s asArray = #(3 2 1).
	s sort.
	self assert: s asArray = #(1 2 3).
	s add: 0.
	self assert: s asArray = #(0 1 2 3)! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:38'!
testSortBlock
	"self run: #testSortBlock"
	"self debug: #testSortBlock"
	
	|aSortedCollection|
	aSortedCollection := SortedCollection new.
	aSortedCollection sortBlock: [:a :b | a < b].
	aSortedCollection add: 'truite' ; add: 'brochet' ; add: 'tortue'.
	self assert: aSortedCollection first = 'brochet'.
	
	aSortedCollection := SortedCollection new.
	aSortedCollection sortBlock: [:a :b | a >b].
	aSortedCollection add: 'truite' ; add: 'brochet' ; add: 'tortue'.
	self assert: aSortedCollection first = 'truite'.
	
	
	! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'nice 5/22/2008 09:59'!
testSpeciesLooseSortBlock
	"This is a non regression test for http://bugs.squeak.org/view.php?id=6535"

	| numbers reverseOrder firstThree |
	numbers := #(1 2 3 4 5).
	reverseOrder := SortedCollection sortBlock: [:x :y | x > y].
	reverseOrder addAll: numbers.

	"The elements are inverted"
	self assert: [reverseOrder asArray = #(5 4 3 2 1)].

	"Copy the first 3 elements"
	firstThree := reverseOrder copyFrom: 1 to: 3.

	"It appears to work"
	self assert: [firstThree asArray = #(5 4 3)].

	"but we have lost the sort block"
	firstThree add: 1.

	" firstThree is now #(1 5 4 3)!! "
	self assert: [firstThree asArray = #(5 4 3 1)] "fails"! !

TestCase subclass: #StackTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Stack'!

StackTest class
	instanceVariableNames: 'testSize'!

StackTest class
	instanceVariableNames: 'testSize'!

!StackTest methodsFor: 'test' stamp: 'dc 6/21/2006 10:55'!
testEmptyError

	| aStack |
	aStack := Stack new.
	self should: [ aStack top ] raise: Error.
	self should: [ aStack pop] raise: Error.
	
	aStack push: 'element'.
	
	self shouldnt: [ aStack top ] raise: Error.
	self shouldnt: [ aStack pop] raise: Error.
	
	
	"The stack is empty again due to previous pop"
	self should: [ aStack top ] raise: Error.
	self should: [ aStack pop] raise: Error.! !

!StackTest methodsFor: 'test' stamp: 'sd 3/21/2006 22:13'!
testPop

	| aStack res elem |
	elem := 'anElement'.	
	aStack := Stack new.
	self assert: aStack isEmpty.
	
	aStack push: 'a'.
	aStack push: elem.
	res := aStack pop.	
	self assert: res = elem.
	self assert: res == elem.
	
	self assert: aStack size = 1.
	aStack pop.
	self assert: aStack isEmpty.

! !

!StackTest methodsFor: 'test' stamp: 'sd 3/21/2006 22:13'!
testPush
	
	| aStack |
	aStack := Stack new.
	aStack push: 'a'.
	self assert: aStack size = 1.	
	aStack push: 'b'.
	self assert: aStack size = 2.
	! !

!StackTest methodsFor: 'test' stamp: 'sd 3/21/2006 22:13'!
testSize
	
	| aStack |
	aStack := Stack new.
	self assert: aStack size = 0.
	aStack push: 'a'.
	self assert: aStack size = 1.
	aStack push: 'b'.
	self assert: aStack size = 2.
	aStack pop.
	self assert: aStack size = 1.
	aStack pop.
	self assert: aStack size = 0.

 
	


! !

!StackTest methodsFor: 'test' stamp: 'sd 3/21/2006 22:13'!
testTop

	| aStack |
	aStack := Stack new.
	self assert: aStack isEmpty.
	aStack push: 'a'.
	aStack push: 'b'.
	self assert: aStack top = 'b'.
	self assert: aStack top = 'b'.
	self assert: aStack size = 2.! !

TestCase subclass: #TextAndTextStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextAndTextStreamTest commentStamp: '<historical>' prior: 0!
At May 09, 2003 Tim Olson sent a bug report to the Squeak developers list that inspired me to examine the protocol of TextStream in greater detail.  (The bug that Tim reported was present in Squeak 3.4, it is shown in testExample1.) In a discussion that followed,  Daniel Vainsencher proposed that we should have tests for Text and TextStreams. This class is an attempt to implement that proposal. For Squeak 3.4, some of the test examples fail.!

!TextAndTextStreamTest methodsFor: 'examples' stamp: 'BG 6/10/2003 20:17'!
example1: size

   | ts text |

  ts := TextStream on: (Text new: size).
  ts  nextPutAll: 'xxxxx' asText.
  ts nextPutAll: ('yyyyy' asText allBold, 'zzzzzzz' asText).
  text := ts contents.
  ^text
  ! !

!TextAndTextStreamTest methodsFor: 'examples' stamp: 'BG 6/10/2003 20:26'!
example2

      | ts text |

  ts := TextStream on: (Text new: 50).
  ts  nextPutAll: 'abc' asText.
  ts nextPutAll: 'def' asText allBold.
  ts nextPutAll: 'ghijk' asText.
  text := ts contents.
  ^text
  ! !

!TextAndTextStreamTest methodsFor: 'examples' stamp: 'BG 6/11/2003 13:09'!
replacementAtStartExample3

   | text1  replacement  length  |

   text1 := 'This is a simple text' copy asText.
    " without the copy, we would modify a constant that the compiler attached at the compiled method. "
   length  := 'This' size.
   replacement := 'Tht' asText.
   text1 replaceFrom: 1 
        to:   length
        with: replacement
        startingAt: 1.
! !

!TextAndTextStreamTest methodsFor: 'examples' stamp: 'BG 6/11/2003 13:05'!
replacementExample3

  " for a Text  t,
     the following assertion should always hold:
     t string size = t run size 
    This test examines the preservation of this assertion for in-place replacement 
 Here, the replacement text is shorteer than the text that is shall replace. "


   | text1 string replacement startPos length startPosInRep string2 |

   text1 := (string := 'This is again simple text' copy) asText.
     " without the copy, we would modify a constant that the compiler attached at the compiled method. "
   startPos := string findString: 'simple'. 
   length  := 'simple' size.
   replacement := (string2 := 'both simple and short') asText.
   startPosInRep :=  string2 findString: 'short'.
   text1 replaceFrom: startPos 
        to: startPos + length - 1
        with: replacement
        startingAt: startPosInRep.
   
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:45'!
testAddStringToTextStream

	"It is possible to add a string into a TextStream.
	This test verifies that the created text has text attributes for all its characters. "

	| ts text |
	ts := TextStream on: (Text new: 50).
	ts nextPutAll: 'abc' asText.
	ts nextPutAll: 'def' asText allBold.
	ts nextPutAll: 'ghijk'.
	text := ts contents.
     " now, check the fundamental invariant of a text: "
	self assert: text string size = text runs size.
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:40'!
testExampleText1
	"self run: #testExampleText1"
	"inspired by a bug report from Tim Olson.
	Text attributes are lost when the stream collection is expanded.
	Documented BUG!!!!!!"

    | text1 text2 atts1 atts2 |
	text1 := self example1: 10. " here we will loose the attribute bold "
	text2 := self example1: 50. " here we have a larger buffer and will not loose text attributes "
	atts1 := text1 runs copyFrom: 6 to: 10. 
	atts2 := text2 runs copyFrom: 6 to: 10. 

	self assert: atts1 = atts2.
      ! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:40'!
testExampleText2
	"a Text looses its attributes when it is reversed "
	
	| text1 text2 |
	text1 := self example2.
	text2 := text1 reversed reversed.
	self assert: text1 runs = text2 runs.
 
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:35'!
testRunArrayAdjacentMerge

	"this demonstrates that adjancent runs with equal attributes are merged. "
	| runArray |
	runArray := RunArray new.
	runArray 
		addLast: TextEmphasis normal times: 5;
		addLast: TextEmphasis bold times: 5;
		addLast: TextEmphasis bold times: 5.
	self assert: (runArray runs size = 2). ! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:37'!
testRunArrayInvariant

	"this verifies that the fundamental invariant of a RunArray is always satisfied. "
	"see comment below"
	| runArray |
	runArray := RunArray new.
	runArray 
		addLast: TextEmphasis normal times: 5;
		addLast: TextEmphasis bold times: 5;
		addLast: TextEmphasis normal times: 5.
	self assert:
       ((1 to: runArray size) allSatisfy:
           [:idx |  | lastIndex lastOffset lastRun lengthOfPreviousRuns |
               runArray at: idx.  " updates the cached values "
               lastIndex := runArray instVarNamed: 'lastIndex'.
               lastRun := runArray instVarNamed: 'lastRun'.
               lastOffset := runArray instVarNamed: 'lastOffset'.
               lengthOfPreviousRuns 
                   := (1 to: lastRun - 1)
                      inject: 0
                       into: [:sum :idx2 | sum + (runArray runs at: idx2)].
               lastIndex = (lastOffset + lengthOfPreviousRuns + 1) 
           ]
       ). 

" This method is a bit tricky. First, it uses Object>>instVarNamed: to access instance variables for which no accessors are defined. The same method is used by the debuggers and by various inspectors.
The assertion itself explains the meaning of the cached values."! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:36'!
testRunArrayReversal

  	"this tests the reversal of a  RunArray "
	| runArray |
	runArray := RunArray new.
	runArray 
		addLast: TextEmphasis normal times: 5;
		addLast: TextEmphasis bold times: 5;
		addLast: TextEmphasis normal times: 5.
	self assert: (runArray reversed runs size = 3). ! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:32'!
testRunArrayRunsAreNotMerged

	" this demonstrates that different runs are not merged "
	| runArray |
	runArray := RunArray new.
	runArray 
		addLast: TextEmphasis normal times: 5;
		addLast: TextEmphasis bold times: 5;
		addLast: TextEmphasis normal times: 5.
	self assert: (runArray runs size = 3). ! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:33'!
testRunArrayRunsSize

 	"this demonstrates that the size of a run array is the sum of the sizes of its runs. "
	| runArray |
	runArray := RunArray new.
  	runArray 
		addLast: TextEmphasis normal times: 5;
		addLast: TextEmphasis bold times: 5;
		addLast: TextEmphasis normal times: 5.
	self assert: (runArray size = 15). ! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:49'!
testTextEmphasisRangeDetection1

	"this tests the detection of the range of a text attribute. "
	| text startPos boldStyle |
	text := 'This is a text with attriute bold for some characters' asText.
	startPos := text findString: 'bold' startingAt: 1.
	text addAttribute: TextEmphasis bold from: startPos to: startPos + 3.
	boldStyle := TextEmphasis bold.

  " uncomment the following statement for examine failures: "
  " -----------------
       (1 to: text size) do:
           [:idx | | range |
              range := text rangeOf: boldStyle startingAt: idx.
             Transcript show: startPos; show: ' -- '; show: idx printString; show: '  '; show: range printString; show: range size printString; show: ((idx between: startPos and: startPos + 3)
                  ifTrue:
                    [range first = startPos & (range size = 4)]
                  ifFalse:
                    [range first = idx & (range size = 0)]) printString; cr.
           ].
    ------------- "

	self assert: 
		((1 to: text size) allSatisfy:
			[:idx | | range |
				range := text rangeOf: boldStyle startingAt: idx.
				(idx between: startPos and: startPos + 3)
					ifTrue: [range first = startPos & (range size = 4)]
					ifFalse: [range first = idx & (range size = 0)]])! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:51'!
testTextEmphasisRangeDetection2

	"this tests the detection of the range of a text attribute.
	Here the searched attribute spans three runs. The objective of the test is whether the entire range is always found."
	| text startPos searchedStyle |
	text := 'This is a text with attriute bold for some characters' asText.
	startPos := text findString: 'bold' startingAt: 1.
	text addAttribute: TextEmphasis bold from: startPos to: startPos + 3.
	text addAttribute: TextEmphasis italic from: startPos - 2 to: startPos + 5.
	searchedStyle := TextEmphasis italic.

  " uncomment the following statement for examine failures: " 
  " -----------------------
       (1 to: text size) do:
           [:idx | | range | 
              range := text rangeOf: searchedStyle startingAt: idx.
             Transcript show: startPos; show: ' -- '; show: idx printString; show: '  '; show: range printString; show: range size printString; show: ((idx between: startPos - 2 and: startPos -2 + 7)
                  ifTrue:
                    [range first = (startPos - 2) & (range size = 8)]
                  ifFalse:
                    [range first = idx & (range size = 0)]) printString; cr.
           ].
   ----------------------- "
	self assert: 
			((1 to: text size) allSatisfy:
				[:idx | | range |
					range := text rangeOf: searchedStyle startingAt: idx.
					(idx between: startPos - 2 and: startPos -2 + 7)
						ifTrue: [range first = (startPos - 2) & (range size = 8)]
						ifFalse: [range first = idx & (range size = 0)]])! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:53'!
testTextEmphasisRangeDetection3

	"this tests the detection of the range of a text attribute.
    Here the searched attribute spans three runs. The the range to be detected begins at text position 1. The objective of the test is whether the entire range is always found."

	| text startPos searchedStyle |
	text := 'This is a text with attriute bold for some characters' asText.
	startPos := text findString: 'bold' startingAt: 1.
	text addAttribute: TextEmphasis bold from: startPos to: startPos + 3.
	text addAttribute: TextEmphasis italic from: 1 to: startPos + 5.
	searchedStyle := TextEmphasis italic.

	" uncomment the following statement to examine failures: "
 	" -----------------------
       (1 to: text size) do:
           [:idx | | range | 
              range := text rangeOf: searchedStyle startingAt: idx.
             Transcript show: startPos;
					show: ' -- ';
					show: idx printString;
					show: '  ';
					show: range printString;
					show: range size printString;
                        show: ' ';
					 show: ((idx between: 1 and: startPos + 5)
                  					ifTrue:
                  					  [range first = 1 & (range size = (startPos + 5))]
                					ifFalse:
                   					 [range first = idx & (range size = 0)]) printString; cr.
           ].
   ----------------------- "
	self assert: 
       ((1 to: text size) allSatisfy:
           [:idx | | range |
              range := text rangeOf: searchedStyle startingAt: idx.
              (idx between: 1 and: startPos + 5)
                  ifTrue:
                    [range first = 1 & (range size = (startPos + 5))]
                  ifFalse:
                    [range first = idx & (range size = 0)]])! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:55'!
testTextEmphasisRangeDetection4

	"this tests the detection of the range of a text attribute.
	Here the searched attribute spans three runs. The the range to be detected extends to the end of the text . The objective of the test is whether the 
	entire range is always found."

	| text startPos searchedStyle |
	text := 'This is a text with attriute bold for some characters' asText.
	startPos := text findString: 'bold' startingAt: 1.
	text addAttribute: TextEmphasis bold from: startPos to: startPos + 3.	
	text addAttribute: TextEmphasis italic from: startPos - 2 to: text size.
	searchedStyle := TextEmphasis italic.

	" uncomment the following statement to examine failures: "
 
 	" -----------------------------------------
       (1 to: text size) do:
           [:idx | | range | 
              range := text rangeOf: searchedStyle startingAt: idx.
             Transcript show: startPos;
					show: ' -- ';
					show: idx printString;
					show: '  ';
					show: range printString;
					show: range size printString;
                        show: ' ';
					 show: ((idx between: startPos - 2 and: text size)
                  			ifTrue:
   			                 [range first = (startPos - 2) & (range size = (text size - (startPos - 2) + 1))]
                  			ifFalse:
 			                 [range first = idx & (range size = 0)]) printString;
					cr.
           ].
   -------------------------------"

	self assert: 
       ((1 to: text size) allSatisfy:
           [:idx | | range |
              range := text rangeOf: searchedStyle startingAt: idx.
              (idx between: startPos - 2 and: text size)
                  ifTrue: [range first = (startPos - 2) & (range size = (text size - (startPos - 2) + 1))]
                  ifFalse: [range first = idx & (range size = 0)]])! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:56'!
testTextReplacement1

	"for a Text  t,
     the following assertion should always hold:
     t string size = t run size 
	This test examines the preservation of this assertion for in-place replacement "

   | text1 string replacement startPos length startPosInRep string2 |
   text1 := (string := 'This is a simple text' copy) asText.
   "without the copy, we would modify a constant that the compiler attached at the compiled method. "
   startPos := string findString: 'simple'. 
   length  := 'simple' size.
   replacement := (string2 := 'both simple and short*') asText.
   startPosInRep :=  string2 findString: 'short'.
   text1 replaceFrom: startPos 
        to: startPos + length - 1
        with: replacement
        startingAt: startPosInRep.
   self assert: text1 string size = text1 runs size.
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:57'!
testTextReplacement2


  	"for a Text  t,
     the following assertion should always hold:
     t string size = t run size 
    	This test examines the preservation of this assertion for in-place replacement.
    	Here, the replacement text has trailing characters. "

	| text1 string replacement startPos length startPosInRep string2 |
	text1 := (string := 'This is simple text' copy) asText.
	"without the copy, we would modify a constant that the compiler attached at the compiled method. "
	startPos := string findString: 'simple'. 
	length := 'simple' size.
	replacement := (string2 := 'both simple and short*************') asText.
	startPosInRep :=  string2 findString: 'short'.
	text1 replaceFrom: startPos 
        to: startPos + length - 1
        with: replacement
        startingAt: startPosInRep.
	self assert: text1 string size = text1 runs size.
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:58'!
testTextReplacement3

	"for a Text  t,
	the following assertion should always hold:
	t string size = t run size 
	This test examines the preservation of this assertion for in-place replacement 
	Here, the replacement text is shorteer than the text that is shall replace. "

	self should: [self replacementExample3]  raise: Error! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:59'!
testTextReplacementAtStartPos1

	"for a Text  t,
     the following assertion should always hold:
     t string size = t run size 
	This test examines the preservation of this assertion for in-place replacement "

	| text1  replacement  length  |
	text1 := 'This is a simple text' copy asText.
	"without the copy, we would modify a constant that the compiler attached at the compiled method. "
	length  := 'This' size.
	replacement := 'That' asText.
	text1 replaceFrom: 1 
        to:   length
        with: replacement
        startingAt: 1.
	self assert: text1 string size = text1 runs size.
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:00'!
testTextReplacementAtStartPos2

  	"for a Text  t,
     the following assertion should always hold:
     t string size = t run size 
	This test examines the preservation of this assertion for in-place replacement "

	| text1  replacement  length  |
	text1 := 'This is a simple text' copy asText.
	"without the copy, we would modify a constant that the compiler attached at the compiled method. "
	length  := 'This' size.
	replacement := 'That********' asText.
	text1 replaceFrom: 1 
        to:   length
        with: replacement
        startingAt: 1.
	self assert: text1 string size = text1 runs size.
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:00'!
testTextReplacementAtStartPos3

	"for a Text  t,
	the following assertion should always hold:
	t string size = t run size 
	This test examines the preservation of this assertion for in-place replacement 
	Here, the replacement text is shorteer than the text that is shall replace. "

	self should: [self replacementAtStartExample3] raise: Error! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:44'!
testTextStreamAdjacentRunsWithIdentitcalAttributes

	"This test verifies that adjacent runs with identical attributes are coalesced."

	| ts text rangeOfBold |
	ts := TextStream on: (Text new: 50).
	ts nextPutAll: 'abc' asText.
	ts nextPutAll: 'def' asText allBold.
	ts nextPutAll: 'ghijk'.
	text := ts contents.
	rangeOfBold := text find:  TextEmphasis bold.
	text removeAttribute: TextEmphasis bold from: rangeOfBold first to: rangeOfBold last. 
     "now, check that only one run is left and that it has the correct size "
	self assert: text runs runs size = 1 & (text runs size = text string size).
! !

TestCase subclass: #TextLineEndingsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextLineEndingsTest commentStamp: 'nk 11/1/2003 07:55' prior: 0!
This is a test case for Text>>withSqueakLineEndings and String>>withSqueakLineEndings.

The main problem we've seen with the Text version is that it doesn't preserve formatting correctly.!

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:06'!
testDecoratedTextConversionCrLF

	| text newText |
	text := ('123456', String crlf, '901234') asText.
	text addAttribute: TextColor blue from: 4 to: 10.
	text addAttribute: TextColor red from: 6 to: 9.
	text addAttribute: TextEmphasis bold.
	newText := text withSqueakLineEndings.
	self assert: ((text size - 1) = newText size).
	self assert: (newText size = newText runs size).
	self assert: (newText attributesAt: 6) = (text attributesAt: 6).
	self assert: (newText attributesAt: 8) = (text attributesAt: 9).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:06'!
testDecoratedTextConversionJustLF

	| text newText |
	text := ('123456', String lf, '901234') asText.
	text addAttribute: TextColor blue from: 4 to: 10.
	text addAttribute: TextColor red from: 6 to: 9.
	text addAttribute: TextEmphasis bold.
	newText := text withSqueakLineEndings.
	self assert: ((text size) = newText size).
	self assert: (newText size = newText runs size).
	self assert: (newText attributesAt: 6) = (text attributesAt: 6).
	self assert: (newText attributesAt: 8) = (text attributesAt: 8).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testDecoratedTextConversionNoLF

	| text newText |
	text := ('123456', String cr, '901234') asText.
	text addAttribute: TextColor blue from: 4 to: 10.
	text addAttribute: TextColor red from: 6 to: 9.
	text addAttribute: TextEmphasis bold.
	newText := text withSqueakLineEndings.
	self assert: ((text size) = newText size).
	self assert: (newText size = newText runs size).
	self assert: (newText attributesAt: 6) = (text attributesAt: 6).
	self assert: (newText attributesAt: 8) = (text attributesAt: 8).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testSimpleTextConversionCrLF

	| string newText |
	string := 'This is a test', String crlf, 'of the conversion'.
	newText := string asText withSqueakLineEndings.
	self assert: ((string size - 1) = newText size).
	self assert: (newText size = newText runs size).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testSimpleTextConversionJustCR

	| string newText |
	string := 'This is a test', String cr, 'of the conversion'.
	newText := string asText withSqueakLineEndings.
	self assert: ((string size) = newText size).
	self assert: (newText size = newText runs size).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testSimpleTextConversionJustLF

	| string newText |
	string := 'This is a test', String lf, 'of the conversion'.
	newText := string asText withSqueakLineEndings.
	self assert: ((string size) = newText size).
	self assert: (newText size = newText runs size).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testStringConversionCrLF

	| string newString |
	string := 'This is a test', String crlf, 'of the conversion'.
	newString := string withSqueakLineEndings.
	self assert: ((string size - 1) = newString size).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testStringConversionJustLF

	| string newString |
	string := 'This is a test', String lf, 'of the conversion'.
	newString := string withSqueakLineEndings.
	self assert: (string size = newString size).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testStringConversionNoLF

	| string newString |
	string := 'This is a test', String cr, 'of the conversion'.
	newString := string withSqueakLineEndings.
	self assert: (string = newString).! !

TestCase subclass: #WeakFinalizersTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Weak'!

!WeakFinalizersTest methodsFor: 'as yet unclassified' stamp: 'ul 11/2/2010 18:29'!
expectedFailures

	^WeakFinalizationList hasNewFinalization
		ifTrue: [ #() ]
		ifFalse: [ #(testNewFinalizationSupported) ]! !

!WeakFinalizersTest methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 3/8/2010 22:15'!
testNewFinalizationSupported
	| list item |

	list := WeakFinalizationList new.
	
	
	item := WeakFinalizerItem new list: list object: Object new executor: nil.
	
	self assert: (list first isNil).
	self assert: (item object notNil).
	
	Smalltalk garbageCollect.
	
	self assert: (item object isNil).
	self assert: (list first == item).
	self assert: (list first next == nil).! !

TestCase subclass: #WideCharacterSetTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Support'!

!WideCharacterSetTest commentStamp: 'nice 11/19/2007 22:45' prior: 0!
WideCharacterSetTest holds tests for WideCharacterSet!

!WideCharacterSetTest methodsFor: 'testing' stamp: 'nice 11/19/2007 23:36'!
testAddingToCharacterSet

	| cs wcs wc |
	cs := CharacterSet newFrom: 'aeiouy'.
	wcs := cs copy.
	wc := 4452 asCharacter.
	
	self shouldnt: [wcs add: wc] raise: Error description: 'adding a WideCharacter to an ordinary CharacterSet should turn it into a WideCharacterSet'.

	self should: [wcs size = (cs size + 1)] description: 'We just added a Character, size should be increased by one'.
	self shouldnt: [wcs = cs] description: 'We just added a Character, sets should not be equal'.
	self shouldnt: [cs = wcs] description: 'We just added a Character, sets should not be equal'.
	self should: [cs allSatisfy: [:char | wcs includes: char]] description: 'Each character of the original CharacterSet should be included in the WideCharacterSet'.
	self should: [wcs hasWideCharacters] description: 'We just added a WideCharacter, so this WideCharacterSet definitely has one'.
	self should: [wcs includes: wc] description: 'We just added this WideCharacter, so this WideCharacterSet should include it'.
	
	wcs add: wc.
	self should: [wcs size = (cs size + 1)] description: 'We just added a Character already included in the set, size should be unchanged'.
	
	wcs remove: wc.
	self should: [wcs size = cs size] description: 'We added then removed a Character, now size should be equal to original'.
	self shouldnt: [wcs hasWideCharacters] description: 'We just removed the only WideCharacter, so this WideCharacterSet definitely has no WideCharacter'.
	
	self should: [wcs = cs] description: 'A WideCharacterSet can be equal to an Ordinary CharacterSet'.
	self should: [cs = wcs] description: 'An ordinary CharacterSet can be equal to a WideCharacterSet'.
	self should: [cs hash = wcs hash] description: 'If some objects are equal, then they should have same hash code'.
	
	! !

!WideCharacterSetTest methodsFor: 'testing' stamp: 'nice 11/19/2007 23:34'!
testCreation
	"By now, only creation method is newFrom:"

	| cs1 wcs1 cs2 wcs2 byteString wideString |
	byteString := 'aeiouy'.
	wideString := 'aeiouy' copyWith: 340 asCharacter.

	cs1 := CharacterSet newFrom: byteString.
	wcs1 := WideCharacterSet newFrom: byteString.
	self should: [wcs1 = cs1].
	self should: [wcs1 size = byteString "asSet" size].
	
	cs2 := CharacterSet newFrom: wideString.
	wcs2 := WideCharacterSet newFrom: wideString.
	self should: [wcs2 = cs2].
	self should: [wcs2 size = wideString "asSet" size].
	
	self should: [(byteString indexOfAnyOf: wcs1) = 1] description: 'This should used optimized byteArrayMap method'.
	self should: [(byteString indexOfAnyOf: wcs2) = 1] description: 'This should used optimized byteArrayMap method'.
	
	self should: [('bcd' indexOfAnyOf: wcs1) = 0] description: 'This should used optimized byteArrayMap method'.
	self should: [('bcd' indexOfAnyOf: wcs2) = 0] description: 'This should used optimized byteArrayMap method'.! !
